@@ -1,7 +1,7 @@
 /*
  * Created by
- * $Id: make_ci.pike,v 1.7 2001/06/21 13:23:20 grubba Exp $
- * on Mon Nov 21 21:27:09 2005
+ * $Id: make_ci.pike,v 1.11 2004/04/12 01:11:15 nilsson Exp $
+ * on Wed May 26 21:25:54 2004
  *
  * Table used for looking up the case of
  * Unicode characters.
@@ -10,10 +10,6 @@
  */
 
 { 0x000000, CIM_NONE, 0x0000, },
-{ 0x000041, CIM_CASEBIT, 0x0020, },
-{ 0x00005b, CIM_NONE, 0x0000, },
-{ 0x000061, CIM_CASEBIT, 0x0020, },
-{ 0x00007b, CIM_NONE, 0x0000, },
 { 0x0000b5, CIM_LOWERDELTA, -0x02e7, },
 { 0x0000b6, CIM_NONE, 0x0000, },
 { 0x0000c0, CIM_CASEBIT, 0x0020, },
@@ -158,11 +154,15 @@
 { 0x0003d8, CIM_CASEBIT, 0x0001, },
 { 0x0003f0, CIM_LOWERDELTA, 0x0056, },
 { 0x0003f1, CIM_LOWERDELTA, 0x0050, },
-{ 0x0003f2, CIM_LOWERDELTA, 0x004f, },
+{ 0x0003f2, CIM_LOWERDELTA, -0x0007, },
 { 0x0003f3, CIM_NONE, 0x0000, },
 { 0x0003f4, CIM_UPPERDELTA, -0x003c, },
 { 0x0003f5, CIM_LOWERDELTA, 0x0060, },
 { 0x0003f6, CIM_NONE, 0x0000, },
+{ 0x0003f7, CIM_CASEBITOFF, 0x0001, },
+{ 0x0003f9, CIM_UPPERDELTA, -0x0007, },
+{ 0x0003fa, CIM_CASEBIT, 0x0001, },
+{ 0x0003fc, CIM_NONE, 0x0000, },
 { 0x000400, CIM_UPPERDELTA, 0x0050, },
 { 0x000410, CIM_CASEBIT, 0x0020, },
 { 0x000420, CIM_CASEBITOFF, 0x0020, },
@@ -284,7 +284,8 @@
 { 0x00ff41, CIM_CASEBITOFF, 0x0020, },
 { 0x00ff5b, CIM_NONE, 0x0000, },
 { 0x010400, CIM_UPPERDELTA, 0x0028, },
-{ 0x010426, CIM_NONE, 0x0000, },
 { 0x010428, CIM_LOWERDELTA, 0x0028, },
-{ 0x01044e, CIM_NONE, 0x0000, },
-#define CASE_INFO_SHIFT0_HIGH 0x000f
+{ 0x010450, CIM_NONE, 0x0000, },
+
+
+#define CASE_INFO_SHIFT0_HIGH 0x000b
