@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: cpp.c,v 1.113 2004/11/01 03:05:23 mast Exp $
+|| $Id: cpp.c,v 1.135 2004/04/15 22:07:48 mast Exp $
 */
 
 #include "global.h"
@@ -19,11 +19,12 @@
 #include "mapping.h"
 #include "builtin_functions.h"
 #include "operators.h"
-#include "opcodes.h"
 #include "constants.h"
 #include "time.h"
 #include "stuff.h"
 #include "version.h"
+#include "pike_types.h"
+#include "cpp.h"
 
 #include <ctype.h>
 
@@ -114,7 +115,7 @@ struct cpp
 struct define *defined_macro =0;
 struct define *constant_macro =0;
 
-void cpp_error(struct cpp *this,char *err)
+void cpp_error(struct cpp *this, const char *err)
 {
   this->compile_errors++;
   if(this->compile_errors > 10) return;
@@ -135,32 +136,24 @@ void cpp_error(struct cpp *this,char *err)
   }
 }
 
-void cpp_error_vsprintf(struct cpp *this, const char *fmt, va_list args)
+void cpp_error_vsprintf (struct cpp *this, const char *fmt, va_list args)
 {
   char buf[8192];
-
-#ifdef HAVE_VSNPRINTF
-  vsnprintf(buf, 8190, fmt, args);
-#else /* !HAVE_VSNPRINTF */
-  VSPRINTF(buf, fmt, args);
-#endif /* HAVE_VSNPRINTF */
-  if((size_t)strlen(buf) >= (size_t)sizeof(buf))
-    Pike_fatal("Buffer overflow in cpp_error.\n");
-
+  VSNPRINTF (buf, sizeof (buf), fmt, args);
   cpp_error(this, buf);
 }
 
 void cpp_error_sprintf(struct cpp *this, const char *fmt, ...)
-     ATTRIBUTE((format(printf,2,3)))
+  ATTRIBUTE((format(printf,2,3)))
 {
   va_list args;
-  va_start(args, fmt);
-  cpp_error_vsprintf(this, fmt, args);
+  va_start(args,fmt);
+  cpp_error_vsprintf (this, fmt, args);
   va_end(args);
 }
 
 void cpp_handle_exception(struct cpp *this, const char *cpp_error_fmt, ...)
-     ATTRIBUTE((format(printf,2,3)))
+  ATTRIBUTE((format(printf,2,3)))
 {
   struct svalue thrown;
   move_svalue (&thrown, &throw_value);
@@ -189,34 +182,183 @@ void cpp_handle_exception(struct cpp *this, const char *cpp_error_fmt, ...)
   free_svalue(&thrown);
 }
 
-void cpp_describe_exception(struct cpp *this, struct svalue *thrown)
+static void cpp_warning(struct cpp *this, const char *err)
 {
-  /* FIXME: Doesn't handle wide string error messages. */
-  struct pike_string *s = 0;
-
-  if ((thrown->type == T_ARRAY) && thrown->u.array->size &&
-      (thrown->u.array->item[0].type == T_STRING)) {
-    /* Old-style backtrace */
-    s = thrown->u.array->item[0].u.string;
-  } else if (thrown->type == T_OBJECT) {
-    struct generic_error_struct *ge;
-    if ((ge = (struct generic_error_struct *)
-	 get_storage(thrown->u.object, generic_error_program))) {
-      s = ge->desc;
-    }
-  }
-
-  if (s && !s->size_shift) {
-    ref_push_string(s);
-    f_string_trim_all_whites(1);
-    push_constant_text("\n");
-    push_constant_text(" ");
-    f_replace(3);
-    cpp_error(this, sp[-1].u.string->str);
+  if((this->handler && this->handler->prog) || get_master())
+  {
+    ref_push_string(this->current_file);
+    push_int(this->current_line);
+    push_text(err);
+    low_safe_apply_handler("compile_warning", this->handler,
+			   this->compat_handler, 3);
     pop_stack();
+  }else{
+    (void)fprintf(stderr, "%s:%ld: %s\n",
+		  this->current_file->str,
+		  (long)this->current_line,
+		  err);
+    fflush(stderr);
   }
 }
 
+/*! @class MasterObject
+ */
+
+/*! @decl inherit CompilationHandler
+ *!
+ *! The master object acts as fallback compilation handler for
+ *! @[compile()] and @[cpp()].
+ */
+
+/*! @decl CompilationHandler get_compilation_handler(int major, int minor)
+ *!
+ *!   Get compilation handler for simulation of Pike v@[major].@[minor].
+ *!
+ *!   This function is called by @[cpp()] when it encounters
+ *!   @expr{#pike@} directives.
+ *!
+ *! @param major
+ *!   Major version.
+ *!
+ *! @param minor
+ *!   Minor version.
+ *!
+ *! @returns
+ *!   Returns a compilation handler for Pike >= @[major].@[minor].
+ */
+
+/*! @decl string decode_charset(string raw, string charset)
+ *!
+ *!   Convert @[raw] from encoding @[charset] to UNICODE.
+ *!
+ *!   This function is called by @[cpp()] when it encounters
+ *!   @expr{#charset@} directives.
+ *!
+ *! @param raw
+ *!   String to convert.
+ *!
+ *! @param charset
+ *!   Name of encoding that @[raw] uses.
+ *!
+ *! @returns
+ *!   @[raw] decoded to UNICODE, or @expr{0@} (zero) if the decoding failed.
+ *!
+ *! @seealso
+ *!   @[Locale.Charset]
+ */
+
+/*! @endclass
+ */
+
+/*! @class CompilationHandler
+ */
+
+/*! @decl void compile_error(string filename, int line, string msg)
+ *!
+ *!   Called by @[compile()] and @[cpp()] when they encounter
+ *!   errors in the code they compile.
+ *!
+ *! @param filename
+ *!   File where the error was detected.
+ *!
+ *! @param line
+ *!   Line where the error was detected.
+ *!
+ *! @param msg
+ *!   Description of error.
+ *!
+ *! @seealso
+ *!   @[compile_warning()].
+ */
+
+/*! @decl void compile_exception(mixed exception)
+ *!
+ *!   Called by @[compile()] and @[cpp()] if they trigger
+ *!   exceptions.
+ */
+
+/*! @decl mapping(string:mixed) get_predefines()
+ *!
+ *!   Called by @[cpp()] to get the set of global symbols.
+ *!
+ *! @returns
+ *!   Returns a mapping from symbol name to symbol value.
+ *!   Returns zero on failure.
+ *!
+ *! @seealso
+ *!   @[resolv()], @[get_default_module()]
+ */
+
+/*! @decl mixed resolv(string symbol, string filename, @
+ *!                    CompilationHandler handler)
+ *!
+ *!   Called by @[compile()] and @[cpp()] to resolv
+ *!   module references.
+ *!
+ *! @returns
+ *!   Returns the resolved value, or @[UNDEFINED] on failure.
+ *!
+ *! @seealso
+ *!   @[get_predefines()]
+ */
+
+/*! @decl mixed handle_import(string path, string filename, @
+ *!                           CompilationHandler handler)
+ *!
+ *!   Called by @[compile()] and @[cpp()] to handle import
+ *!   directives specifying specific paths.
+ *!
+ *! @returns
+ *!   Returns the resolved value, or @[UNDEFINED] on failure.
+ */
+
+/*! @decl string handle_include(string header_file, string current_file, @
+ *!                             int(0..1) is_local_ref)
+ *!
+ *!   Called by @[cpp()] to resolv @expr{#include@} and @expr{#string@}
+ *!   directives.
+ *!
+ *! @param header_file
+ *!   File that was requested for inclusion.
+ *!
+ *! @param current_file
+ *!   File where the directive was found.
+ *!
+ *! @param is_local_ref
+ *!   Specifies reference method.
+ *!   @int
+ *!     @value 0
+ *!       Directive was @expr{#include <header_file>@}.
+ *!     @value 1
+ *!       Directive was @expr{#include "header_file"@}.
+ *!   @endint
+ *!
+ *! @returns
+ *!   Returns the filename to pass to @[read_include()] if found,
+ *!   and @expr{0@} (zero) on failure.
+ *!
+ *! @seealso
+ *!   @[read_include()]
+ */
+
+/*! @decl string read_include(string filename)
+ *!
+ *!   Called by @[cpp()] to read included files.
+ *!
+ *! @param filename
+ *!   Filename as returned by @[handle_include()].
+ *!
+ *! @returns
+ *!   Returns a string with the content of the header file on success,
+ *!   and @expr{0@} (zero) on failure.
+ *!
+ *! @seealso
+ *!   @[handle_include()]
+ */
+
+/*! @endclass
+ */
+
 void cpp_change_compat(struct cpp *this, int major, int minor)
 {
   if(this->compat_major == major &&
@@ -256,8 +398,8 @@ static struct mapping *initial_predefs_mapping()
   if (!use_initial_predefs) Pike_fatal ("Initial predefs has been taken over.\n");
 #endif
   for (def = first_predef; def; def = def->next) {
-    push_string (make_shared_string (def->name));
-    push_string (make_shared_string (def->value));
+    push_text (def->name);
+    push_text (def->value);
     mapping_insert (map, sp - 2, sp - 1);
     pop_n_elems (2);
   }
@@ -577,13 +719,11 @@ while(1)					\
     if(data[pos+1]=='\n')			\
     {						\
       pos++;					\
-      this->current_line++;			\
       continue;					\
     }						\
     if(data[pos+1]=='\r' && data[pos+2]=='\n')	\
     {						\
       pos+=2;					\
-      this->current_line++;			\
       continue;					\
     }						\
     READCHAR(tmp);				\
@@ -697,7 +837,7 @@ while(1)					\
 
 #define FINDTOK() 				\
   do {						\
-  SKIPWHITE();					\
+  SKIPSPACE();					\
   if(data[pos]=='/')				\
   {						\
     ptrdiff_t tmp;				\
@@ -730,7 +870,7 @@ while(1)					\
   break;					\
   }while(1)
 
-static struct pike_string *recode_string(struct pike_string *data)
+static struct pike_string *recode_string(struct cpp *this, struct pike_string *data)
 {
   /* Observations:
    *
@@ -862,13 +1002,18 @@ static struct pike_string *recode_string(struct pike_string *data)
 
     push_constant_text("ebcdic-us");
 
-    SAFE_APPLY_MASTER("decode_charset", 2);
-
-    /* Various consistency checks. */
-    if ((sp[-1].type != T_STRING) || (sp[-1].u.string->size_shift) ||
-	(((size_t)sp[-1].u.string->len) < CONSTANT_STRLEN("#charset")) ||
-	(sp[-1].u.string->str[0] != '#')) {
-      pop_stack();
+    if (safe_apply_handler ("decode_charset", this->handler, this->compat_handler,
+			    2, BIT_STRING)) {
+      /* Various consistency checks. */
+      if ((sp[-1].u.string->size_shift) ||
+	  (((size_t)sp[-1].u.string->len) < CONSTANT_STRLEN("#charset")) ||
+	  (sp[-1].u.string->str[0] != '#')) {
+	pop_stack();
+	return data;
+      }
+    }
+    else {
+      cpp_handle_exception (this, "Error decoding with charset 'ebcdic-us'");
       return data;
     }
 
@@ -925,16 +1070,17 @@ static struct pike_string *recode_string(struct pike_string *data)
     MEMCPY(new_str->str, p, len);
 
     pop_stack();
-    push_string(end_shared_string(new_str));
+    ref_push_string(end_shared_string(new_str));
 		
     /* Decode the string */
 
-    SAFE_APPLY_MASTER("decode_charset", 2);
-
-    if (sp[-1].type != T_STRING) {
-      pop_stack();
+    if (!safe_apply_handler ("decode_charset", this->handler, this->compat_handler,
+			     2, BIT_STRING)) {
+      cpp_handle_exception (this, "Error decoding with charset '%s'", new_str->str);
+      free_string (new_str);
       return data;
     }
+    free_string (new_str);
 
     /* Accept the new string */
 
@@ -1218,8 +1364,28 @@ static void check_constant(struct cpp *this,
       }
 
       if (safe_apply_handler("resolv", this->handler,
-			     this->compat_handler, 3, 0))
-	res = !(SAFE_IS_ZERO(sp-1) && sp[-1].subtype == NUMBER_UNDEFINED);
+			     this->compat_handler, 3, 0)) {
+	if ((Pike_sp[-1].type == T_OBJECT &&
+	     Pike_sp[-1].u.object == placeholder_object) ||
+	    (Pike_sp[-1].type == T_PROGRAM &&
+	     Pike_sp[-1].u.program == placeholder_program)) {
+	  if (!data.shift) {
+	    char *str = malloc(dlen + 1);
+	    MEMCPY(str, data.ptr, dlen);
+	    str[dlen] = 0;
+	    cpp_error_sprintf (this, "Got placeholder %s (resolver problem) "
+			       "when resolving '%s'.",
+			       get_name_of_type (Pike_sp[-1].type), str);
+	    free (str);
+	  }
+	  else
+	    cpp_error_sprintf (this, "Got placeholder %s (resolver problem).",
+			       get_name_of_type (Pike_sp[-1].type));
+	  res = 0;
+	}
+	else
+	  res = !(SAFE_IS_ZERO(sp-1) && sp[-1].subtype == NUMBER_UNDEFINED);
+      }
       else {
 	if (throw_value.type == T_STRING && !throw_value.u.string->size_shift) {
 	  cpp_error(this, throw_value.u.string->str);
@@ -1228,25 +1394,15 @@ static void check_constant(struct cpp *this,
 	  res = 0;
 	}
 	else {
-	  struct svalue thrown = throw_value;
-	  throw_value.type = T_INT;
-
 	  if (!data.shift) {
 	    char *str = malloc(dlen + 1);
 	    MEMCPY(str, data.ptr, dlen);
 	    str[dlen] = 0;
-	    cpp_error_sprintf(this, "Error resolving '%s'.", str);
+	    cpp_handle_exception (this, "Error resolving '%s'.", str);
 	    free(str);
 	  }
 	  else
-	    cpp_error(this, "Error resolving identifier.");
-
-	  push_svalue(&thrown);
-	  low_safe_apply_handler("compile_exception", this->handler,
-				 this->compat_handler, 1);
-	  if (SAFE_IS_ZERO(sp-1)) cpp_describe_exception(this, &thrown);
-	  pop_stack();
-	  free_svalue(&thrown);
+	    cpp_handle_exception (this, "Error resolving identifier.");
 	  res = 0;
 	}
       }
@@ -1269,17 +1425,7 @@ static void check_constant(struct cpp *this,
 			   BIT_MAPPING|BIT_OBJECT|BIT_PROGRAM))
       res = !(SAFE_IS_ZERO(sp-1) && sp[-1].subtype == NUMBER_UNDEFINED);
     else {
-      struct svalue thrown = throw_value;
-      throw_value.type = T_INT;
-
-      cpp_error(this, "Error importing '.'.");
-
-      push_svalue(&thrown);
-      low_safe_apply_handler("compile_exception", this->handler,
-			     this->compat_handler, 1);
-      if (SAFE_IS_ZERO(sp-1)) cpp_describe_exception(this, &thrown);
-      pop_stack();
-      free_svalue(&thrown);
+      cpp_handle_exception (this, "Error importing '.'.");
       res = 0;
     }
   }
@@ -1388,15 +1534,16 @@ static int do_safe_index_call(struct cpp *this, struct pike_string *s)
   if(!s) return 0;
 
   if (SETJMP_SP(recovery, 1)) {
-#if 0
-    if (!s->size_shift)
-      cpp_handle_exception (this, "Error indexing module with \"%s\".", s->str);
-    else
-      cpp_handle_exception (this, "Error indexing module in '.' operator.");
-#else
-    free_svalue (&throw_value);
-    throw_value.type = T_INT;
-#endif
+    if (CPP_TEST_COMPAT (this, 7, 4)) {
+      free_svalue (&throw_value);
+      throw_value.type = T_INT;
+    }
+    else {
+      if (!s->size_shift)
+	cpp_handle_exception (this, "Error indexing module with \"%s\".", s->str);
+      else
+	cpp_handle_exception (this, "Error indexing module in '.' operator.");
+    }
     res = 0;
     push_undefined();
   } else {
@@ -1409,6 +1556,12 @@ static int do_safe_index_call(struct cpp *this, struct pike_string *s)
   return res;
 }
 
+/*! @namespace predef:: */
+/*! @decl import cpp:: */
+/*! @endnamespace */
+
+/*! @namespace cpp:: */
+
 /*! @decl constant __VERSION__
  *!
  *! This define contains the current Pike version as a float. If
@@ -1531,6 +1684,8 @@ static int do_safe_index_call(struct cpp *this, struct pike_string *s)
  *! This define is defined when the Pike is running on Amiga OS.
  */
 
+/*! @endnamespace */
+
 /*! @decl string cpp(string data, string|void current_file, @
  *!                  int|string|void charset, object|void handler, @
  *!                  void|int compat_major, void|int compat_minor)
@@ -1590,10 +1745,12 @@ void f_cpp(INT32 args)
 	charset = sp[2 - args].u.string;
 	push_string(data);
 	ref_push_string(charset);
-	SAFE_APPLY_MASTER("decode_charset", 2);
-	if (sp[-1].type != T_STRING) {
+	if (!safe_apply_handler ("decode_charset", this.handler, this.compat_handler,
+				 2, BIT_STRING)) {
 	  free_string(this.current_file);
-	  Pike_error("Unknown charset\n");
+	  cpp_handle_exception (&this, "Error decoding with charset '%s'",
+				charset->str);
+	  Pike_error("Unknown charset.\n");
 	}
 	data = sp[-1].u.string;
 	sp--;
@@ -1683,7 +1840,7 @@ void f_cpp(INT32 args)
 
   if (auto_convert && (!data->size_shift) && (data->len > 1)) {
     /* Try to determine if we need to recode the string */
-    struct pike_string *new_data = recode_string(data);
+    struct pike_string *new_data = recode_string(&this, data);
     free_string(data);
     data = new_data;
   }
@@ -1730,6 +1887,10 @@ void f_cpp(INT32 args)
 #ifdef __amigaos__
     simple_add_define(&this, "__amigaos__", " 1 ");
 #endif
+    simple_add_define(&this, "SIZEOF_INT",
+		      " " DEFINETOSTR(SIZEOF_INT) " ");
+    simple_add_define(&this, "SIZEOF_FLOAT",
+		      " " DEFINETOSTR(SIZEOF_FLOAT) " ");
   }
 
   if (predefs) {
