@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: cpp.h,v 1.8 2004/03/23 13:41:56 grubba Exp $
+|| $Id: cpp.h,v 1.9 2004/04/15 22:07:48 mast Exp $
 */
 
 #ifndef CPP_H
@@ -18,7 +18,7 @@ struct define_part;
 struct define_argument;
 struct define;
 struct cpp;
-void cpp_error(struct cpp *this,char *err);
+void cpp_error(struct cpp *this, const char *err);
 void cpp_error_vsprintf (struct cpp *this, const char *fmt, va_list args);
 void cpp_error_sprintf(struct cpp *this, const char *fmt, ...)
   ATTRIBUTE((format(printf,2,3)));
@@ -34,4 +34,10 @@ void add_predefine(char *s);
 void exit_cpp(void);
 /* Prototypes end here */
 
+/* Return true if compat version is equal or less than MAJOR.MINOR */
+#define CPP_TEST_COMPAT(THIS,MAJOR,MINOR)      \
+  (THIS->compat_major < (MAJOR) ||	       \
+   (THIS->compat_major == (MAJOR) &&	       \
+    THIS->compat_minor <= (MINOR)))
+
 #endif
