@@ -1,7 +1,7 @@
-/* A Bison parser, made by GNU Bison 2.1.  */
+/* A Bison parser, made by GNU Bison 1.875a.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.  */
 
 /* As a special exception, when this file is copied by Bison into a
    Bison output file, you may use that output file without restriction.
@@ -110,7 +110,6 @@
      TOK_OPTIONAL = 336
    };
 #endif
-/* Tokens.  */
 #define TOK_ARROW 258
 #define TOK_CONSTANT 259
 #define TOK_FLOAT 260
@@ -195,7 +194,7 @@
 
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 176 "/tmp/pikedeb.5a120871e4/7.4/src/language.yacc"
+#line 176 "/tmp/pikedeb.ea910e171c/7.6/src/language.yacc"
 typedef union YYSTYPE {
   int number;
   FLOAT_TYPE fnum;
@@ -203,8 +202,8 @@ typedef union YYSTYPE {
   char *str;
   void *ptr;
 } YYSTYPE;
-/* Line 1447 of yacc.c.  */
-#line 208 "y.tab.h"
+/* Line 1240 of yacc.c.  */
+#line 207 "y.tab.h"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
