@@ -1,12 +1,13 @@
 /*
- * $Id: requestobject.c,v 1.16 2000/12/01 08:09:58 hubbe Exp $
- */
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: requestobject.c,v 1.29 2004/03/10 16:31:41 nilsson Exp $
+*/
 
 #include "global.h"
 #include "config.h"
 	  
-#include "array.h"
-#include "backend.h"
 #include "machine.h"
 #include "mapping.h"
 #include "module_support.h"
@@ -14,7 +15,6 @@
 #include "object.h"
 #include "operators.h"
 #include "pike_memory.h"
-#include "program.h"
 #include "constants.h"
 #include "stralloc.h"
 #include "svalue.h"
@@ -41,17 +41,18 @@
 #include <sys/uio.h>
 #endif
 
-/* This must be included last! */
-#include "module_magic.h"
+
+#define sp Pike_sp
 
 #ifdef _REENTRANT
+#include "pike_netlib.h"
 #include "accept_and_parse.h"
 #include "log.h"
 #include "util.h"
 #include "cache.h"
 #include "requestobject.h"
 
-/* Used when fatal() can't be. */
+/* Used when Pike_fatal() can't be. */
 #define DWERROR(X)	write(2, X, sizeof(X) - sizeof(""))
 
 /* All current implementations of sendfile(2) are broken. */
@@ -246,7 +247,7 @@ void f_aap_scan_for_query(INT32 args)
 	 push_string(make_shared_binary_string(work_area+last,k-last));
 	 n++;
 	 begin = k+1;
-	 f_aggregate_multiset(DO_NOT_WARN(n));
+	 f_aggregate_multiset(DO_NOT_WARN((INT32)n));
 	 goto done2;
       }
     }
@@ -465,7 +466,14 @@ void f_aap_index_op(INT32 args)
 
   if(s == s_remoteaddr)
   {
-    push_text(inet_ntoa(THIS->request->from.sin_addr));
+#ifdef HAVE_INET_NTOP
+    char buffer[64];
+    push_text(inet_ntop(SOCKADDR_FAMILY(THIS->request->from),
+			SOCKADDR_IN_ADDR(THIS->request->from),
+			buffer, sizeof(buffer)) );
+#else
+    push_text(inet_ntoa(*SOCKADDR_IN_ADDR(THIS->request->from)));
+#endif
     push_string(s_remoteaddr);
     mapping_insert(THIS->misc_variables, sp-1, sp-2);
     sp--;
@@ -700,7 +708,7 @@ struct send_args
 };
 
 static int num_send_args;
-struct send_args *new_send_args()
+struct send_args *new_send_args(void)
 {
   num_send_args++;
   return aap_malloc( sizeof( struct send_args ) );
@@ -724,7 +732,7 @@ void actually_send(struct send_args *a)
   foo[9]=0; foo[6]=0;
   if( a->data )
   {
-    data = a->data->str;
+    data = (unsigned char *)a->data->str;
     data_len = a->data->len;
   }
 #ifdef HAVE_FREEBSD_SENDFILE
@@ -780,7 +788,7 @@ void actually_send(struct send_args *a)
 	}
 	break;
       case EFAULT:	/* Invalid address specified as arg */
-	/* NOTE: Can't use fatal(), since we're not in a valid Pike context. */
+	/* NOTE: Can't use Pike_fatal(), since we're not in a valid Pike context. */
 	DWERROR("FreeBSD-style sendfile() returned EFAULT.\n");
 	abort();
 	break;
@@ -830,7 +838,7 @@ void actually_send(struct send_args *a)
 #endif
   if(data)
   {
-    MEMCPY(foo, data+MY_MIN((data_len-4),9), 4);
+    MEMCPY(foo, data+MINIMUM((data_len-4),9), 4);
     first=1;
 #ifdef TCP_CORK
 #ifdef AAP_DEBUG
@@ -841,7 +849,7 @@ void actually_send(struct send_args *a)
       setsockopt( a->to->fd, SOL_TCP, TCP_CORK, &true, 4 );
     }
 #endif
-    fail = WRITE(a->to->fd, data, data_len);
+    fail = WRITE(a->to->fd, (char *)data, data_len);
     a->sent += fail;
     if(fail != data_len)
       goto end;
@@ -882,7 +890,10 @@ void actually_send(struct send_args *a)
   }
 #endif /* HAVE_SENDFILE && !HAVE_FREEBSD_SENDFILE */
 
+#if defined(HAVE_FREEBSD_SENDFILE) || defined(HAVE_SENDFILE)
  normal:
+#endif /* HAVE_FREEBSD_SENDFILE || HAVE_SENDFILE */
+
 #ifdef AAP_DEBUG
   fprintf(stderr, "using normal fallback... \n");
 #endif
@@ -903,7 +914,7 @@ void actually_send(struct send_args *a)
   while(a->len)
   {
     ptrdiff_t nread, written=0;
-    nread = fd_read(a->from_fd, a->buffer, MY_MIN(BUFFER,a->len));
+    nread = fd_read(a->from_fd, a->buffer, MINIMUM(BUFFER,a->len));
 #ifdef AAP_DEBUG
   fprintf(stderr, "writing %d bytes... \n", nread);
 #endif
@@ -1028,7 +1039,7 @@ void f_aap_reply_with_cache(INT32 args)
   if(!THIS->request)
     Pike_error("Reply already called.\n");
 
-  get_all_args("reply_with_cache", args, "%S%d", &reply, &time_to_keep);
+  get_all_args("reply_with_cache", args, "%S%i", &reply, &time_to_keep);
 
   if((size_t)reply->len < (size_t)THIS->request->cache->max_size/2)
   {
