@@ -1,60 +1,43 @@
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: psd.c,v 1.41 2004/03/06 00:06:59 nilsson Exp $
+*/
+
 #include "global.h"
-RCSID("$Id: psd.c,v 1.28 2003/09/24 08:48:06 jonasw Exp $");
+RCSID("$Id: psd.c,v 1.41 2004/03/06 00:06:59 nilsson Exp $");
 
 #include "image_machine.h"
 
 #include "pike_macros.h"
 #include "object.h"
-#include "constants.h"
 #include "module_support.h"
 #include "interpret.h"
 #include "object.h"
 #include "svalue.h"
 #include "threads.h"
-#include "array.h"
 #include "interpret.h"
 #include "svalue.h"
 #include "mapping.h"
 #include "pike_error.h"
 #include "stralloc.h"
 #include "builtin_functions.h"
-#include "operators.h"
-#include "dynamic_buffer.h"
-#include "pike_macros.h"
 
 #include "image.h"
 #include "colortable.h"
-
-/* MUST BE INCLUDED LAST */
-#include "module_magic.h"
-
-
-/* The sp macro conflicts with Solaris 2.5.1's <netinet/in.h>. */
-#ifdef sp
-#undef sp
-#define STACKPOINTER_WAS_DEFINED
-#endif /* sp */
+#include "bignum.h"
 
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
 
-/* Restore the sp macro */
-#ifdef STACKPOINTER_WAS_DEFINED
-#define sp Pike_sp
-#undef STACK_POINTER_WAS_DEFINED
-#endif /* STACKPOINTER_WAS_DEFINED */
 
+#define sp Pike_sp
 
 extern struct program *image_colortable_program;
 extern struct program *image_program;
 
-/*
-**! module Image
-**! submodule PSD
-**!
-*/
-
 #define STRING(X) static struct pike_string *PIKE_CONCAT(s_, X)
 #include "psd_constant_strings.h"
 #undef STRING
@@ -205,7 +188,6 @@ struct layer
   unsigned int num_channels;
   unsigned int clipping;
   unsigned int flags;
-  int compression;
   struct channel_info channel_info[24];
   struct buffer mode;
   struct buffer extra_data;
@@ -216,10 +198,7 @@ static void decode_layers_and_masks( struct psd_image *dst,
                                      struct buffer *src )
 {
   short count, first_alpha_is_magic;
-  unsigned int normal;
-  unsigned int *q;
   struct layer *layer = NULL;
-  int i;
   ptrdiff_t exp_offset;
   if(!src->len) 
     return;
@@ -258,7 +237,7 @@ static void decode_layers_and_masks( struct psd_image *dst,
     layer->opacity = psd_read_uchar( src );
     layer->clipping = psd_read_uchar( src );
     layer->flags = psd_read_uchar( src );
-    psd_read_uchar( src );
+    psd_read_uchar( src ); /* filler byte */
     layer->extra_data = psd_read_string( src );
     layer->extra_data.len++;
     if(layer->extra_data.len)
@@ -278,7 +257,7 @@ static void decode_layers_and_masks( struct psd_image *dst,
       tmp2 = psd_read_string( &tmp );
       if( tmp2.len )
       {
-	/* ranges (?) */
+	/* Layer blending ranges data */
       }
       layer->name = psd_read_pstring( &tmp );
     }
@@ -304,7 +283,7 @@ packbitsdecode(struct buffer src,
                struct buffer dst, 
                int nbytes)
 {
-  int n, b;
+  int n;
 
   while( nbytes--  )
   {
@@ -355,7 +334,7 @@ static void f_decode_packbits_encoded(INT32 args)
   int compression = 0;
   struct buffer b, ob, d;
   if(sp[-args].type != T_STRING)
-    Pike_error("Internal argument error");
+    Pike_error("Internal argument error.\n");
 
 
   if(args == 5)
@@ -401,13 +380,13 @@ static void f_decode_packbits_encoded(INT32 args)
 
 static void f_decode_image_channel( INT32 args )
 {
-  INT_TYPE w, h, d;
+  INT_TYPE w, h;
   int y;
   struct pike_string *s;
   struct object *io;
   unsigned char *source;
   rgb_group *dst;
-  get_all_args( "_decode_image_channel",args, "%d%d%S", &w,&h,&s);
+  get_all_args( "_decode_image_channel",args, "%i%i%S", &w,&h,&s);
 
   ref_push_string( s );
   push_int( h );
@@ -441,7 +420,7 @@ static void f_decode_image_data( INT32 args )
   struct object *io;
   unsigned char *source, *source2, *source3, *source4;
   rgb_group *dst;
-  get_all_args( "_decode_image_data",args, "%d%d%d%d%d%S%S", 
+  get_all_args( "_decode_image_data",args, "%i%i%i%i%i%S%S", 
                 &w,&h,&d,&m,&c,&s,&ct);
 
   if(!ct->len) ct = NULL;
@@ -567,7 +546,6 @@ void push_layer( struct layer  *l)
   ref_push_string( s_opacity );       push_int( l->opacity );
   ref_push_string( s_clipping );      push_int( l->clipping );
   ref_push_string( s_flags );         push_int( l->flags );
-  ref_push_string( s_compression );   push_int( l->compression );
   ref_push_string( s_mode );          push_buffer( &l->mode );
   ref_push_string( s_extra_data );    push_buffer( &l->extra_data );
   ref_push_string( s_name );          push_buffer( &l->name );
@@ -581,7 +559,7 @@ void push_layer( struct layer  *l)
     f_aggregate_mapping( 4 );
   }
   f_aggregate( l->num_channels );
-  f_aggregate_mapping(DO_NOT_WARN(sp - osp));
+  f_aggregate_mapping(DO_NOT_WARN((INT32)(sp - osp)));
 }
 
 
@@ -594,10 +572,9 @@ void push_psd_image( struct psd_image *i )
   ref_push_string( s_height ); push_int( i->rows );
   ref_push_string( s_width );  push_int( i->columns );
   ref_push_string( s_compression ); push_int( i->compression );
-  ref_push_string( s_depth ); push_int( i->compression );
+  ref_push_string( s_depth ); push_int( i->depth );
   ref_push_string( s_mode ); push_int( i->mode );
   ref_push_string( s_color_data ); push_buffer( &i->color_data );
-/*   ref_push_string( s_resource_data ); push_buffer( &i->resource_data ); */
   ref_push_string( s_resources ); decode_resources( &i->resource_data );
   ref_push_string( s_image_data ); push_buffer( &i->image_data );
   ref_push_string( s_layers );
@@ -608,8 +585,8 @@ void push_psd_image( struct psd_image *i )
     push_layer( l );
     l = l->next;
   }
-  f_aggregate(DO_NOT_WARN(sp - tsp));
-  f_aggregate_mapping(DO_NOT_WARN(sp - osp));
+  f_aggregate(DO_NOT_WARN((INT32)(sp - tsp)));
+  f_aggregate_mapping(DO_NOT_WARN((INT32)(sp - osp)));
 }
 
 static void decode_resources( struct buffer *b )
@@ -623,6 +600,10 @@ static void decode_resources( struct buffer *b )
     int id;
     struct buffer data;
     struct buffer name;
+
+    /* Non-photoshop software may use another identifier and
+       still be compatible with the Adobe PSD file format specification.
+       FIXME? */
     if( signature[0] != '8' ||  signature[1] != 'B' ||
         signature[2] != 'I' ||  signature[3] != 'M' )
       break;
@@ -666,19 +647,26 @@ static void decode_resources( struct buffer *b )
 	  }
 	  for (i=0; i<num_guides; i++)
 	  {
-	    int p = psd_read_int( &data );
+	    ptrdiff_t p = psd_read_int( &data );
 	    int h = psd_read_uchar( &data );
 	    if( h )
-	      p = (int)((((double)p) * (magic4>>8)) / ((double)(magic4&255)));
+	      p = (ptrdiff_t)((((double)p) * (magic4>>8)) / ((double)(magic4&255)));
 	    else
-	      p = (int)((((double)p) * (magic6>>8)) / ((double)(magic6&255)));
-	    push_constant_text( "pos" );      push_int( p );
+	      p = (ptrdiff_t)((((double)p) * (magic6>>8)) / ((double)(magic6&255)));
+	    push_constant_text( "pos" );      push_int64( p );
 	    push_constant_text( "vertical" ); push_int( !h );
 	    f_aggregate_mapping( 4 );
 	  }
 	  f_aggregate( num_guides );
 	}
 	break;
+      case 0x040b: /* URL */
+	{
+	  struct buffer b = psd_read_pstring( &data );
+	  push_constant_text( "url" );
+	  push_buffer( &b );
+	}
+	break;
       case 0x03ed: /* res. info. */
 	push_constant_text( "resinfo" );
 
@@ -692,13 +680,70 @@ static void decode_resources( struct buffer *b )
 
 	f_aggregate_mapping( 12 );
 	break;
+
+      case 0x03fc: /* Declared completely obsolete by Adobe */
+      case 0x03ff:
+      case 0x0403:
+	break;
+
+      case 0x03e8: /* Photoshop 2.0. Obsolete! */
+	/* 5*int16  channels, rows, columns, depth, mode */
+
+      case 0x03e9: /* Machintosh print manager print info record. */
+
+      case 0x03eb: /* Photoshop 2.0. Obsolete! Indexed color table */
+
+      case 0x03ee:
+        /* Names of alpha channels as a series of pascal strings */
+
+      case 0x03ef: /* DisplayInfo structure */
+
+      case 0x03f1: /* Border information */
+	/* Contains a fixed-number for the border width and an int16 for the
+	   border units (1=inches, 2=cm, 3=points, 4=picas, 5=columns) */
+
+      case 0x03f2: /* Background color. */
+
+      case 0x03f3: /* Print flags */
+	/* A series of one byte boolean values for
+	   labels, crop marks, color bars, registration marks, negative, flip
+	   interpolate and caption */
+
+      case 0x03f4: /* Grayscale and multichannel halftoning information. */
+      case 0x03f5: /* Color halftoning information. */
+      case 0x03f6: /* Duotone halftoning information. */
+      case 0x03f7: /* Grayscale and multichannel transfer functions. */
+      case 0x03f8: /* Color transfer functions. */
+      case 0x03f9: /* Duotone transfer functions. */
+      case 0x03fa: /* Duotone image information. */
+      case 0x03fb: /* Two bytes for the effective black and white values
+		      for the dot range. */
+      case 0x03fd: /* EPS options */
+      case 0x03fe: /* Quick Mask information. 2 bytes containing Quick Mask
+		      channel ID. One byte boolean indicating whether the
+		      mask was initially empty. */
+      case 0x0401: /* Working path */
+      case 0x0402: /* Layers group information. 2 bytes per layer containing a
+		      group ID for the dragging groups. */
+      case 0x0404: /* IPTC-NAA record */
+      case 0x0405: /* Image mode for raw format files. */
+      case 0x0406: /* JPEG quality */
+      case 0x040A: /* Copyright flag */
+      case 0x0bb7: /* Name of clipping path */
+      case 0x2710: /* Print flags information .
+		      2 bytes version (=1), 1 byte center crop marks,
+		      1 zeroed byte, 4 bytes bleed width value,
+		      2 bytes bleed width scale. */
       default:
+	if ((id >= 0x07d0) && (id <= 0x0bb6)) {
+	  /* Path information */
+	}
 	push_int( id );
 	push_buffer( &data );
 	break;
     }
   }
-  f_aggregate_mapping( sp-osp );
+  f_aggregate_mapping( DO_NOT_WARN((int)(sp-osp)) );
 }
 
 static void image_f_psd___decode( INT32 args )
@@ -708,6 +753,8 @@ static void image_f_psd___decode( INT32 args )
   get_all_args( "Image.PSD.___decode", args, "%S", &s );
   if(args > 1)
     pop_n_elems( args-1 );
+  if(s->len < 26+4+4+4 ) /* header+color mode+image res+layers */
+    Pike_error("This is not a Photoshop PSD file (too short)\n");
   if(s->str[0] != '8' || s->str[1] != 'B'  
      || s->str[2] != 'P'  || s->str[3] != 'S' )
     Pike_error("This is not a Photoshop PSD file (invalid signature)\n");
@@ -795,4 +842,3 @@ void exit_image_psd()
 #include "psd_constant_strings.h"
 #undef STRING
 }
-
