@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: jvm.c,v 1.50 2004/01/02 12:11:04 jonasw Exp $
+|| $Id: jvm.c,v 1.69 2004/04/17 16:51:28 marcus Exp $
 */
 
 /*
@@ -22,12 +22,11 @@
 #endif /* HAVE_CONFIG_H */
 
 #include "global.h"
-RCSID("$Id: jvm.c,v 1.50 2004/01/02 12:11:04 jonasw Exp $");
+RCSID("$Id: jvm.c,v 1.69 2004/04/17 16:51:28 marcus Exp $");
 #include "program.h"
 #include "interpret.h"
 #include "stralloc.h"
 #include "object.h"
-#include "mapping.h"
 #include "builtin_functions.h"
 #include "pike_error.h"
 #include "module_support.h"
@@ -39,6 +38,7 @@ RCSID("$Id: jvm.c,v 1.50 2004/01/02 12:11:04 jonasw Exp $");
 #ifdef HAVE_JAVA
 
 #include <stdarg.h>
+#include <locale.h>
 
 #ifdef HAVE_JNI_H
 #include <jni.h>
@@ -225,7 +225,7 @@ static void push_java_class(jclass c, struct object *jvm, JNIEnv *env)
   jo = (struct jobj_storage *)(oo->storage);
   jo->jvm = jvm;
   jo->jobj = c2;
-  jvm->refs++;
+  add_ref(jvm);
 }
 
 static void push_java_throwable(jthrowable t, struct object *jvm, JNIEnv *env)
@@ -244,7 +244,7 @@ static void push_java_throwable(jthrowable t, struct object *jvm, JNIEnv *env)
   jo = (struct jobj_storage *)(oo->storage);
   jo->jvm = jvm;
   jo->jobj = t2;
-  jvm->refs++;
+  add_ref(jvm);
 }
 
 static void push_java_array(jarray t, struct object *jvm, JNIEnv *env, int ty)
@@ -266,7 +266,7 @@ static void push_java_array(jarray t, struct object *jvm, JNIEnv *env, int ty)
   jo->jobj = t2;
   a = (struct jarray_storage *)(oo->storage+jarray_stor_offs);
   a->ty = ty;
-  jvm->refs++;
+  add_ref(jvm);
 }
 
 static void push_java_anyobj(jobject o, struct object *jvm, JNIEnv *env)
@@ -303,7 +303,7 @@ static void push_java_anyobj(jobject o, struct object *jvm, JNIEnv *env)
   jo = (struct jobj_storage *)(oo->storage);
   jo->jvm = jvm;
   jo->jobj = o2;
-  jvm->refs++;
+  add_ref(jvm);
 }
 
 static void init_jobj_struct(struct object *o)
@@ -554,11 +554,9 @@ static void f_method_create(INT32 args)
   }
 
   m->class = class;
-  m->name = name;
-  m->sig = sig;
-  class->refs++;
-  name->refs++;
-  sig->refs++;
+  copy_shared_string(m->name, name);
+  copy_shared_string(m->sig, sig);
+  add_ref(class);
   pop_n_elems(args);
   push_int(0);
 
@@ -1167,7 +1165,7 @@ static void f_field_create(INT32 args)
 
   if(name == NULL || sig == NULL) {
     f->class = class;
-    class->refs++;
+    add_ref(class);
     pop_n_elems(args);
     f->type = 0;
     return;
@@ -1189,11 +1187,9 @@ static void f_field_create(INT32 args)
   }
 
   f->class = class;
-  f->name = name;
-  f->sig = sig;
-  class->refs++;
-  name->refs++;
-  sig->refs++;
+  copy_shared_string(f->name, name);
+  copy_shared_string(f->sig, sig);
+  add_ref(class);
   pop_n_elems(args);
   push_int(0);
 
@@ -1861,17 +1857,14 @@ static void make_java_exception(struct object *jvm, JNIEnv *env,
   if(!j)
     return;
 
-  if(v->type == PIKE_T_ARRAY && v->u.array->size &&
-     (a=low_array_get_item_ptr(v->u.array, 0, PIKE_T_STRING))!=NULL) {
-    (*env)->ThrowNew(env, j->class_runtimex, a->string->str);
-  } else if(v->type == PIKE_T_OBJECT &&
-	    (gen_err = (struct generic_error_struct *)
-	     get_storage(v->u.object, generic_error_program)) != NULL) {
-    (*env)->ThrowNew(env, j->class_runtimex, gen_err->desc->str);
-  } else {
-    (*env)->ThrowNew(env, j->class_runtimex,
-		     "Nonstandard pike exception thrown.");
-  }
+  push_svalue (v);
+  SAFE_APPLY_MASTER ("describe_error", 1);
+#ifdef PIKE_DEBUG
+  if (Pike_sp[-1].type != PIKE_T_STRING)
+    Pike_fatal ("Unexpected return value from destribe_error\n");
+#endif
+  (*env)->ThrowNew(env, j->class_runtimex, Pike_sp[-1].u.string->str);
+  pop_stack();
 }
 
 #ifdef VARARG_NATIVE_DISPATCH
@@ -2010,28 +2003,28 @@ static void native_dispatch(struct native_method_context *ctx,
     }
   } else {
     /* Not a pike thread.  Create a temporary thread_id... */
+    struct object *thread_obj;
+
     mt_lock_interpreter();
     init_interpreter();
     Pike_interpreter.stack_top=((char *)&state)+ (thread_stack_size-16384) * STACK_DIRECTION;
     Pike_interpreter.recoveries = NULL;
-    {
-      struct object *o = Pike_interpreter.thread_id =
-	low_clone(thread_id_prog);
-      call_c_initializers(Pike_interpreter.thread_id);
-      SWAP_OUT_THREAD(OBJ2THREAD(Pike_interpreter.thread_id));
-      Pike_interpreter.thread_id = o;
-      OBJ2THREAD(Pike_interpreter.thread_id)->swapped=0;
-      OBJ2THREAD(Pike_interpreter.thread_id)->id=th_self();
-      num_threads++;
-      thread_table_insert(Pike_interpreter.thread_id);
-      do_native_dispatch(ctx, env, cls, args, rc);
-      OBJ2THREAD(Pike_interpreter.thread_id)->status=THREAD_EXITED;
-      co_signal(& OBJ2THREAD(Pike_interpreter.thread_id)->status_change);
-      thread_table_delete(Pike_interpreter.thread_id);
-      free_object(Pike_interpreter.thread_id);
-      Pike_interpreter.thread_id=NULL;
-    }
-    cleanup_interpret();
+    thread_obj = fast_clone_object(thread_id_prog);
+    INIT_THREAD_STATE((struct thread_state *)(thread_obj->storage +
+					      thread_storage_offset));
+    num_threads++;
+    thread_table_insert(Pike_interpreter.thread_state);
+
+    do_native_dispatch(ctx, env, cls, args, rc);
+
+    cleanup_interpret();	/* Must be done before EXIT_THREAD_STATE */
+    Pike_interpreter.thread_state->status=THREAD_EXITED;
+    co_signal(&Pike_interpreter.thread_state->status_change);
+    thread_table_delete(Pike_interpreter.thread_state);
+    EXIT_THREAD_STATE(Pike_interpreter.thread_state);
+    Pike_interpreter.thread_state=NULL;
+    free_object(thread_obj);
+    thread_obj = NULL;
     num_threads--;
     mt_unlock_interpreter();
   }
@@ -2152,10 +2145,8 @@ static void build_native_entry(JNIEnv *env, jclass cls,
   int statc, args=0, wargs=0, flt_args=0, dbl_args=0;
   char *p = sig->str;
 
-  con->name = name;
-  con->sig = sig;
-  name->refs++;
-  sig->refs++;
+  copy_shared_string(con->name, name);
+  copy_shared_string(con->sig, sig);
 
   if((*env)->GetMethodID(env, cls, name->str, sig->str))
     statc = 0;
@@ -2316,8 +2307,8 @@ static void f_natives_create(INT32 args)
     
     n->jvm = c->jvm;
     n->cls = cls;
-    n->jvm->refs++;
-    n->cls->refs++;
+    add_ref(n->jvm);
+    add_ref(n->cls);
 
     rc = (*env)->RegisterNatives(env, c->jobj, n->jnms, n->num_methods);
     jvm_vacate_env(c->jvm, env);
@@ -2589,8 +2580,7 @@ static void javaarray_subarray(struct object *jvm, struct object *oo,
     if(e2==size) {
       /* Entire array selected */
       jvm_vacate_env(jvm, env);
-      oo->refs++;
-      push_object(oo);
+      ref_push_object(oo);
       return;
     }
 
@@ -2656,7 +2646,7 @@ static void f_javaarray_getelt(INT32 args)
 
   if(args<1 || Pike_sp[-args].type != PIKE_T_INT ||
      (args>1 && Pike_sp[1-args].type != PIKE_T_INT))
-    Pike_error("Bad args to `[].");
+    Pike_error("Bad args to `[].\n");
 
   n = Pike_sp[-args].u.integer;
 
@@ -2733,7 +2723,7 @@ static void f_javaarray_setelt(INT32 args)
   char ty2;
 
   if(args<2 || Pike_sp[-args].type != PIKE_T_INT)
-    Pike_error("Bad args to `[]=.");
+    Pike_error("Bad args to `[]=.\n");
 
   if(args>2)
     pop_n_elems(args-2);
@@ -3062,7 +3052,7 @@ static void f_monitor_create(INT32 args)
 #endif /* _REENTRANT */
 
   m->obj = obj;
-  obj->refs++;
+  add_ref(obj);
   pop_n_elems(args);
   return;
 }
@@ -3095,8 +3085,7 @@ static void f_create(INT32 args)
   /* Set classpath */
   if(args>0 && Pike_sp[-args].type == PIKE_T_STRING) {
     classpath = Pike_sp[-args].u.string->str;
-    Pike_sp[-args].u.string->refs++;
-    j->classpath_string = Pike_sp[-args].u.string;
+    copy_shared_string(j->classpath_string, Pike_sp[-args].u.string);
   } else {
     if(getenv("CLASSPATH"))
       classpath = getenv("CLASSPATH");
@@ -3118,8 +3107,7 @@ static void f_create(INT32 args)
     push_string(j->classpath_string);
     j->classpath_string = NULL;
     f_add(2);
-    Pike_sp[-1].u.string->refs++;
-    j->classpath_string = Pike_sp[-1].u.string;
+    copy_shared_string(j->classpath_string, Pike_sp[-1].u.string);
     pop_n_elems(1);
     j->vm_args.options[j->vm_args.nOptions].optionString =
       j->classpath_string->str;
@@ -3140,6 +3128,25 @@ static void f_create(INT32 args)
   if(JNI_CreateJavaVM(&j->jvm, (void**)&j->env, &j->vm_args))
     Pike_error( "Failed to create virtual machine\n" );
 
+  /* Java tries to be a wiseguy with the locale... */
+#ifdef HAVE_SETLOCALE
+#ifdef LC_NUMERIC
+  setlocale(LC_NUMERIC, "C");
+#endif
+#ifdef LC_CTYPE
+  setlocale(LC_CTYPE, "");
+#endif
+#ifdef LC_TIME
+  setlocale(LC_TIME, "C");
+#endif
+#ifdef LC_COLLATE
+  setlocale(LC_COLLATE, "");
+#endif
+#ifdef LC_MESSAGES
+  setlocale(LC_MESSAGES, "");
+#endif
+#endif
+
   cls = (*j->env)->FindClass(j->env, "java/lang/Object");
   j->class_object = (*j->env)->NewGlobalRef(j->env, cls);
   (*j->env)->DeleteLocalRef(j->env, cls);
@@ -3182,7 +3189,7 @@ static void f_create(INT32 args)
   f_thread_local(0);
   if(Pike_sp[-1].type == PIKE_T_OBJECT) {
     j->tl_env = Pike_sp[-1].u.object;
-    j->tl_env->refs ++;
+    add_ref(j->tl_env);
   }
   pop_n_elems(args+1);
 #else
@@ -3527,6 +3534,14 @@ PIKE_MODULE_INIT
     return;
 #endif /* __NT__ */
 
+#ifdef HAVE_IBMFINDDLL
+  {
+    /* Debug... */
+    extern char *ibmFindDLL(void);
+    fprintf(stderr, "ibmFindDLL(): \"%s\"\n", ibmFindDLL());
+  }
+#endif
+
   start_new_program();
   ADD_STORAGE(struct jobj_storage);
   pike_add_function("cast", f_jobj_cast, "function(string:mixed)", 0);
@@ -3671,7 +3686,7 @@ PIKE_MODULE_INIT
   pike_add_function("get_version", f_get_version, "function(:int)", 0);
   pike_add_function("find_class", f_find_class, "function(string:object)", 0);
   pike_add_function("define_class", f_define_class,
-		    "function(object,string:object)", 0);
+		    "function(string,object,string:object)", 0);
   pike_add_function("exception_check", f_exception_check, "function(:int)", 0);
   pike_add_function("exception_occurred", f_exception_occurred,
 		    "function(:object)", 0);
