@@ -1,6 +1,11 @@
 /*
- * $Id: mysql.c,v 1.45 2004/07/21 12:35:15 grubba Exp $
- *
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: mysql.c,v 1.81 2004/03/19 14:49:44 nilsson Exp $
+*/
+
+/*
  * SQL database functionality for Pike
  *
  * Henrik Grubbstrm 1996-12-21
@@ -19,7 +24,9 @@
  * Includes
  */
 
-#ifdef HAVE_WINSOCK_H
+#ifdef HAVE_WINSOCK2_H
+#include <winsock2.h>
+#elif defined(HAVE_WINSOCK_H)
 #include <winsock.h>
 #endif
 
@@ -53,15 +60,10 @@
 #endif /* HAVE_ERRMGS_H */
 
 
-/* dynamic_buffer.h contains a conflicting typedef for string
- * we don't use any dynamic buffers, so we have this work-around
- */
-#define DYNAMIC_BUFFER_H
-typedef struct dynamic_buffer_s dynamic_buffer;
-
 #endif /* HAVE_MYSQL */
 
 /* From the Pike-dist */
+#include "module.h"
 #include "svalue.h"
 #include "object.h"
 #include "stralloc.h"
@@ -73,6 +75,8 @@ typedef struct dynamic_buffer_s dynamic_buffer;
 #include "operators.h"
 #include "builtin_functions.h"
 #include "fd_control.h"
+#include "mapping.h"
+#include "bignum.h"
 
 /* System includes */
 #ifdef HAVE_STRING_H
@@ -82,7 +86,7 @@ typedef struct dynamic_buffer_s dynamic_buffer;
 #include <memory.h>
 #endif
 
-#include "module_magic.h"
+#define sp Pike_sp
 
 #ifdef HAVE_MYSQL
 
@@ -93,17 +97,17 @@ typedef struct dynamic_buffer_s dynamic_buffer;
  * Globals
  */
 
-RCSID("$Id: mysql.c,v 1.45 2004/07/21 12:35:15 grubba Exp $");
+RCSID("$Id: mysql.c,v 1.81 2004/03/19 14:49:44 nilsson Exp $");
 
 /*! @module Mysql
  *!
  *! This module enables access to the Mysql database from within Pike.
- *! Use @[Sql.sql] for general database access.
+ *! Use @[Sql.Sql] for general database access.
  *!
  *! Mysql is available from @url{http://www.mysql.com@}.
  *!
  *! @seealso
- *!  @[Mysql.mysql], @[Mysql.mysql_result], @[Sql.sql]
+ *!  @[Mysql.mysql], @[Mysql.mysql_result], @[Sql.Sql]
  */
 
 /*! @class mysql
@@ -115,7 +119,7 @@ RCSID("$Id: mysql.c,v 1.45 2004/07/21 12:35:15 grubba Exp $");
  *! Mysql is available from @url{http://www.mysql.com@}.
  *!
  *! @seealso
- *!   @[Mysql.result], @[Sql.sql]
+ *!   @[Mysql.mysql_result], @[Sql.Sql]
  */
 
 struct program *mysql_program = NULL;
@@ -154,6 +158,20 @@ static MUTEX_T stupid_port_lock;
 #define MYSQL_DISALLOW()
 #endif /* _REENTRANT */
 
+#define CHECK_8BIT_NONBINARY_STRING(FUNC, ARG) do {			\
+    if (sp[ARG-1-args].type != T_STRING ||				\
+	sp[ARG-1-args].u.string->size_shift ||				\
+	((ptrdiff_t) strlen (sp[ARG-1-args].u.string->str) !=		\
+	 sp[ARG-1-args].u.string->len))					\
+      SIMPLE_BAD_ARG_ERROR (FUNC, ARG, "string (nonbinary 8bit)");	\
+  } while (0)
+
+#define CHECK_8BIT_STRING(FUNC, ARG) do {				\
+    if (sp[ARG-1-args].type != T_STRING ||				\
+	sp[ARG-1-args].u.string->size_shift)				\
+      SIMPLE_BAD_ARG_ERROR (FUNC, ARG, "string (8bit)");	\
+  } while (0)
+
 
 /*
  * Functions
@@ -211,40 +229,84 @@ static void exit_mysql_struct(struct object *o)
 
   DESTROY_MYSQL_LOCK();
 }
+void pike_mysql_set_ssl(struct mapping *options) {
+
+#ifdef HAVE_MYSQL_SSL
+    char *ssl_key = NULL;
+    char *ssl_cert = NULL;
+    char *ssl_ca = NULL;
+    char *ssl_capath = NULL;
+    char *ssl_cipher = NULL;
+    struct svalue *val = NULL;
+
+    if ((val = simple_mapping_string_lookup(options, "ssl_key")) &&
+	(val->type == T_STRING) &&
+	(!val->u.string->size_shift))
+      ssl_key = val->u.string->str;
+
+    if ((val = simple_mapping_string_lookup(options, "ssl_cert")) &&
+	(val->type == T_STRING) &&
+	(!val->u.string->size_shift))
+      ssl_cert = val->u.string->str;
+
+    if ((val = simple_mapping_string_lookup(options, "ssl_ca")) &&
+	(val->type == T_STRING) &&
+	(!val->u.string->size_shift))
+      ssl_ca = val->u.string->str;
+
+    if ((val = simple_mapping_string_lookup(options, "ssl_capath")) &&
+	(val->type == T_STRING) &&
+	(!val->u.string->size_shift))
+      ssl_capath = val->u.string->str;
+
+    if ((val = simple_mapping_string_lookup(options, "ssl_cipher")) &&
+	(val->type == T_STRING) &&
+	(!val->u.string->size_shift))
+      ssl_cipher = val->u.string->str;
+
+    if (ssl_key || ssl_cert || ssl_ca || ssl_capath || ssl_cipher)
+      mysql_ssl_set(PIKE_MYSQL->mysql,
+		    ssl_key,
+		    ssl_cert,
+		    ssl_ca,
+		    ssl_capath,
+		    ssl_cipher);
+#endif /* HAVE_MYSQL_SSL */
+}
 
 static void pike_mysql_set_options(struct mapping *options)
 {
   struct svalue *val;
 
 #ifdef HAVE_MYSQL_OPTIONS
-#ifdef MYSQL_READ_DEFAULT_FILE
+#ifdef HAVE_MYSQL_READ_DEFAULT_FILE
   if ((val = simple_mapping_string_lookup(options, "mysql_config_file")) &&
       (val->type == T_STRING) && (!val->u.string->size_shift)) {
     mysql_options(PIKE_MYSQL->mysql, MYSQL_READ_DEFAULT_FILE,
 		  val->u.string->str);
   }
 #endif /* MYSQL_READ_DEFAULT_FILE */
-#ifdef MYSQL_READ_DEFAULT_GROUP
+#ifdef HAVE_MYSQL_READ_DEFAULT_GROUP
   if ((val = simple_mapping_string_lookup(options, "mysql_group")) &&
       (val->type == T_STRING) && (!val->u.string->size_shift)) {
     mysql_options(PIKE_MYSQL->mysql, MYSQL_READ_DEFAULT_GROUP,
 		  val->u.string->str);
   }
 #endif /* MYSQL_READ_DEFAULT_GROUP */
-#ifdef MYSQL_INIT_COMMAND
+#ifdef HAVE_MYSQL_INIT_COMMAND
   if ((val = simple_mapping_string_lookup(options, "init_command")) &&
       (val->type == T_STRING) && (!val->u.string->size_shift)) {
     mysql_options(PIKE_MYSQL->mysql, MYSQL_INIT_COMMAND,
 		  val->u.string->str);
   }
 #endif /* MYSQL_INIT_COMMAND */
-#ifdef MYSQL_OPT_NAMED_PIPE
+#ifdef HAVE_MYSQL_OPT_NAMED_PIPE
   if ((val = simple_mapping_string_lookup(options, "mysql_named_pipe")) &&
       (val->type == T_INT) && (val->u.integer)) {
     mysql_options(PIKE_MYSQL->mysql, MYSQL_OPT_NAMED_PIPE, NULL);
   }
 #endif /* MYSQL_OPT_NAMED_PIPE */
-#ifdef MYSQL_OPT_CONNECT_TIMEOUT
+#ifdef HAVE_MYSQL_OPT_CONNECT_TIMEOUT
   if ((val = simple_mapping_string_lookup(options, "timeout")) &&
       (val->type == T_INT)) {
     unsigned int timeout = (unsigned int)val->u.integer;
@@ -252,13 +314,13 @@ static void pike_mysql_set_options(struct mapping *options)
 		  (char *)&timeout);
   }
 #endif /* MYSQL_OPT_CONNECT_TIMEOUT */
-#ifdef MYSQL_OPT_COMPRESS
+#ifdef HAVE_MYSQL_OPT_COMPRESS
   if ((val = simple_mapping_string_lookup(options, "compress")) &&
       (val->type == T_INT) && (val->u.integer)) {
     mysql_options(PIKE_MYSQL->mysql, MYSQL_OPT_COMPRESS, NULL);
   }
 #endif /* MYSQL_OPT_COMPRESS */
-#ifdef MYSQL_OPT_LOCAL_INFILE
+#ifdef HAVE_MYSQL_OPT_LOCAL_INFILE
   if ((val = simple_mapping_string_lookup(options, "mysql_local_infile")) &&
       (val->type == T_INT)) {
     unsigned int allowed = (unsigned int)val->u.integer;
@@ -271,14 +333,14 @@ static void pike_mysql_set_options(struct mapping *options)
 		  (char *)&allowed);    
   }
 #endif /* MYSQL_OPT_LOCAL_INFILE */
-#ifdef MYSQL_SET_CHARSET_DIR
+#ifdef HAVE_MYSQL_SET_CHARSET_DIR
   if ((val = simple_mapping_string_lookup(options, "mysql_charset_dir")) &&
       (val->type == T_STRING) && (!val->u.string->size_shift)) {
     mysql_options(PIKE_MYSQL->mysql, MYSQL_SET_CHARSET_DIR,
 		  val->u.string->str);
   }
 #endif /* MYSQL_SET_CHARSET_DIR */
-#ifdef MYSQL_SET_CHARSET_NAME
+#ifdef HAVE_MYSQL_SET_CHARSET_NAME
   if ((val = simple_mapping_string_lookup(options, "mysql_charset_name")) &&
       (val->type == T_STRING) && (!val->u.string->size_shift)) {
     mysql_options(PIKE_MYSQL->mysql, MYSQL_SET_CHARSET_NAME,
@@ -301,6 +363,8 @@ static void pike_mysql_reconnect(void)
   char *saved_unix_port = NULL;
   unsigned int port = 0;
   unsigned int saved_port = 0;
+  unsigned int options = 0;
+  struct svalue *val;
 
   if (PIKE_MYSQL->host) {
     hostptr = strdup(PIKE_MYSQL->host->str);
@@ -329,6 +393,12 @@ static void pike_mysql_reconnect(void)
   socket = PIKE_MYSQL->socket;
   PIKE_MYSQL->socket = NULL;
 
+  if ((val = simple_mapping_string_lookup(PIKE_MYSQL->options,
+					  "connect_options")) &&
+      (val->type == T_INT) && (val->u.integer)) {
+    options = (unsigned int)val->u.integer;
+  }
+
   MYSQL_ALLOW();
 
 #if defined(HAVE_MYSQL_PORT) || defined(HAVE_MYSQL_UNIX_PORT)
@@ -355,7 +425,7 @@ static void pike_mysql_reconnect(void)
 
 #ifdef HAVE_MYSQL_REAL_CONNECT
   socket = mysql_real_connect(mysql, host, user, password,
-                              NULL, port, portptr, 0);
+                              NULL, port, portptr, options);
 #else
   socket = mysql_connect(mysql, host, user, password);
 #endif /* HAVE_MYSQL_REAL_CONNECT */
@@ -436,8 +506,8 @@ static void pike_mysql_reconnect(void)
  *! done with this function.
  *!
  *! @param host
- *!   If you give no argument, or give @tt{""@} as @[host] it will connect with
- *!   a UNIX-domain socket, which can be a big performance gain.
+ *!   If you give no argument, or give @expr{""@} as @[host] it will connect
+ *!   with a UNIX-domain socket, which can be a big performance gain.
  *!
  *! @param options
  *!   This optional mapping can contain zero or more of the following
@@ -454,7 +524,7 @@ static void pike_mysql_reconnect(void)
  *!       Enable compressed protocol.
  *!
  *!     @member string "mysql_config_file"
- *!       Change config file from @tt{"my.cnf"@}.
+ *!       Change config file from @expr{"my.cnf"@}.
  *!
  *!     @member string "mysql_group"
  *!       Specify additional group to read from config file.
@@ -470,11 +540,38 @@ static void pike_mysql_reconnect(void)
  *!
  *!     @member string "mysql_charset_name"
  *!       Change charset name.
+ *!
+ *!     @member string "ssl_key"
+ *!       Path to SSL-key for use in SSL-communication.
+ *!
+ *!     @member string "ssl_cert"
+ *!       Path to SSL-cert for use in SSL-communication.
+ *!
+ *!     @member string "ssl_ca"
+ *!       Path to SSL-CA for use in SSL-communication.
+ *!
+ *!     @member string "ssl_capath"
+ *!       Path to SSL-CAPATH for use in SSL-communication.
+ *!
+ *!     @member string "ssl_cipher"
+ *!       FIXME
+ *!
+ *!     @member int "connect_options"
+ *!       Options used when connecting to the server. See mysql documentation
+ *!       for more information.
+ *!
  *!   @endmapping
  *!
  *! @note
  *!   Some options may not be implemented. Unimplemented options are
  *!   silently ignored.
+ *!
+ *!   To use SSL-connections, set the SSL-parameters correctly. They corespond
+ *!   to the parameters given to the mysql-client with the same name so make
+ *!   sure that the mysql-client works with SSL and set these parameters to
+ *!   the same values and everything should work. If SSL-options are loaded
+ *!   from a config-file, one may set the connect_options to include
+ *!   CLIENT_SSL.
  */
 static void f_create(INT32 args)
 {
@@ -486,40 +583,33 @@ static void f_create(INT32 args)
 #endif /* HAVE_MYSQL_OPTIONS */
 
   if (args >= 1) {
-    if (sp[-args].type != T_STRING) {
-      Pike_error("Bad argument 1 to mysql()\n");
-    }
+    CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->create", 1);
     if (sp[-args].u.string->len) {
       add_ref(PIKE_MYSQL->host = sp[-args].u.string);
     }
 
     if (args >= 2) {
-      if (sp[1-args].type != T_STRING) {
-	Pike_error("Bad argument 2 to mysql()\n");
-      }
+      CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->create", 2);
       if (sp[1-args].u.string->len) {
 	add_ref(PIKE_MYSQL->database = sp[1-args].u.string);
       }
       
       if (args >= 3) {
-	if (sp[2-args].type != T_STRING) {
-	  Pike_error("Bad argument 3 to mysql()\n");
-	}
+	CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->create", 3);
 	if (sp[2-args].u.string->len) {
 	  add_ref(PIKE_MYSQL->user = sp[2-args].u.string);
 	}
 	
 	if (args >= 4) {
-	  if (sp[3-args].type != T_STRING) {
-	    Pike_error("Bad argument 4 to mysql()\n");
-	  }
+	  CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->create", 4);
 	  if (sp[3-args].u.string->len) {
 	    add_ref(PIKE_MYSQL->password = sp[3-args].u.string);
 	  }
 	  if (args >= 5) {
 	    if (sp[4-args].type != T_MAPPING) {
-	      Pike_error("Bad argument 5 to mysql()\n");
+	      SIMPLE_BAD_ARG_ERROR ("Mysql.mysql->create", 5, "mapping(string:mixed)");
 	    }
+	    add_ref(PIKE_MYSQL->options = sp[4-args].u.mapping);
 	    pike_mysql_set_options(sp[4-args].u.mapping);
 	  }
 	}
@@ -529,9 +619,71 @@ static void f_create(INT32 args)
 
   pop_n_elems(args);
 
+  pike_mysql_set_ssl(PIKE_MYSQL->options);
+
   pike_mysql_reconnect();
 }
 
+/*! @decl string _sprintf(int type, void|mapping flags)
+ */
+static void mysql__sprintf(INT32 args)
+{
+  INT_TYPE type;
+  struct mapping * flags;
+
+  if(args < 1)
+    SIMPLE_TOO_FEW_ARGS_ERROR("_sprintf",2);
+  if(sp[-args].type!=T_INT)
+    SIMPLE_BAD_ARG_ERROR("_sprintf",0,"integer");
+
+  type = sp[-args].u.integer;
+  pop_n_elems(args);
+  switch( type )
+  {
+    case 'O':
+    {
+      struct pike_string *res;
+      MYSQL *socket;
+      const char *info;
+
+      if(!PIKE_MYSQL->socket)
+	pike_mysql_reconnect();
+      socket = PIKE_MYSQL->socket;
+
+      MYSQL_ALLOW();
+      info = mysql_get_host_info(socket);
+      MYSQL_DISALLOW();
+
+      push_text("mysql(/* %s %s*/)");
+      push_text(info);
+#ifdef HAVE_MYSQL_SSL
+      if (PIKE_MYSQL->mysql->options.use_ssl) {
+	push_text("using SSL");
+      }
+      else push_text("");
+#else
+      push_text("");
+#endif /* HAVE_MYSQL_SSL */
+
+      f_sprintf(3);
+
+      res = Pike_sp[-1].u.string;
+      Pike_sp--;
+      push_string(res);
+      return;
+    }
+
+    case 't':
+    {
+      struct pike_string * res = make_shared_binary_string("mysql", 5);
+      push_string(res);
+      return;
+    }
+  }
+  push_int( 0 );
+  Pike_sp[-1].subtype = 1;
+}
+
 /*! @decl int affected_rows()
  *!
  *! Returns the number of rows affected by the last query.
@@ -539,7 +691,7 @@ static void f_create(INT32 args)
 static void f_affected_rows(INT32 args)
 {
   MYSQL *socket;
-  int count;
+  INT64 count;
 
   if (!PIKE_MYSQL->socket) {
     pike_mysql_reconnect();
@@ -551,7 +703,7 @@ static void f_affected_rows(INT32 args)
   count = mysql_affected_rows(socket);
   MYSQL_DISALLOW();
 
-  push_int(count);
+  push_int64(count);
 }
 
 /*! @decl int insert_id()
@@ -562,7 +714,7 @@ static void f_affected_rows(INT32 args)
 static void f_insert_id(INT32 args)
 {
   MYSQL *socket;
-  int id;
+  INT64 id;
 
   if (!PIKE_MYSQL->socket) {
     pike_mysql_reconnect();
@@ -575,19 +727,19 @@ static void f_insert_id(INT32 args)
   id = mysql_insert_id(socket);
   MYSQL_DISALLOW();
 
-  push_int(id);
+  push_int64(id);
 }
 
 /*! @decl string error()
  *!
  *! Returns a string describing the last error from the Mysql-server.
  *!
- *! Returns @tt{0@} (zero) if there was no error.
+ *! Returns @expr{0@} (zero) if there was no error.
  */
 static void f_error(INT32 args)
 {
   MYSQL *socket;
-  char *error_msg;
+  const char *error_msg;
 
   if (!PIKE_MYSQL->socket) {
     pike_mysql_reconnect();
@@ -627,11 +779,9 @@ static void f_select_db(INT32 args)
   int tmp = -1;
 
   if (!args) {
-    Pike_error("Too few arguments to mysql->select_db()\n");
-  }
-  if (sp[-args].type != T_STRING) {
-    Pike_error("Bad argument 1 to mysql->select_db()\n");
+    SIMPLE_TOO_FEW_ARGS_ERROR ("Mysql.mysql->select_db", 1);
   }
+  CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->select_db", 1);
 
   database = sp[-args].u.string->str;
 
@@ -656,13 +806,13 @@ static void f_select_db(INT32 args)
   }
 
   if (tmp) {
-    char *err;
+    const char *err;
 
     MYSQL_ALLOW();
     err = mysql_error(socket);
     MYSQL_DISALLOW();
 
-    Pike_error("mysql->select_db(): Couldn't select database \"%s\" (%s)\n",
+    Pike_error("Mysql.mysql->select_db(): Couldn't select database \"%s\" (%s)\n",
 	  sp[-args].u.string->str, err);
   }
   if (PIKE_MYSQL->database) {
@@ -680,7 +830,7 @@ static void f_select_db(INT32 args)
  *! This function sends the SQL query @[query] to the Mysql-server. The result
  *! of the query is returned as a @[Mysql.mysql_result] object.
  *!
- *! Returns @tt{0@} (zero) if the query didn't return any result
+ *! Returns @expr{0@} (zero) if the query didn't return any result
  *! (e.g. @tt{INSERT@} or similar).
  *!
  *! @seealso
@@ -695,11 +845,13 @@ static void f_big_query(INT32 args)
   int tmp = -1;
 
   if (!args) {
-    Pike_error("Too few arguments to mysql->big_query()\n");
-  }
-  if (sp[-args].type != T_STRING) {
-    Pike_error("Bad argument 1 to mysql->big_query()\n");
+    SIMPLE_TOO_FEW_ARGS_ERROR ("Mysql.mysql->big_query", 1);
   }
+#ifdef HAVE_MYSQL_REAL_QUERY
+  CHECK_8BIT_STRING ("Mysql.mysql->big_query", 1);
+#else
+  CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->big_query", 1);
+#endif
 
   query = sp[-args].u.string->str;
   qlen = sp[-args].u.string->len;
@@ -753,17 +905,17 @@ static void f_big_query(INT32 args)
   }
 
   if (tmp) {
-    char *err;
+    const char *err;
 
     MYSQL_ALLOW();
     err = mysql_error(socket);
     MYSQL_DISALLOW();
 
     if (sp[-args].u.string->len <= 512) {
-      Pike_error("mysql->big_query(): Query \"%s\" failed (%s)\n",
+      Pike_error("Mysql.mysql->big_query(): Query \"%s\" failed (%s)\n",
 	    sp[-args].u.string->str, err);
     } else {
-      Pike_error("mysql->big_query(): Query failed (%s)\n", err);
+      Pike_error("Mysql.mysql->big_query(): Query failed (%s)\n", err);
     }
   }
 
@@ -787,7 +939,8 @@ static void f_big_query(INT32 args)
     MYSQL_DISALLOW();
 
     if (err) {
-      Pike_error("mysql->big_query(): Couldn't create result for query\n");
+      Pike_error("Mysql.mysql->big_query(): Couldn't create result for query (%s)\n",
+		 mysql_error(socket));
     }
     /* query was INSERT or similar - return 0 */
 
@@ -805,7 +958,7 @@ static void f_big_query(INT32 args)
     if ((!(res = (struct precompiled_mysql_result *)
 	   get_storage(o, mysql_result_program))) || res->result) {
       mysql_free_result(result);
-      Pike_error("mysql->big_query(): Bad mysql result object!\n");
+      Pike_error("Mysql.mysql->big_query(): Bad mysql result object!\n");
     }
     res->result = result;
   }
@@ -820,6 +973,7 @@ static void f_big_query(INT32 args)
  *! @seealso
  *!   @[select_db()], @[drop_db()]
  */
+#ifdef USE_OLD_FUNCTIONS
 static void f_create_db(INT32 args)
 {
   MYSQL *socket = PIKE_MYSQL->socket;
@@ -827,18 +981,16 @@ static void f_create_db(INT32 args)
   int tmp = -1;
 
   if (!args) {
-    Pike_error("Too few arguments to mysql->create_db()\n");
-  }
-  if (sp[-args].type != T_STRING) {
-    Pike_error("Bad argument 1 to mysql->create_db()\n");
+    SIMPLE_TOO_FEW_ARGS_ERROR ("Mysql.mysql->create_db", 1);
   }
+  CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->create_db", 1);
   if (sp[-args].u.string->len > 127) {
     if (sp[-args].u.string->len < 1024) {
       Pike_error("Database name \"%s\" is too long (max 127 characters)\n",
 	    sp[-args].u.string->str);
     } else {
       Pike_error("Database name (length %ld) is too long (max 127 characters)\n",
-		 (long)sp[-args].u.string->len);
+		 DO_NOT_WARN((long)sp[-args].u.string->len));
     }
   }
   database = sp[-args].u.string->str;
@@ -863,8 +1015,8 @@ static void f_create_db(INT32 args)
   }
 
   if (tmp) {
-    Pike_error("mysql->create_db(): Creation of database \"%s\" failed\n",
-	       sp[-args].u.string->str);
+    Pike_error("Mysql.mysql->create_db(): Creation of database \"%s\" failed\n",
+	  sp[-args].u.string->str);
   }
 
   pop_n_elems(args);
@@ -886,18 +1038,16 @@ static void f_drop_db(INT32 args)
   int tmp = -1;
 
   if (!args) {
-    Pike_error("Too few arguments to mysql->drop_db()\n");
-  }
-  if (sp[-args].type != T_STRING) {
-    Pike_error("Bad argument 1 to mysql->drop_db()\n");
+    SIMPLE_TOO_FEW_ARGS_ERROR ("Mysql.mysql->drop_db", 1);
   }
+  CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->drop_db", 1);
   if (sp[-args].u.string->len > 127) {
     if (sp[-args].u.string->len < 1024) {
       Pike_error("Database name \"%s\" is too long (max 127 characters)\n",
 	    sp[-args].u.string->str);
     } else {
       Pike_error("Database name (length %ld) is too long (max 127 characters)\n",
-		 (long)sp[-args].u.string->len);
+		 DO_NOT_WARN((long)sp[-args].u.string->len));
     }
   }
   database = sp[-args].u.string->str;
@@ -923,12 +1073,13 @@ static void f_drop_db(INT32 args)
   }    
 
   if (tmp) {
-    Pike_error("mysql->drop_db(): Drop of database \"%s\" failed\n",
+    Pike_error("Mysql.mysql->drop_db(): Drop of database \"%s\" failed\n",
 	  sp[-args].u.string->str);
   }
 
   pop_n_elems(args);
 }
+#endif
 
 /*! @decl void shutdown()
  *!
@@ -965,7 +1116,7 @@ static void f_shutdown(INT32 args)
   }
 
   if (tmp) {
-    Pike_error("mysql->shutdown(): Shutdown failed\n");
+    Pike_error("Mysql.mysql->shutdown(): Shutdown failed\n");
   }
 
   pop_n_elems(args);
@@ -1006,7 +1157,7 @@ static void f_reload(INT32 args)
   }
 
   if (tmp) {
-    Pike_error("mysql->reload(): Reload failed\n");
+    Pike_error("Mysql.mysql->reload(): Reload failed\n");
   }
 
   pop_n_elems(args);
@@ -1024,7 +1175,7 @@ static void f_reload(INT32 args)
 static void f_statistics(INT32 args)
 {
   MYSQL *socket = PIKE_MYSQL->socket;
-  char *stats;
+  const char *stats;
 
   if (!socket) {
     pike_mysql_reconnect();
@@ -1052,7 +1203,7 @@ static void f_statistics(INT32 args)
 static void f_server_info(INT32 args)
 {
   MYSQL *socket = PIKE_MYSQL->socket;
-  char *info;
+  const char *info;
 
   if (!socket) {
     pike_mysql_reconnect();
@@ -1083,7 +1234,7 @@ static void f_server_info(INT32 args)
 static void f_host_info(INT32 args)
 {
   MYSQL *socket;
-  char *info;
+  const char *info;
 
   if (!PIKE_MYSQL->socket) {
     pike_mysql_reconnect();
@@ -1143,7 +1294,7 @@ static void f_protocol_info(INT32 args)
  *!
  *! @seealso
  *!   @[list_tables()], @[list_fields()], @[list_processes()],
- *!   @[mysql_result]
+ *!   @[Mysql.mysql_result]
  */
 static void f_list_dbs(INT32 args)
 {
@@ -1152,16 +1303,14 @@ static void f_list_dbs(INT32 args)
   char *wild = NULL;
 
   if (args) {
-    if (sp[-args].type != T_STRING) {
-      Pike_error("Bad argument 1 to mysql->list_dbs()\n");
-    }
+    CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->list_dbs", 1);
     if (sp[-args].u.string->len > 80) {
       if (sp[-args].u.string->len < 1024) {
 	Pike_error("Wildcard \"%s\" is too long (max 80 characters)\n",
 	      sp[-args].u.string->str);
       } else {
 	Pike_error("Wildcard (length %ld) is too long (max 80 characters)\n",
-		   (long)sp[-args].u.string->len);
+		   DO_NOT_WARN((long)sp[-args].u.string->len));
       }
     }
     wild = sp[-args].u.string->str;
@@ -1188,7 +1337,7 @@ static void f_list_dbs(INT32 args)
   }
 
   if (!result) {
-    char *err;
+    const char *err;
 
     MYSQL_ALLOW();
 
@@ -1196,7 +1345,7 @@ static void f_list_dbs(INT32 args)
 
     MYSQL_DISALLOW();
 
-    Pike_error("mysql->list_dbs(): Cannot list databases: %s\n", err);
+    Pike_error("Mysql.mysql->list_dbs(): Cannot list databases: %s\n", err);
   }
 
   pop_n_elems(args);
@@ -1214,7 +1363,7 @@ static void f_list_dbs(INT32 args)
     if ((!(res = (struct precompiled_mysql_result *)
 	   get_storage(o, mysql_result_program))) || res->result) {
       mysql_free_result(result);
-      Pike_error("mysql->list_dbs(): Bad mysql result object!\n");
+      Pike_error("Mysql.mysql->list_dbs(): Bad mysql result object!\n");
     }
     res->result = result;
   }
@@ -1231,7 +1380,7 @@ static void f_list_dbs(INT32 args)
  *!
  *! @seealso
  *!   @[list_dbs()], @[list_fields()], @[list_processes()],
- *!   @[mysql_result]
+ *!   @[Mysql.mysql_result]
  */
 static void f_list_tables(INT32 args)
 {
@@ -1240,16 +1389,14 @@ static void f_list_tables(INT32 args)
   char *wild = NULL;
 
   if (args) {
-    if (sp[-args].type != T_STRING) {
-      Pike_error("Bad argument 1 to mysql->list_tables()\n");
-    }
+    CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->list_tables", 1);
     if (sp[-args].u.string->len > 80) {
       if (sp[-args].u.string->len < 1024) {
 	Pike_error("Wildcard \"%s\" is too long (max 80 characters)\n",
 	      sp[-args].u.string->str);
       } else {
 	Pike_error("Wildcard (length %ld) is too long (max 80 characters)\n",
-		   (long)sp[-args].u.string->len);
+		   DO_NOT_WARN((long)sp[-args].u.string->len));
       }
     }
     wild = sp[-args].u.string->str;
@@ -1276,7 +1423,7 @@ static void f_list_tables(INT32 args)
   }
 
   if (!result) {
-    char *err;
+    const char *err;
 
     MYSQL_ALLOW();
 
@@ -1284,7 +1431,7 @@ static void f_list_tables(INT32 args)
 
     MYSQL_DISALLOW();
 
-    Pike_error("mysql->list_tables(): Cannot list databases: %s\n", err);
+    Pike_error("Mysql.mysql->list_tables(): Cannot list databases: %s\n", err);
   }
 
   pop_n_elems(args);
@@ -1302,7 +1449,7 @@ static void f_list_tables(INT32 args)
     if ((!(res = (struct precompiled_mysql_result *)
 	   get_storage(o, mysql_result_program))) || res->result) {
       mysql_free_result(result);
-      Pike_error("mysql->list_tables(): Bad mysql result object!\n");
+      Pike_error("Mysql.mysql->list_tables(): Bad mysql result object!\n");
     }
     res->result = result;
   }
@@ -1339,10 +1486,12 @@ static void f_list_tables(INT32 args)
  *! @endmapping
  *!
  *! The type of the field can be any of:
- *!   @tt{"decimal"@}, @tt{"char"@}, @tt{"short"@}, @tt{"long"@},
- *!   @tt{"float"@}, @tt{"double"@}, @tt{"null"@}, @tt{"time"@},
- *!   @tt{"longlong"@}, @tt{"int24"@}, @tt{"tiny blob"@}, @tt{"medium blob"@},
- *!   @tt{"long blob"@}, @tt{"var string"@}, @tt{"string"@} or @tt{"unknown"@}.
+ *!   @expr{"decimal"@}, @expr{"char"@}, @expr{"short"@}, @expr{"long"@},
+ *!   @expr{"float"@}, @expr{"double"@}, @expr{"null"@}, @expr{"time"@},
+ *!   @expr{"longlong"@}, @expr{"int24"@}, @expr{"tiny blob"@},
+ *!   @expr{"medium blob"@},
+ *!   @expr{"long blob"@}, @expr{"var string"@}, @expr{"string"@} or
+ *!   @expr{"unknown"@}.
  *!
  *! The flags multiset can contain any of:
  *! @multiset
@@ -1359,7 +1508,8 @@ static void f_list_tables(INT32 args)
  *!   @tt{show fields in 'table' like "wild"@}.
  *!
  *! @seealso
- *!   @[list_dbs()], @[list_tables()], @[list_processes()], @[mysql_result.fetch_field()]
+ *!   @[list_dbs()], @[list_tables()], @[list_processes()],
+ *!   @[Mysql.mysql_result.fetch_field()]
  */
 static void f_list_fields(INT32 args)
 {
@@ -1371,26 +1521,22 @@ static void f_list_fields(INT32 args)
   char *wild = NULL;
 
   if (!args) {
-    Pike_error("Too few arguments to mysql->list_fields()\n");
-  }
-  
-  if (sp[-args].type != T_STRING) {
-    Pike_error("Bad argument 1 to mysql->list_fields()\n");
+    SIMPLE_TOO_FEW_ARGS_ERROR ("Mysql.mysql->list_fields", 1);
   }
+
+  CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->list_fields", 1);
   if (sp[-args].u.string->len > 125) {
     if (sp[-args].u.string->len < 1024) {
       Pike_error("Table name \"%s\" is too long (max 125 characters)\n",
 	    sp[-args].u.string->str);
     } else {
       Pike_error("Table name (length %ld) is too long (max 125 characters)\n",
-		 (long)sp[-args].u.string->len);
+		 DO_NOT_WARN((long)sp[-args].u.string->len));
     }
   }
   table = sp[-args].u.string->str;
   if (args > 1) {
-    if (sp[-args+1].type != T_STRING) {
-      Pike_error("Bad argument 2 to mysql->list_fields()\n");
-    }
+    CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->list_fields", 2);
     if (sp[-args+1].u.string->len + sp[-args].u.string->len > 125) {
       /* The length of the table name has already been checked. */
       if (sp[-args+1].u.string->len < 1024) {
@@ -1400,7 +1546,7 @@ static void f_list_fields(INT32 args)
       } else {
 	Pike_error("Wildcard (length %ld) + table name \"%s\" is too long "
 		   "(max 125 characters)\n",
-		   (long)sp[-args+1].u.string->len,
+		   DO_NOT_WARN((long)sp[-args+1].u.string->len),
 		   sp[-args].u.string->str);
       }
     }
@@ -1428,7 +1574,7 @@ static void f_list_fields(INT32 args)
   }
 
   if (!result) {
-    char *err;
+    const char *err;
 
     MYSQL_ALLOW();
 
@@ -1436,7 +1582,7 @@ static void f_list_fields(INT32 args)
 
     MYSQL_DISALLOW();
 
-    Pike_error("mysql->list_fields(): Cannot list databases: %s\n", err);
+    Pike_error("Mysql.mysql->list_fields(): Cannot list databases: %s\n", err);
   }
 
   pop_n_elems(args);
@@ -1488,7 +1634,7 @@ static void f_list_processes(INT32 args)
   }
 
   if (!result) {
-    char *err;
+    const char *err;
 
     MYSQL_ALLOW();
 
@@ -1496,7 +1642,7 @@ static void f_list_processes(INT32 args)
 
     MYSQL_DISALLOW();
 
-    Pike_error("mysql->list_processes(): Cannot list databases: %s\n", err);
+    Pike_error("Mysql.mysql->list_processes(): Cannot list databases: %s\n", err);
   }
 
   {
@@ -1512,7 +1658,7 @@ static void f_list_processes(INT32 args)
     if ((!(res = (struct precompiled_mysql_result *)
 	   get_storage(o, mysql_result_program))) || res->result) {
       mysql_free_result(result);
-      Pike_error("mysql->list_processes(): Bad mysql result object!\n");
+      Pike_error("Mysql.mysql->list_processes(): Bad mysql result object!\n");
     }
     res->result = result;
   }
@@ -1526,7 +1672,7 @@ static void f_list_processes(INT32 args)
  *! and retreived with this version of the mysql-module.
  *!
  *! Usually, there is no problem storing binary data in mysql-tables,
- *! but data containing @tt{'\0'@} (NUL) couldn't be fetched with old
+ *! but data containing @expr{'\0'@} (NUL) couldn't be fetched with old
  *! versions (prior to 3.20.5) of the mysql-library.
  */
 static void f_binary_data(INT32 args)
@@ -1552,7 +1698,7 @@ static void f_binary_data(INT32 args)
  */
 
 
-void pike_module_init(void)
+PIKE_MODULE_INIT
 {
 #ifdef HAVE_MYSQL
   /*
@@ -1578,7 +1724,10 @@ void pike_module_init(void)
   /* function(void:int|string) */
   ADD_FUNCTION("error", f_error,tFunc(tVoid,tOr(tInt,tStr)), ID_PUBLIC);
   /* function(string|void, string|void, string|void, string|void:void) */
-  ADD_FUNCTION("create", f_create,tFunc(tOr(tStr,tVoid) tOr(tStr,tVoid) tOr(tStr,tVoid) tOr(tStr,tVoid),tVoid), ID_PUBLIC);
+  ADD_FUNCTION("create", f_create,tFunc(tOr(tStr,tVoid) tOr(tStr,tVoid) tOr(tStr,tVoid) tOr(tStr,tVoid) tOr(tMapping, tVoid),tVoid), ID_PUBLIC);
+  /* function(int, void|mapping:string) */
+  ADD_FUNCTION("_sprintf",mysql__sprintf,
+	       tFunc(tInt tOr(tVoid,tMapping),tString),0);
   /* function(void:int) */
   ADD_FUNCTION("affected_rows", f_affected_rows,tFunc(tVoid,tInt), ID_PUBLIC);
   /* function(void:int) */
@@ -1587,10 +1736,14 @@ void pike_module_init(void)
   ADD_FUNCTION("select_db", f_select_db,tFunc(tStr,tVoid), ID_PUBLIC);
   /* function(string:int|object) */
   ADD_FUNCTION("big_query", f_big_query,tFunc(tStr,tOr(tInt,tObj)), ID_PUBLIC);
+#ifdef USE_OLD_FUNCTIONS
   /* function(string:void) */
   ADD_FUNCTION("create_db", f_create_db,tFunc(tStr,tVoid), ID_PUBLIC);
   /* function(string:void) */
   ADD_FUNCTION("drop_db", f_drop_db,tFunc(tStr,tVoid), ID_PUBLIC);
+#else
+  add_integer_constant( "MYSQL_NO_ADD_DROP_DB", 1, 0 );
+#endif
   /* function(void:void) */
   ADD_FUNCTION("shutdown", f_shutdown,tFunc(tVoid,tVoid), ID_PUBLIC);
   /* function(void:void) */
@@ -1615,6 +1768,17 @@ void pike_module_init(void)
   /* function(void:int) */
   ADD_FUNCTION("binary_data", f_binary_data,tFunc(tVoid,tInt), ID_PUBLIC);
 
+  add_integer_constant( "CLIENT_COMPRESS", CLIENT_COMPRESS, 0);
+  add_integer_constant( "CLIENT_FOUND_ROWS", CLIENT_FOUND_ROWS, 0);
+  add_integer_constant( "CLIENT_IGNORE_SPACE", CLIENT_IGNORE_SPACE, 0);
+  add_integer_constant( "CLIENT_INTERACTIVE", CLIENT_INTERACTIVE, 0);
+  add_integer_constant( "CLIENT_NO_SCHEMA", CLIENT_NO_SCHEMA, 0);
+  add_integer_constant( "CLIENT_ODBC", CLIENT_ODBC, 0);
+
+#ifdef HAVE_MYSQL_SSL
+  add_integer_constant( "CLIENT_SSL", CLIENT_SSL, 0);
+#endif
+
   set_init_callback(init_mysql_struct);
   set_exit_callback(exit_mysql_struct);
 
@@ -1629,7 +1793,7 @@ void pike_module_init(void)
 #endif /* HAVE_MYSQL */
 }
 
-void pike_module_exit(void)
+PIKE_MODULE_EXIT
 {
 #ifdef HAVE_MYSQL
   exit_mysql_res();
@@ -1644,4 +1808,3 @@ void pike_module_exit(void)
   }
 #endif /* HAVE_MYSQL */
 }
-
