@@ -1,6 +1,11 @@
 /*
- * $Id: odbc.c,v 1.26 2003/12/23 17:31:43 grubba Exp $
- *
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: odbc.c,v 1.35 2003/12/23 17:30:17 grubba Exp $
+*/
+
+/*
  * Pike interface to ODBC compliant databases.
  *
  * Henrik Grubbstrm
@@ -16,7 +21,7 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-RCSID("$Id: odbc.c,v 1.26 2003/12/23 17:31:43 grubba Exp $");
+RCSID("$Id: odbc.c,v 1.35 2003/12/23 17:30:17 grubba Exp $");
 
 #include "interpret.h"
 #include "object.h"
@@ -33,8 +38,9 @@ RCSID("$Id: odbc.c,v 1.26 2003/12/23 17:31:43 grubba Exp $");
 
 #include "precompiled_odbc.h"
 
-/* must be included last */
-#include "module_magic.h"
+
+#define sp Pike_sp
+#define fp Pike_fp
 
 #ifdef HAVE_ODBC
 
@@ -44,7 +50,7 @@ RCSID("$Id: odbc.c,v 1.26 2003/12/23 17:31:43 grubba Exp $");
 
 struct program *odbc_program = NULL;
 
-HENV odbc_henv = SQL_NULL_HENV;
+SQLHENV odbc_henv = SQL_NULL_HENV;
 
 /*
  * Functions
@@ -55,7 +61,7 @@ HENV odbc_henv = SQL_NULL_HENV;
  */
 
 void odbc_error(const char *fun, const char *msg,
-		struct precompiled_odbc *odbc, HSTMT hstmt,
+		struct precompiled_odbc *odbc, SQLHSTMT hstmt,
 		RETCODE code, void (*clean)(void));
 
 static INLINE void odbc_check_error(const char *fun, const char *msg,
@@ -67,7 +73,7 @@ static INLINE void odbc_check_error(const char *fun, const char *msg,
 }
 
 void odbc_error(const char *fun, const char *msg,
-		struct precompiled_odbc *odbc, HSTMT hstmt,
+		struct precompiled_odbc *odbc, SQLHSTMT hstmt,
 		RETCODE code, void (*clean)(void))
 {
   RETCODE _code;
@@ -149,7 +155,7 @@ static void init_odbc_struct(struct object *o)
 
 static void exit_odbc_struct(struct object *o)
 {
-  HDBC hdbc = PIKE_ODBC->hdbc;
+  SQLHDBC hdbc = PIKE_ODBC->hdbc;
 
   if (hdbc != SQL_NULL_HDBC) {
     if (PIKE_ODBC->flags & PIKE_ODBC_CONNECTED) {
@@ -315,7 +321,7 @@ static void f_big_query(INT32 args)
   }
 #ifdef PIKE_DEBUG
   if (sp != save_sp) {
-    fatal("Stack error in odbc->big_query().\n");
+    Pike_fatal("Stack error in odbc->big_query().\n");
   }
 #endif /* PIKE_DEBUG */
 }
@@ -375,7 +381,7 @@ static void f_list_tables(INT32 args)
   }
 #ifdef PIKE_DEBUG
   if (sp != save_sp) {
-    fatal("Stack error in odbc->list_tables().\n");
+    Pike_fatal("Stack error in odbc->list_tables().\n");
   }
 #endif /* PIKE_DEBUG */
 }
@@ -430,7 +436,7 @@ static void f_list_dbs(INT32 args)
  * Module linkage
  */
 
-void pike_module_init(void)
+PIKE_MODULE_INIT
 {
 #ifdef HAVE_ODBC
   RETCODE err = SQLAllocEnv(&odbc_henv);
@@ -455,6 +461,8 @@ void pike_module_init(void)
   ADD_FUNCTION("big_query", f_big_query,tFunc(tStr,tOr(tInt,tObj)), ID_PUBLIC);
   /* function(void:int) */
   ADD_FUNCTION("affected_rows", f_affected_rows,tFunc(tVoid,tInt), ID_PUBLIC);
+  /* function(void|string:object) */
+  ADD_FUNCTION("list_tables", f_list_tables,tFunc(tOr(tVoid,tStr),tObj), ID_PUBLIC);
   /* NOOP's: */
   /* function(string:void) */
   ADD_FUNCTION("create_db", f_create_db,tFunc(tStr,tVoid), ID_PUBLIC);
@@ -464,8 +472,6 @@ void pike_module_init(void)
   ADD_FUNCTION("shutdown", f_shutdown,tFunc(tVoid,tVoid), ID_PUBLIC);
   /* function(void:void) */
   ADD_FUNCTION("reload", f_reload,tFunc(tVoid,tVoid), ID_PUBLIC);
-  /* function(void|string:object) */
-  ADD_FUNCTION("list_tables", f_list_tables,tFunc(tOr(tVoid,tStr),tObj), ID_PUBLIC);
 #if 0
   /* function(void:int) */
   ADD_FUNCTION("insert_id", f_insert_id,tFunc(tVoid,tInt), ID_PUBLIC);
@@ -500,7 +506,7 @@ void pike_module_init(void)
 #endif /* HAVE_ODBC */
 }
 
-void pike_module_exit(void)
+PIKE_MODULE_EXIT
 {
 #ifdef HAVE_ODBC
   exit_odbc_res();
@@ -520,4 +526,3 @@ void pike_module_exit(void)
   }
 #endif /* HAVE_ODBC */
 }
- 
