@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: oracle.c,v 1.75 2002/11/28 23:45:49 marcus Exp $
+|| $Id: oracle.c,v 1.82 2004/04/15 00:09:47 nilsson Exp $
 */
 
 /*
@@ -35,7 +35,6 @@
 #include "mapping.h"
 #include "multiset.h"
 #include "builtin_functions.h"
-#include "opcodes.h"
 #include "pike_macros.h"
 #include "version.h"
 
@@ -54,7 +53,7 @@
 
 #include <math.h>
 
-RCSID("$Id: oracle.c,v 1.75 2002/11/28 23:45:49 marcus Exp $");
+RCSID("$Id: oracle.c,v 1.82 2004/04/15 00:09:47 nilsson Exp $");
 
 
 /* User-changable defines: */
@@ -887,7 +886,7 @@ static void f_fetch_fields(INT32 args)
 	rc=OCIAttrGet((void *)column_parameter,
 		      OCI_DTYPE_PARAM,
 		      &type,
-		      (ub4*)0,
+		      (ub4*)NULL,
 		      OCI_ATTR_DATA_TYPE,
 		      dbcon->error_handle);
 	
@@ -896,7 +895,7 @@ static void f_fetch_fields(INT32 args)
 	rc=OCIAttrGet((void *)column_parameter,
 		      OCI_DTYPE_PARAM,
 		      &size,
-		      (ub4*)0,
+		      (ub4*)NULL,
 		      OCI_ATTR_DATA_SIZE,
 		      dbcon->error_handle);
 	
@@ -905,7 +904,7 @@ static void f_fetch_fields(INT32 args)
 	rc=OCIAttrGet((void *)column_parameter,
 		      OCI_DTYPE_PARAM,
 		      &scale,
-		      (ub4*)0,
+		      (ub4*)NULL,
 		      OCI_ATTR_SCALE,
 		      dbcon->error_handle);
 	
@@ -1004,7 +1003,7 @@ static void f_fetch_fields(INT32 args)
 				    (dvoid **) &info->data.lob, 
 				    (ub4)OCI_DTYPE_LOB, 
 				    (size_t) 0, 
-				    (dvoid **) 0)))
+				    (dvoid **) NULL)))
 	  {
 #ifdef ORACLE_DEBUG
 	    fprintf(stderr,"OCIDescriptorAlloc failed!\n");
@@ -1208,7 +1207,7 @@ static void push_inout_value(struct inout *inout,
 			       1, 
 			       (dvoid *) bufp,
 			       loblen, 
-			       (dvoid *)0, 
+			       (dvoid *)NULL,
 			       (sb4 (*)(dvoid *, CONST dvoid *, ub4, ub1)) 0,
 			       (ub2) 0, 
 			       (ub1) SQLCS_IMPLICIT)) != OCI_SUCCESS) 
@@ -1508,13 +1507,13 @@ static void f_oracle_create(INT32 args)
 static void f_dbcon_timeout_limit(INT32 args)
 {
   struct dbcon *dbcon=THIS_DBCON;
-  int           new_timeout;
   /* No arguments: get timeout. One argument: set timeout. */
 #ifdef ORACLE_DEBUG
   fprintf(stderr, "%s, dbcon=%p, args=%d\n", __FUNCTION__, dbcon, args);
 #endif
   if (args)
   {
+    INT_TYPE new_timeout;
     get_all_args("Oracle->timeout", args, "%i", &new_timeout);
     if (new_timeout < 0)
       Pike_error("Negative timeout specified.\n");
@@ -1739,8 +1738,7 @@ static void f_big_typed_query_create(INT32 args)
   if(d_flag)
   {
       CHECK_INTERPRETER_LOCK();
-      if(d_flag>1 && thread_for_id(th_self()) != Pike_thread_id)
-        Pike_fatal("thread_for_id() (or Pike_thread_id) failed in interpreter.h! %p != %p\n",thread_for_id(th_self()),Pike_thread_id);
+      DEBUG_CHECK_THREAD();
   }
 #endif
 
@@ -1788,8 +1786,7 @@ static void f_big_typed_query_create(INT32 args)
   if(d_flag)
   {
       CHECK_INTERPRETER_LOCK();
-      if(d_flag>1 && thread_for_id(th_self()) != Pike_thread_id)
-        Pike_fatal("thread_for_id() (or Pike_thread_id) failed in interpreter.h! %p != %p\n",thread_for_id(th_self()),Pike_thread_id);
+      DEBUG_CHECK_THREAD();
   }
 #endif
 
@@ -1799,7 +1796,8 @@ static void f_big_typed_query_create(INT32 args)
   {
     INT32 e;
     struct keypair *k;
-    MAPPING_LOOP(bnds)
+    struct mapping_data *md = bnds->data;
+    NEW_MAPPING_LOOP(md)
       {
 	struct svalue *value=&k->val;
 	sword rc = 0;
@@ -1870,7 +1868,6 @@ static void f_big_typed_query_create(INT32 args)
 	  case T_MULTISET:
 	    if(multiset_sizeof(value->u.multiset) == 1) {
 	      struct pike_string *s;
-#ifdef PIKE_NEW_MULTISETS
 	      {
 		struct svalue tmp;
 		if (use_multiset_index (value->u.multiset,
@@ -1880,12 +1877,6 @@ static void f_big_typed_query_create(INT32 args)
 		else
 		  s = NULL;
 	      }
-#else
-	      if (ITEM(value->u.multiset->ind)[0].type == T_STRING)
-		s = ITEM(value->u.multiset->ind)[0].u.string;
-	      else
-		s = NULL;
-#endif
 	      if (s) {
 		addr = (ub1 *)s->str;
 		len = s->len;
@@ -2001,8 +1992,7 @@ static void f_big_typed_query_create(INT32 args)
   if(d_flag)
   {
       CHECK_INTERPRETER_LOCK();
-      if(d_flag>1 && thread_for_id(th_self()) != Pike_thread_id)
-        Pike_fatal("thread_for_id() (or Pike_thread_id) failed in interpreter.h! %p != %p\n",thread_for_id(th_self()),Pike_thread_id);
+      DEBUG_CHECK_THREAD();
   }
 #endif
 
@@ -2046,8 +2036,7 @@ static void f_big_typed_query_create(INT32 args)
   if(d_flag)
   {
       CHECK_INTERPRETER_LOCK();
-      if(d_flag>1 && thread_for_id(th_self()) != Pike_thread_id)
-        Pike_fatal("thread_for_id() (or Pike_thread_id) failed in interpreter.h! %p != %p\n",thread_for_id(th_self()),Pike_thread_id);
+      DEBUG_CHECK_THREAD();
   }
 #endif
 
