@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: perlmod.c,v 1.32 2002/11/26 14:22:47 nilsson Exp $
+|| $Id: perlmod.c,v 1.36 2004/04/15 00:11:26 nilsson Exp $
 */
 
 #define NO_PIKE_SHORTHAND
@@ -17,6 +17,7 @@
 #include "threads.h"
 #include "mapping.h"
 #include "perl_machine.h"
+#include "pike_security.h"
 
 
 #ifdef HAVE_PERL
@@ -357,7 +358,10 @@ static void perlmod_create(INT32 args)
 #endif
 #endif
     
-  if (args != 0) Pike_error("Perl->create takes no arguments.");
+  CHECK_SECURITY_OR_ERROR(SECURITY_BIT_SECURITY,
+			  ("Perl->create: Permission denied.\n"));
+
+  if (args != 0) Pike_error("Perl->create takes no arguments.\n");
 
   if (!ps || !ps->perl) Pike_error("No perl interpreter available.\n");
 
@@ -401,6 +405,9 @@ static void perlmod_parse(INT32 args)
   fprintf(stderr, "[perlmod_parse, %d args]\n", args);
 #endif
     
+  CHECK_SECURITY_OR_ERROR(SECURITY_BIT_SECURITY,
+			  ("Perl->parse: Permission denied.\n"));
+
   check_all_args("Perl->parse",args,BIT_ARRAY, BIT_MAPPING|BIT_VOID, 0);
   if(!ps->perl) Pike_error("No perl interpreter available.\n");
 
@@ -442,7 +449,8 @@ static void perlmod_parse(INT32 args)
     int env_block_size=0;
     char *env_blockp;
     struct keypair *k;
-    MAPPING_LOOP(env_mapping)
+    struct mapping_data *md = env_mapping->data;
+    NEW_MAPPING_LOOP(md)
       env_block_size+=k->ind.u.string->len+k->val.u.string->len+2;
 
     ps->env_block=xalloc(env_block_size);
@@ -450,7 +458,7 @@ static void perlmod_parse(INT32 args)
 
     env_blockp = ps->env_block;
     d=0;
-    MAPPING_LOOP(env_mapping)
+    NEW_MAPPING_LOOP(md)
       {
 	ps->env[d++]=env_blockp;
 	MEMCPY(env_blockp,k->ind.u.string->str,k->ind.u.string->len);
@@ -483,6 +491,9 @@ static void perlmod_run(INT32 args)
   INT32 i;
   struct perlmod_storage *ps = _THIS;
 
+  CHECK_SECURITY_OR_ERROR(SECURITY_BIT_SECURITY,
+			  ("Perl->run: Permission denied.\n"));
+
   if(!ps->perl) Pike_error("No perl interpreter available.\n");
   pop_n_elems(args);
 
@@ -580,7 +591,12 @@ static void _perlmod_eval(INT32 args, int perlflags)
  */
 
 static void perlmod_eval(INT32 args)
-  { _perlmod_eval(args, G_SCALAR); }
+{
+  CHECK_SECURITY_OR_ERROR(SECURITY_BIT_SECURITY,
+			  ("Perl->eval: Permission denied.\n"));
+
+  _perlmod_eval(args, G_SCALAR);
+}
 
 /*! @decl mixed eval_list(string expression)
  *!
@@ -590,7 +606,12 @@ static void perlmod_eval(INT32 args)
  */
 
 static void perlmod_eval_list(INT32 args)
-  { _perlmod_eval(args, G_ARRAY); }
+{
+  CHECK_SECURITY_OR_ERROR(SECURITY_BIT_SECURITY,
+			  ("Perl->eval_list: Permission denied.\n"));
+
+  _perlmod_eval(args, G_ARRAY);
+}
 
 static void _perlmod_call(INT32 args, int perlflags)
 {
@@ -611,7 +632,7 @@ static void _perlmod_call(INT32 args, int perlflags)
 
   if (Pike_sp[-args].type != PIKE_T_STRING ||
       Pike_sp[-args].u.string->size_shift)
-       Pike_error("bad Perl function name (must be an 8-bit string)");
+       Pike_error("bad Perl function name (must be an 8-bit string)\n");
 
   ENTER;
   SAVETMPS;
@@ -632,7 +653,7 @@ static void _perlmod_call(INT32 args, int perlflags)
       case PIKE_T_STRING:
         if (s->u.string->size_shift)
         { PUTBACK; FREETMPS; LEAVE;
-          Pike_error("widestrings not supported in Pike-to-Perl call interface");
+          Pike_error("widestrings not supported in Pike-to-Perl call interface\n");
           return;
         }
         XPUSHs(sv_2mortal(newSVpv(s->u.string->str, s->u.string->len)));
@@ -727,7 +748,12 @@ static void _perlmod_call(INT32 args, int perlflags)
  */
 
 static void perlmod_call(INT32 args)
-  { _perlmod_call(args, G_SCALAR | G_EVAL);}
+{
+  CHECK_SECURITY_OR_ERROR(SECURITY_BIT_SECURITY,
+			  ("Perl->call: Permission denied.\n"));
+
+  _perlmod_call(args, G_SCALAR | G_EVAL);
+}
 
 /*! @decl mixed call_list(string name, mixed ... arguments)
  *!
@@ -745,7 +771,12 @@ static void perlmod_call(INT32 args)
  */
 
 static void perlmod_call_list(INT32 args)
-  { _perlmod_call(args, G_ARRAY | G_EVAL);}
+{
+  CHECK_SECURITY_OR_ERROR(SECURITY_BIT_SECURITY,
+			  ("Perl->call_list: Permission denied.\n"));
+
+  _perlmod_call(args, G_ARRAY | G_EVAL);
+}
 
 static void _perlmod_varop(INT32 args, int op, int type)
 /* To avoid excessive code duplication, this function does most of the
@@ -834,7 +865,7 @@ static void _perlmod_varop(INT32 args, int op, int type)
  *!   Name of the scalar variable, as an 8-bit string.
  */
 static void perlmod_get_scalar(INT32 args)
-   { _perlmod_varop(args, 'R', 'S');}
+ { _perlmod_varop(args, 'R', 'S');}
 
 /*! @decl void set_scalar(string name, mixed value)
  *!
@@ -848,7 +879,12 @@ static void perlmod_get_scalar(INT32 args)
  *!   an error for unsupported value types. 
  */
 static void perlmod_set_scalar(INT32 args)
-   { _perlmod_varop(args, 'W', 'S');}
+{
+  CHECK_SECURITY_OR_ERROR(SECURITY_BIT_SECURITY,
+			  ("Perl->set_scalar: Permission denied.\n"));
+
+  _perlmod_varop(args, 'W', 'S');
+}
 
 /*! @decl mixed get_array_item(string name, int index)
  *!
@@ -884,7 +920,12 @@ static void perlmod_get_array_item(INT32 args)
  *!   thrown for unsupported value types.
  */
 static void perlmod_set_array_item(INT32 args)
-   { _perlmod_varop(args, 'W', 'A');}
+{
+  CHECK_SECURITY_OR_ERROR(SECURITY_BIT_SECURITY,
+			  ("Perl->create: Permission denied.\n"));
+
+  _perlmod_varop(args, 'W', 'A');
+}
 
 /*! @decl mixed get_hash_item(string name, mixed key)
  *!
@@ -918,7 +959,12 @@ static void perlmod_get_hash_item(INT32 args)
  *!   thrown for unsupported value types.
  */
 static void perlmod_set_hash_item(INT32 args)
-   { _perlmod_varop(args, 'W', 'H');}
+{
+  CHECK_SECURITY_OR_ERROR(SECURITY_BIT_SECURITY,
+			  ("Perl->set_hash_item: Permission denied.\n"));
+
+  _perlmod_varop(args, 'W', 'H');
+}
 
 /*! @decl int array_size(string name)
  *!
@@ -1070,7 +1116,7 @@ static void perlmod_array_size_limit(INT32 args)
 
     case 1:
       if (Pike_sp[-args].type != PIKE_T_INT || Pike_sp[-args].u.integer < 1)
-           Pike_error("Argument must be a integer in range 1 to 2147483647.");
+           Pike_error("Argument must be a integer in range 1 to 2147483647.\n");
       _THIS->array_size_limit = Pike_sp[-args].u.integer;
       break;
 
