@@ -2,12 +2,12 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: freetype.c,v 1.19 2005/10/31 09:12:10 grubba Exp $
+|| $Id: freetype.c,v 1.21 2004/04/19 23:03:01 nilsson Exp $
 */
 
 #include "config.h"
 #include "global.h"
-RCSID("$Id: freetype.c,v 1.19 2005/10/31 09:12:10 grubba Exp $");
+RCSID("$Id: freetype.c,v 1.21 2004/04/19 23:03:01 nilsson Exp $");
 #include "module.h"
 #include "pike_error.h"
 
@@ -36,6 +36,11 @@ RCSID("$Id: freetype.c,v 1.19 2005/10/31 09:12:10 grubba Exp $");
 
 #ifdef HAVE_LIBFT2
 
+#if !defined(FT_ENC_TAG) && defined(FT_MAKE_TAG)
+#define FT_ENC_TAG( value, _x1, _x2, _x3, _x4) \
+	value = FT_MAKE_TAG( _x1, _x2, _x3, _x4 )
+#endif /* !defined(FT_ENC_TAG) && defined(FT_MAKE_TAG) */
+
 #define sp Pike_sp
 #define fp Pike_fp
 
@@ -54,39 +59,6 @@ struct face
 
 #define TFACE   ((struct face*)Pike_fp->current_storage)->face
 
-#undef __FTERRORS_H__
-#define FT_ERROR_START_LIST				\
-  struct image_ft_error_lookup {			\
-    const char *sym;					\
-    FT_Error code;					\
-    const char *msg;					\
-  } image_ft_error_lookup[] = {
-#define FT_ERRORDEF(SYM, VAL, MSG) { TOSTR(SYM), VAL, MSG },
-#define FT_ERROR_END_LIST			\
-  { NULL, 0, NULL },				\
-    };
-#ifdef FT_ERRORS_H
-#include FT_ERRORS_H
-#else
-#include <freetype/fterrors.h>
-#endif
-
-static void image_ft_error(const char *msg, FT_Error errcode)
-{
-  const char *errmsg = NULL;
-  if (errcode) {
-    struct image_ft_error_lookup *entry;
-    for (entry = image_ft_error_lookup; entry->sym; entry++) {
-      if (entry->code == errcode) {
-	errmsg = entry->msg;
-	break;
-      }
-    }
-  }
-  if (!errmsg) Pike_error("%s\n", msg);
-  Pike_error("%s: %s\n", msg, errmsg);
-}
-
 /*! @module Image */
 
 /*! @module FreeType
@@ -97,11 +69,6 @@ static void image_ft_error(const char *msg, FT_Error errcode)
  *! A FreeType font face. We recommend using the more generic font handling
  *! interfaces in @[Image.Fonts] instead.
  */
-static void image_ft_face_init( struct object *o )
-{
-  TFACE = NULL;
-}
-
 static void image_ft_face_free( struct object *o )
 {
   FT_Done_Face( TFACE );
@@ -135,7 +102,7 @@ static void image_ft_face_write_char( INT32 args )
   {
     int p = slot->bitmap.pitch;
     int g = slot->bitmap.num_grays;
-    char *s = slot->bitmap.buffer;
+    unsigned char *s = slot->bitmap.buffer;
     if( s )
       for( y = 0; y<i->ysize; y++ )
         for( x = 0; x<i->xsize; x++ )
@@ -146,20 +113,6 @@ static void image_ft_face_write_char( INT32 args )
           d->b = pv;
           d++;
         }
-  } else if( slot->bitmap.pixel_mode == ft_pixel_mode_mono ) {
-    int p = slot->bitmap.pitch;
-    char *s = slot->bitmap.buffer;
-    p *= 8;
-    if( s )
-      for( y = 0; y<i->ysize; y++ )
-        for( x = 0; x<i->xsize; x++ )
-        {
-          int pv =  ( ((s[(x + y*p) / 8]) << ((x + y*p) % 8)) & 128) ? 255 : 0;
-          d->r = pv;
-          d->g = pv;
-          d->b = pv;
-          d++;
-        }
   } else
     Pike_error("Unhandled bitmap format received from renderer\n");
 
@@ -181,7 +134,7 @@ static void image_ft_face_get_kerning( INT32 args )
 {
   INT_TYPE l, r;
   FT_Vector kern;
-  get_all_args( "get_kerning", args, "%d%d", &l, &r );
+  get_all_args( "get_kerning", args, "%i%i", &l, &r );
   l = FT_Get_Char_Index( TFACE, l );
   r = FT_Get_Char_Index( TFACE, r );
   if( FT_Get_Kerning( TFACE, l, r, ft_kerning_default, &kern ) )
@@ -196,10 +149,9 @@ static void image_ft_face_get_kerning( INT32 args )
 static void image_ft_face_attach_file( INT32 args )
 {
   char *path;
-  FT_Error errcode;
   get_all_args( "attach_file", args, "%s", &path );
-  if ((errcode = FT_Attach_File( TFACE, path )))
-    image_ft_error("Failed to attach file", errcode);
+  if( FT_Attach_File( TFACE, path ) )
+    Pike_error("Failed to attach file\n");
   pop_n_elems( args );
   push_int( 0 );
 }
@@ -209,19 +161,60 @@ static void image_ft_face_attach_file( INT32 args )
 static void image_ft_face_set_size( INT32 args )
 {
   int w, h;
-  FT_Error errcode;
   if( (args != 2) || (sp[-args].type != sp[1-args].type)
       || (sp[-args].type != T_INT) )
     Pike_error("Illegal arguments to set_size\n");
   w = sp[-2].u.integer;
   h = sp[-1].u.integer;
 
-  if((errcode = FT_Set_Pixel_Sizes( TFACE, w, h )))
-    image_ft_error("Failed to set size", errcode);
+  if( FT_Set_Pixel_Sizes( TFACE, w, h ) )
+    Pike_error("Failed to set size\n");
   pop_n_elems( 2 );
   ref_push_object( fp->current_object );
 }
 
+/*! @decl array(string) list_encodings()
+ */
+static void image_ft_face_list_encodings( INT32 args )
+{
+  FT_Int enc_no;
+  pop_n_elems( args );
+  for(enc_no=0; enc_no<TFACE->num_charmaps; enc_no++) {
+    FT_Encoding e = TFACE->charmaps[enc_no]->encoding;
+    if(e == ft_encoding_none)
+      push_int( 0 );
+    else {
+      push_constant_text( "%4c" );
+      push_int( e );
+      f_sprintf( 2 );
+    }
+  }
+  f_aggregate( enc_no );
+}
+
+/*! @decl void select_encoding(string|int encoding)
+ */
+static void image_ft_face_select_encoding( INT32 args )
+{
+  FT_Encoding e;
+  FT_Error er;
+  if( args != 1 || (sp[-args].type != PIKE_T_INT &&
+		    sp[-args].type != PIKE_T_STRING) )
+    Pike_error("Illegal arguments to select_encoding\n");
+  if( sp[-args].type == PIKE_T_INT )
+    e = (sp[-args].u.integer == 0? ft_encoding_none : sp[-args].u.integer);
+  else if( sp[-args].u.string->len != 4 || sp[-args].u.string->size_shift )
+    Pike_error("Invalid encoding name in select_encoding\n");
+  else {
+    p_wchar0 *s = STR0(sp[-args].u.string);
+    FT_ENC_TAG( e, s[0], s[1], s[2], s[3] );
+  }
+  pop_n_elems( args );
+  er = FT_Select_Charmap(TFACE, e);
+  if( er )
+    Pike_error( "Character encoding not available in this font\n" );
+}
+
 /*! @decl mapping info()
  *! @returns
  *!   @mapping
@@ -236,6 +229,7 @@ static void image_ft_face_set_size( INT32 args )
  */
 static void image_ft_face_info( INT32 args )
 {
+  pop_n_elems( args );
   push_text( "family" );
   if( TFACE->family_name )
     push_text( TFACE->family_name );
@@ -268,13 +262,27 @@ static void image_ft_face_create( INT32 args )
   else if( er )
     Pike_error( "Failed to open the font file %s\n", sp[-args].u.string->str );
   for(enc_no=0; enc_no<TFACE->num_charmaps; enc_no++) {
-    enc_score = 0;
     switch(TFACE->charmaps[enc_no]->encoding) {
     case ft_encoding_symbol: enc_score = -1; break;
     case ft_encoding_unicode: enc_score = 2; break;
 #if HAVE_DECL_FT_ENCODING_LATIN_1
     case ft_encoding_latin_1: enc_score = 1; break;
 #endif
+#if 0
+      /* FIXME: Should we rate any of these? */
+    case ft_encoding_none:
+    case ft_encoding_latin_2:
+    case ft_encoding_sjis:
+    case ft_encoding_gb2312:
+    case ft_encoding_big5:
+    case ft_encoding_wansung:
+    case ft_encoding_johab:
+    case ft_encoding_adobe_standard:
+    case ft_encoding_adobe_expert:
+    case ft_encoding_adobe_custom:
+    case ft_encoding_apple_roman:
+#endif
+    default: enc_score = 0; break;
     }
     if(enc_score > best_enc_score) {
       best_enc_score = enc_score;
@@ -323,15 +331,10 @@ PIKE_MODULE_INIT
   if( !FT_Init_FreeType( &library ) )
   {
 #ifdef DYNAMIC_MODULE
-    push_string(make_shared_string("Image"));
-    push_int(0);
-    SAFE_APPLY_MASTER("resolv",2);
-    if (sp[-1].type==T_OBJECT)
-    {
-      push_string(make_shared_string("Image"));
-      f_index(2);
+    push_text("Image.Image");
+    SAFE_APPLY_MASTER("resolv",1);
+    if (sp[-1].type==T_PROGRAM)
       image_program=program_from_svalue(sp-1);
-    }
     pop_n_elems(1);
 #endif /* DYNAMIC_MODULE */
 
@@ -341,12 +344,13 @@ PIKE_MODULE_INIT
     ADD_FUNCTION("create",image_ft_face_create, tFunc(tStr,tVoid), 0 );
     ADD_FUNCTION("set_size",image_ft_face_set_size,tFunc(tInt tInt,tObj),0);
     ADD_FUNCTION("attach_file",image_ft_face_attach_file,tFunc(tString,tVoid),0);
+    ADD_FUNCTION("list_encodings",image_ft_face_list_encodings,tFunc(tNone,tArr(tString)),0);
+    ADD_FUNCTION("select_encoding",image_ft_face_select_encoding,tFunc(tOr(tString,tInt),tVoid),0);
     ADD_FUNCTION("info",image_ft_face_info,tFunc(tVoid,tMapping),0);
     ADD_FUNCTION("write_char",image_ft_face_write_char,tFunc(tInt,tObj),0);
     ADD_FUNCTION("get_kerning",image_ft_face_get_kerning,
                  tFunc(tInt tInt,tInt),0);
     
-    set_init_callback( image_ft_face_init );
     set_exit_callback( image_ft_face_free );
 
     face_program = end_program();
