@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: image_jpeg.c,v 1.60 2002/10/29 14:07:58 nilsson Exp $
+|| $Id: image_jpeg.c,v 1.65 2004/04/15 00:10:17 nilsson Exp $
 */
 
 #include "global.h"
@@ -34,6 +34,18 @@
 #undef FAR
 #endif
 
+#ifdef HAVE_JCONFIG_H_HAVE_BOOLEAN
+#ifdef HAVE_WINDOWS_H
+#include <windows.h>
+#ifdef HAVE_WTYPES_H
+/* jconfig.h has probably been compiled without WIN32_LEAN_AND_MEAN...
+ * So we need this one to get the boolean typedef.
+ */
+#include <wtypes.h>
+#endif
+#endif
+#endif
+
 #include <jpeglib.h>
 #include "transupp.h" /* Support routines for jpeg transformations */
 
@@ -47,7 +59,7 @@
 #ifdef HAVE_STDLIB_H
 #undef HAVE_STDLIB_H
 #endif
-RCSID("$Id: image_jpeg.c,v 1.60 2002/10/29 14:07:58 nilsson Exp $");
+RCSID("$Id: image_jpeg.c,v 1.65 2004/04/15 00:10:17 nilsson Exp $");
 
 /* jpeglib defines EXTERN for some reason.
  * This is not good, since it confuses compilation.h.
@@ -322,7 +334,7 @@ static int parameter_qt(struct svalue *map,struct pike_string *what,
 {
    struct svalue *v;
    unsigned int table[DCTSIZE2];
-   struct mapping *m;
+   struct mapping_data *md;
    INT32 e;
    struct keypair *k;
 
@@ -332,8 +344,8 @@ static int parameter_qt(struct svalue *map,struct pike_string *what,
    else if (v->type!=T_MAPPING) 
       Pike_error("Image.JPEG.encode: illegal value of option quant_table; expected mapping\n");
 
-   m=v->u.mapping;
-   MAPPING_LOOP(m)
+   md=v->u.mapping->data;
+   NEW_MAPPING_LOOP(md)
       {
 	 int z;
 	 if (k->ind.type!=T_INT || k->val.type!=T_ARRAY)
@@ -345,9 +357,9 @@ static int parameter_qt(struct svalue *map,struct pike_string *what,
 	 if ((z=store_int_in_table(k->val.u.array,DCTSIZE2,table))!=
 	     DCTSIZE2)
 	    Pike_error("Image.JPEG.encode: illegal value of option quant_table;"
-		  " quant_table %d array is of illegal size (%d), "
-		  "expected %d integers\n",
-		  k->ind.u.integer,z,DCTSIZE2);
+		       " quant_table %"PRINTPIKEINT"d array is of illegal size (%d), "
+		       "expected %d integers\n",
+		       k->ind.u.integer,z,DCTSIZE2);
 
 	 jpeg_add_quant_table(cinfo,k->ind.u.integer,table,100,0);
       }
@@ -360,7 +372,7 @@ static int parameter_qt_d(struct svalue *map,struct pike_string *what,
 {
    struct svalue *v;
    unsigned int table[DCTSIZE2];
-   struct mapping *m;
+   struct mapping_data *md;
    INT32 e;
    struct keypair *k;
 
@@ -370,8 +382,8 @@ static int parameter_qt_d(struct svalue *map,struct pike_string *what,
    else if (v->type!=T_MAPPING) 
       Pike_error("Image.JPEG.encode: illegal value of option quant_table; expected mapping\n");
 
-   m=v->u.mapping;
-   MAPPING_LOOP(m)
+   md=v->u.mapping->data;
+   NEW_MAPPING_LOOP(md)
       {
 	 int z;
 	 if (k->ind.type!=T_INT || k->val.type!=T_ARRAY)
@@ -383,9 +395,9 @@ static int parameter_qt_d(struct svalue *map,struct pike_string *what,
 	 if ((z=store_int_in_table(k->val.u.array,DCTSIZE2,table))!=
 	     DCTSIZE2)
 	    Pike_error("Image.JPEG.encode: illegal value of option quant_table;"
-		  " quant_table %d array is of illegal size (%d), "
-		  "expected %d integers\n",
-		  k->ind.u.integer,z,DCTSIZE2);
+		       " quant_table %"PRINTPIKEINT"d array is of illegal size (%d), "
+		       "expected %d integers\n",
+		       k->ind.u.integer,z,DCTSIZE2);
 
 	 /* jpeg_add_quant_table(cinfo,k->ind.u.integer,table,100,0); */
 
@@ -418,7 +430,7 @@ static int parameter_marker(struct svalue *map,struct pike_string *what,
 			    struct jpeg_compress_struct *cinfo)
 {
    struct svalue *v;
-   struct mapping *m;
+   struct mapping_data *md;
    INT32 e;
    struct keypair *k;
 
@@ -428,8 +440,8 @@ static int parameter_marker(struct svalue *map,struct pike_string *what,
    else if (v->type!=T_MAPPING) 
       Pike_error("Image.JPEG.encode: illegal value of option marker;"
 		 " expected mapping\n");
-   m=v->u.mapping;
-   MAPPING_LOOP(m)
+   md=v->u.mapping->data;
+   NEW_MAPPING_LOOP(md)
       {
 	 if (k->ind.type!=T_INT || k->val.type!=T_STRING ||
 	     k->val.u.string->size_shift)
@@ -509,7 +521,7 @@ static int marker_exists_in_args(INT32 args, int which)
 {
    struct svalue *map=sp+1-args;
    struct svalue *v=NULL;
-   struct mapping *m=NULL;
+   struct mapping_data *md;
    INT32 e;
    struct keypair *k;
    v=low_mapping_string_lookup(map->u.mapping, param_comment);
@@ -517,15 +529,15 @@ static int marker_exists_in_args(INT32 args, int which)
      return (v->type==T_STRING && !v->u.string->size_shift);
    } else {
      v=low_mapping_string_lookup(map->u.mapping, param_marker);
-     if (v && v->type==T_MAPPING) m=v->u.mapping;
-     if (m) {
-       MAPPING_LOOP(m)
-	 {
-	   if (k->ind.type==T_INT && k->val.type==T_STRING
-	       && k->ind.u.integer==which && !k->val.u.string->size_shift) {
-	     return 1;
-	   }
+     if (v && v->type==T_MAPPING) {
+       md=v->u.mapping->data;
+       NEW_MAPPING_LOOP(md)
+       {
+	 if (k->ind.type==T_INT && k->val.type==T_STRING
+	     && k->ind.u.integer==which && !k->val.u.string->size_shift) {
+	   return 1;
 	 }
+       }
      }
    }
    return 0;
@@ -1497,15 +1509,10 @@ PIKE_MODULE_INIT
 {
 #ifdef HAVE_JPEGLIB_H
 #ifdef DYNAMIC_MODULE
-   push_string(make_shared_string("Image"));
-   push_int(0);
-   SAFE_APPLY_MASTER("resolv",2);
-   if (sp[-1].type==T_OBJECT) 
-   {
-      push_string(make_shared_string("image"));
-      f_index(2);
+   push_text("Image.Image");
+   SAFE_APPLY_MASTER("resolv",1);
+   if (sp[-1].type==T_PROGRAM)
       image_program=program_from_svalue(sp-1);
-   }
    pop_n_elems(1);
 #endif /* DYNAMIC_MODULE */
 
