@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: sendfile.c,v 1.63 2003/12/09 09:57:14 grubba Exp $
+|| $Id: sendfile.c,v 1.69 2004/04/15 10:52:34 grubba Exp $
 */
 
 /*
@@ -59,6 +59,16 @@
 #include <sys/uio.h>
 #endif /* HAVE_SYS_UIO_H */
 
+#ifdef HAVE_NETINET_TCP_H
+#include <netinet/tcp.h>
+#endif /* HAVE_NETINET_TCP_H */
+
+#ifdef HAVE_SYS_SENDFILE_H
+#ifndef HAVE_BROKEN_SYS_SENDFILE_H
+#include <sys/sendfile.h>
+#endif /* !HAVE_BROKEN_SYS_SENDFILE_H */
+#endif /* HAVE_SYS_SENDFILE_H */
+
 #if 0
 #ifdef HAVE_SYS_MMAN_H
 #include <sys/mman.h>
@@ -112,26 +122,28 @@
 #undef THIS
 #define THIS	((struct pike_sendfile *)(Pike_fp->current_storage))
 
+/*
+ * We assume sendfile(2) has been fixed on all OS's by now.
+ * FIXME: Configure tests?
+ *	/grubba 2004-03-30
+ */
+#if 0
 /*
  * All known versions of sendfile(2) are broken.
+ *	/grubba 2000-10-19
  */
 #ifndef HAVE_BROKEN_SENDFILE
 #define HAVE_BROKEN_SENDFILE
 #endif /* !HAVE_BROKEN_SENDFILE */
+#endif /* 0 */
 
 /*
  * Disable any use of sendfile(2) if HAVE_BROKEN_SENDFILE is defined.
  */
 #ifdef HAVE_BROKEN_SENDFILE
-#ifdef HAVE_SENDFILE
 #undef HAVE_SENDFILE
-#endif /* HAVE_SENDFILE */
-#ifdef HAVE_FREEBSD_SENDFILE
 #undef HAVE_FREEBSD_SENDFILE
-#endif /* HAVE_FREEBSD_SENDFILE */
-#ifdef HAVE_HPUX_SENDFILE
 #undef HAVE_HPUX_SENDFILE
-#endif /* HAVE_HPUX_SENDFILE */
 #endif /* HAVE_BROKEN_SENDFILE */
 
 /*
@@ -340,9 +352,50 @@ static ptrdiff_t send_iov(int fd, struct iovec *iov, int iovcnt)
 
 void low_do_sendfile(struct pike_sendfile *this)
 {
+#if defined(SOL_TCP) && (defined(TCP_CORK) || defined(TCP_NODELAY))
+  int old_val = -1;
+  size_t old_len = sizeof(old_val);	/* Might want to use socklen_t here. */
+#ifdef TCP_CORK
+  int new_val = 1;
+#else /* !TCP_CORK */
+  int new_val = 0;
+#endif /* TCP_CORK */
+#endif /* SOL_TCP && (TCP_CORK || TCP_NODELAY) */
+
   /* Make sure we're using blocking I/O */
   set_nonblocking(this->to_fd, 0);
 
+#ifdef SOL_TCP
+#ifdef TCP_CORK
+  /* Attempt to set the out socket into cork mode.
+   * FIXME: Do we need to adjust TCP_NODELAY here?
+   */
+  while ((getsockopt(this->to_fd, SOL_TCP, TCP_CORK,
+		     &old_val, &old_len)<0) &&
+	 (errno == EINTR))
+    ;
+  if (!old_val) {
+    while ((setsockopt(this->to_fd, SOL_TCP, TCP_CORK,
+		       &new_val, sizeof(new_val))<0) &&
+	   (errno == EINTR))
+      ;
+  }
+#elif defined(TCP_NODELAY)
+  /* Attempt to set the out socket into nagle mode.
+   */
+  while ((getsockopt(this->to_fd, SOL_TCP, TCP_NODELAY,
+		     &old_val, &old_len)<0) &&
+	 (errno == EINTR))
+    ;
+  if (old_val == 1) {
+    while ((setsockopt(this->to_fd, SOL_TCP, TCP_NODELAY,
+		       &new_val, sizeof(new_val))<0) &&
+	   (errno == EINTR))
+      ;
+  }  
+#endif /* TCP_CORK || TCP_NODELAY */
+#endif /* SOL_TCP */
+
   SF_DFPRINTF((stderr, "sendfile: Worker started\n"));
 
   if ((this->from_file) && (this->len)) {
@@ -391,6 +444,7 @@ void low_do_sendfile(struct pike_sendfile *this)
     res = sendfile(this->from_fd, this->to_fd, this->offset, len,
 		   &hdtr, &sent, 0);
 #else /* !HAVE_FREEBSD_SENDFILE */
+    /* HPUX_SENDFILE */
     res = sendfile(this->to_fd, this->from_fd, this->offset, len,
 		   hdtr, 0);
 #endif /* HAVE_FREEBSD_SENDFILE */
@@ -410,6 +464,7 @@ void low_do_sendfile(struct pike_sendfile *this)
 #ifdef HAVE_FREEBSD_SENDFILE
 	Pike_fatal("FreeBSD style sendfile(): EFAULT\n");
 #else /* !HAVE_FREEBSD_SENDFILE */
+	/* HPUX_SENDFILE */
 	Pike_fatal("HP/UX style sendfile(): EFAULT\n");
 #endif /* HAVE_FREEBSD_SENDFILE */
 	break;
@@ -422,6 +477,7 @@ void low_do_sendfile(struct pike_sendfile *this)
 	break;
       }
 #ifndef HAVE_FREEBSD_SENDFILE
+      /* HPUX_SENDFILE */
     } else {
       sent = res;
 #endif /* !HAVE_FREEBSD_SENDFILE */
@@ -606,9 +662,29 @@ void low_do_sendfile(struct pike_sendfile *this)
  done:
 #endif /* HAVE_FREEBSD_SENDFILE || HAVE_HPUX_SENDFILE */
 
+#ifdef SOL_TCP
+#ifdef TCP_CORK
+  /* Restore the cork mode for the socket. */
+  if (!old_val) {
+    while((setsockopt(this->to_fd, SOL_TCP, TCP_CORK,
+		      &old_val, old_len)<0) && (errno == EINTR))
+      ;
+  }
+#elif defined(TCP_NODELAY)
+  /* Restore the nagle mode for the socket. */
+  if (old_val == 1) {
+    while((setsockopt(this->to_fd, SOL_TCP, TCP_NODELAY,
+		      &old_val, old_len)<0) && (errno == EINTR))
+      ;
+  }
+#endif /* TCP_CORK || TCP_NODELAY */
+#endif /* SOL_TCP */
+
   SF_DFPRINTF((stderr,
 	      "sendfile: Done. Setting up callback\n"
 	      "%d bytes sent\n", this->sent));
+
+  /* FIXME: Restore non-blocking mode here? */
 }
 
 static void worker(void *this_)
@@ -628,11 +704,10 @@ static void worker(void *this_)
     } else {
       /* Destructed. */
       if (this->from_fd >= 0) {
-	set_backend_for_fd(this->from_fd, NULL);
 	fd_close(this->from_fd);
       }
       /* Paranoia */
-      this->from->fd = -1;
+      change_fd_for_box (&this->from->box, -1);
     }
   }
   if (this->to_file->prog) {
@@ -640,11 +715,10 @@ static void worker(void *this_)
   } else {
     /* Destructed */
     if (this->to_fd >= 0) {
-      set_backend_for_fd(this->to_fd, NULL);
       fd_close(this->to_fd);
     }
     /* Paranoia */
-    this->to->fd = -1;
+    change_fd_for_box (&this->to->box, -1);
   }
     
   /* Neither of the following can be done in our current context
@@ -688,6 +762,7 @@ static void sf_create(INT32 args)
   struct pike_sendfile sf;
   int iovcnt = 0;
   struct svalue *cb = NULL;
+  LONGEST offset, len;
 
   if (THIS->to_file) {
     Pike_error("sendfile->create(): Called a second time!\n");
@@ -708,8 +783,11 @@ static void sf_create(INT32 args)
   sf.callback.type = T_INT;
 
   get_all_args("sendfile", args, "%A%O%l%l%A%o%*",
-	       &(sf.headers), &(sf.from_file), &(sf.offset),
-	       &(sf.len), &(sf.trailers), &(sf.to_file), &cb);
+	       &(sf.headers), &(sf.from_file), &offset,
+	       &len, &(sf.trailers), &(sf.to_file), &cb);
+
+  sf.offset = offset;
+  sf.len = len;
 
   /* We need to give 'cb' another reference /Hubbe */
   /* No, we don't. We steal the reference from the stack.
@@ -747,10 +825,10 @@ static void sf_create(INT32 args)
     sp[5-args].u.object = sf.to_file;
   }
   if ((sf.to->flags & FILE_LOCK_FD) ||
-      (sf.to->fd < 0)) {
+      (sf.to->box.fd < 0)) {
     SIMPLE_BAD_ARG_ERROR("sendfile", 6, "object(Stdio.File)");
   }
-  sf.to_fd = sf.to->fd;
+  sf.to_fd = sf.to->box.fd;
 
   if (sf.from_file && !sf.len) {
     /* No need for the from_file object. */
@@ -786,10 +864,10 @@ static void sf_create(INT32 args)
       sp[1-args].u.object = sf.from_file;
     }
     if ((sf.from->flags & FILE_LOCK_FD) ||
-	(sf.from->fd < 0)) {
+	(sf.from->box.fd < 0)) {
       SIMPLE_BAD_ARG_ERROR("sendfile", 2, "object(Stdio.File)");
     }
-    sf.from_fd = sf.from->fd;
+    sf.from_fd = sf.from->box.fd;
   }
 
   /* Do some extra arg checking */
@@ -1020,7 +1098,7 @@ static void sf_create(INT32 args)
 void init_sendfile(void)
 {
 #ifdef _REENTRANT
-  start_new_program();
+  START_NEW_PROGRAM_ID (STDIO_SENDFILE);
   ADD_STORAGE(struct pike_sendfile);
   map_variable("_args", "array(mixed)", 0, OFFSETOF(pike_sendfile, args),
 	       T_ARRAY);
