@@ -1,9 +1,12 @@
 /*
- * $Id: stat.c,v 1.18 2003/02/04 09:42:30 grubba Exp $
- */
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: stat.c,v 1.29 2003/11/14 10:26:43 mast Exp $
+*/
 
 #include "global.h"
-RCSID("$Id: stat.c,v 1.18 2003/02/04 09:42:30 grubba Exp $");
+RCSID("$Id: stat.c,v 1.29 2003/11/14 10:26:43 mast Exp $");
 #include "fdlib.h"
 #include "interpret.h"
 #include "svalue.h"
@@ -12,7 +15,6 @@ RCSID("$Id: stat.c,v 1.18 2003/02/04 09:42:30 grubba Exp $");
 #include "object.h"
 #include "builtin_functions.h"
 #include "operators.h"
-#include "opcodes.h"
 #include "program_id.h"
 
 #include <sys/stat.h>
@@ -20,6 +22,109 @@ RCSID("$Id: stat.c,v 1.18 2003/02/04 09:42:30 grubba Exp $");
 #include <sys/param.h>
 #endif /* HAVE_SYS_PARAM_H */
 
+#define sp Pike_sp
+#define fp Pike_fp
+
+/*! @module Stdio
+ */
+/*! @class Stat
+ *!
+ *!   This object is used to represent file status information
+ *!   from e.g. @[file_stat()].
+ *!
+ *!   It contains the following items usually found in a C @tt{struct
+ *!   stat@}:
+ *!   @dl
+ *!     @item mode
+ *!       File mode (see @tt{mknod(2)@}).
+ *!     @item size
+ *!       File size in bytes.
+ *!     @item uid
+ *!       User ID of the file's owner.
+ *!     @item gid
+ *!       Group ID of the file's owner.
+ *!     @item atime
+ *!       Time of last access in seconds since 00:00:00 UTC, 1970-01-01.
+ *!     @item mtime
+ *!       Time of last data modification.
+ *!     @item ctime
+ *!       Time of last file status change.
+ *!     @item ino
+ *!       Inode number.
+ *!     @item nlink
+ *!       Number of links.
+ *!     @item dev
+ *!       ID of the device containing a directory entry for this file.
+ *!     @item rdev
+ *!       ID of the device.
+ *!   @enddl
+ *!
+ *!   It also contains some items that correspond to the C @tt{IS*@} macros:
+ *!   @dl
+ *!     @item isreg
+ *!       Set if the file is a regular file.
+ *!     @item isdir
+ *!       Set if the file is a directory.
+ *!     @item islnk
+ *!       Set if the file is a symbolic link. Note that symbolic links
+ *!       are normally followed by the stat functions, so this might
+ *!       only be set if you turn that off, e.g. by giving a nonzero
+ *!       second argument to @[file_stat()].
+ *!     @item isfifo
+ *!       Set if the file is a FIFO (aka named pipe).
+ *!     @item issock
+ *!       Set if the file is a socket.
+ *!     @item ischr
+ *!       Set if the file is a character device.
+ *!     @item isblk
+ *!       Set if the file is a block device.
+ *!   @enddl
+ *!
+ *!   There are also some items that provide alternative representations
+ *!   of the above:
+ *!   @dl
+ *!     @item type
+ *!       The type as a string, can be any of @expr{"reg"@},
+ *!       @expr{"dir"@}, @expr{"lnk"@}, @expr{"fifo"@}, @expr{"sock"@},
+ *!       @expr{"chr"@}, @expr{"blk"@}, and @expr{"unknown"@}.
+ *!     @item mode_string
+ *!       The file mode encoded as a string in @tt{ls -l@} style, e.g.
+ *!       @expr{"drwxr-xr-x"@}.
+ *!   @enddl
+ *!
+ *!   Note that some items might not exist or have meaningful values
+ *!   on some platforms.
+ *!
+ *!   Additionally, the object may be initialized from or casted to an
+ *!   @expr{array@} on the form of a 'traditional' LPC stat-array, and
+ *!   it's also possible to index the object directly with integers as
+ *!   if it were such an array. The stat-array has this format:
+ *!
+ *!   @array
+ *!     @elem int 0
+ *!       File mode, same as @tt{mode@}.
+ *!     @elem int 1
+ *!       If zero or greater, the file is a regular file and this is
+ *!       its size in bytes. If less than zero it gives the type:
+ *!       -2=directory, -3=symlink and -4=device.
+ *!     @elem int 2
+ *!       Time of last access, same as @tt{atime@}.
+ *!     @elem int 3
+ *!       Time of last data modification, same as @tt{mtime@}.
+ *!     @elem int 4
+ *!       Time of last file status change, same as @tt{ctime@}.
+ *!     @elem int 5
+ *!       User ID of the file's owner, same as @tt{uid@}.
+ *!     @elem int 6
+ *!       Group ID of the file's owner, same as @tt{gid@}.
+ *!   @endarray
+ *!
+ *!   It's possible to modify the stat struct by assigning values to
+ *!   the items. They essentially work as variables, although some of
+ *!   them affect others, e.g. setting @expr{isdir@} clears @expr{isreg@}
+ *!   and setting @expr{mode_string@} changes many of the other items.
+ */
+
 /* Let's define these mode flags if they don't exist, so reading and
  * writing the Stat structure will behave identically regardless of
  * OS. */
@@ -92,7 +197,7 @@ struct program *stat_program=NULL;
 struct stat_storage
 {
    /* Note: stat_create assumes there are no refs in here. */
-   struct stat s;
+   PIKE_STAT_T s;
 };
 
 static struct mapping *stat_map=NULL;
@@ -117,7 +222,7 @@ static struct pike_string *stat_index_strs[STAT_ENUM_END];
 static struct pike_string *str_type_reg, *str_type_dir, *str_type_lnk,
   *str_type_chr, *str_type_blk, *str_type_fifo, *str_type_sock, *str_type_unknown;
 
-#define THIS_STAT ((struct stat_storage*)(fp->current_storage))
+#define THIS_STAT ((struct stat_storage*)(Pike_fp->current_storage))
 
 static void stat_index_set (INT32 args);
 
@@ -188,6 +293,22 @@ static void stat_push_compat(INT_TYPE n)
    }
 }
 
+/*! @decl void create (void|object|array stat);
+ *!
+ *! A new @[Stdio.Stat] object can be initialized in two ways:
+ *!
+ *! @ul
+ *!   @item
+ *!     @[stat] is an object, typically another @[Stdio.Stat]. The
+ *!     stat info is copied from the object by getting the values of
+ *!     @expr{mode@}, @expr{size@}, @expr{atime@}, @expr{mtime@},
+ *!     @expr{ctime@}, @expr{uid@}, @expr{gid@}, @expr{dev@}, @expr{ino@},
+ *!     @expr{nlink@}, and @expr{rdev@}.
+ *!   @item
+ *!     @[stat] is a seven element array on the 'traditional' LPC
+ *!     stat-array form (see the class doc).
+ *! @endul
+ */
 static void stat_create (INT32 args)
 {
   if (args >= 1) {
@@ -447,7 +568,7 @@ static void stat_index(INT32 args)
 	       break;
 
 	   default:
-	     fatal ("stat_index is not kept up-to-date with stat_map.\n");
+	     Pike_fatal ("stat_index is not kept up-to-date with stat_map.\n");
 	 }
       }
       else
@@ -518,7 +639,9 @@ static void stat_index_set (INT32 args)
       got_int_val = 1;
   }
 #endif
-  /* FIXME: shouldn't there be an else clause here ? */
+  /* shouldn't there be an else clause here ? */
+  /* No, the second argument is checked further below depending on
+   * what the first is. /mast */
 
   if (sp[-2].type == T_INT) {
     if (!got_int_val)
@@ -648,31 +771,31 @@ static void stat_index_set (INT32 args)
 
       case STAT_ISREG:
 	THIS_STAT->s.st_mode &= ~S_IFMT;
-	if (!IS_ZERO (sp-1)) THIS_STAT->s.st_mode |= S_IFREG;
+	if (!UNSAFE_IS_ZERO (sp-1)) THIS_STAT->s.st_mode |= S_IFREG;
 	break;
       case STAT_ISDIR:
 	THIS_STAT->s.st_mode &= ~S_IFMT;
-	if (!IS_ZERO (sp-1)) THIS_STAT->s.st_mode |= S_IFDIR;
+	if (!UNSAFE_IS_ZERO (sp-1)) THIS_STAT->s.st_mode |= S_IFDIR;
 	break;
       case STAT_ISLNK:
 	THIS_STAT->s.st_mode &= ~S_IFMT;
-	if (!IS_ZERO (sp-1)) THIS_STAT->s.st_mode |= S_IFLNK;
+	if (!UNSAFE_IS_ZERO (sp-1)) THIS_STAT->s.st_mode |= S_IFLNK;
 	break;
       case STAT_ISCHR:
 	THIS_STAT->s.st_mode &= ~S_IFMT;
-	if (!IS_ZERO (sp-1)) THIS_STAT->s.st_mode |= S_IFCHR;
+	if (!UNSAFE_IS_ZERO (sp-1)) THIS_STAT->s.st_mode |= S_IFCHR;
 	break;
       case STAT_ISBLK:
 	THIS_STAT->s.st_mode &= ~S_IFMT;
-	if (!IS_ZERO (sp-1)) THIS_STAT->s.st_mode |= S_IFBLK;
+	if (!UNSAFE_IS_ZERO (sp-1)) THIS_STAT->s.st_mode |= S_IFBLK;
 	break;
       case STAT_ISFIFO:
 	THIS_STAT->s.st_mode &= ~S_IFMT;
-	if (!IS_ZERO (sp-1)) THIS_STAT->s.st_mode |= S_IFIFO;
+	if (!UNSAFE_IS_ZERO (sp-1)) THIS_STAT->s.st_mode |= S_IFIFO;
 	break;
       case STAT_ISSOCK:
 	THIS_STAT->s.st_mode &= ~S_IFMT;
-	if (!IS_ZERO (sp-1)) THIS_STAT->s.st_mode |= S_IFSOCK;
+	if (!UNSAFE_IS_ZERO (sp-1)) THIS_STAT->s.st_mode |= S_IFSOCK;
 	break;
 
       default:
@@ -697,7 +820,7 @@ static void stat_index_set (INT32 args)
 	  case STAT_CTIME: DO_NOT_WARN(THIS_STAT->s.st_ctime = int_val); break;
 
 	  default:
-	    fatal ("stat_index_set is not kept up-to-date with stat_map.\n");
+	    Pike_fatal ("stat_index_set is not kept up-to-date with stat_map.\n");
 	}
     }
   }
@@ -783,12 +906,12 @@ static void stat_values(INT32 args)
       f_index(2);
    }
    f_aggregate(z);
-   stack_pop_n_elems_keep_top(1);
+   stack_pop_keep_top();
 }
 
 #undef THIS_STAT
 
-void push_stat(struct stat *s)
+void push_stat(PIKE_STAT_T *s)
 {
    struct object *o;
    struct stat_storage *stor;
@@ -798,6 +921,11 @@ void push_stat(struct stat *s)
    push_object(o);
 }
 
+/*! @endclass
+ */
+/*! @endmodule
+ */
+
 /* ---------------------------------------------------------------- */
 
 void init_files_stat()
@@ -854,7 +982,7 @@ void init_files_stat()
    ADD_STORAGE(struct stat_storage);
 
    ADD_FUNCTION ("create", stat_create,
-		 tFunc(tOr5(tVoid,tObjImpl_STDIO_STAT,tProgram,tMapping,tArr(tInt)),
+		 tFunc(tOr5(tVoid,tObjImpl_STDIO_STAT,tPrg(tObj),tMapping,tArr(tInt)),
 		       tVoid), ID_STATIC);
 
    ADD_FUNCTION("`[]",stat_index,
