@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: udp.c,v 1.42 2003/11/03 21:04:36 mast Exp $
+|| $Id: udp.c,v 1.67 2004/05/11 15:59:04 grubba Exp $
 */
 
 #define NO_PIKE_SHORTHAND
@@ -10,8 +10,9 @@
 
 #include "file_machine.h"
 
-RCSID("$Id: udp.c,v 1.42 2003/11/03 21:04:36 mast Exp $");
+RCSID("$Id: udp.c,v 1.67 2004/05/11 15:59:04 grubba Exp $");
 #include "fdlib.h"
+#include "pike_netlib.h"
 #include "interpret.h"
 #include "svalue.h"
 #include "stralloc.h"
@@ -29,6 +30,7 @@ RCSID("$Id: udp.c,v 1.42 2003/11/03 21:04:36 mast Exp $");
 
 #include "module_support.h"
 #include "builtin_functions.h"
+#include "file.h"
 
 #ifdef HAVE_SYS_TYPE_H
 #include <sys/types.h>
@@ -77,10 +79,6 @@ RCSID("$Id: udp.c,v 1.42 2003/11/03 21:04:36 mast Exp $");
 
 #endif /* HAVE_POLL */
 
-#ifdef HAVE_WINSOCK_H
-#include <winsock.h>
-#endif
-
 #if ! defined(EWOULDBLOCK) && defined(WSAEWOULDBLOCK)
 #define EWOULDBLOCK WSAEWOULDBLOCK
 #endif
@@ -134,23 +132,22 @@ RCSID("$Id: udp.c,v 1.42 2003/11/03 21:04:36 mast Exp $");
 #include "dmalloc.h"
 
 struct udp_storage {
-  int fd;
+  struct fd_callback_box box;	/* Must be first. */
   int my_errno;
    
   int type;
   int protocol;
 
-  struct svalue read_callback;
+  struct svalue read_callback;	/* Mapped. */
 };
 
-// FIXME: This class does not keep an extra ref while the read
-// callback is registered. It'll probably segfault in
-// udp_read_callback if the object has run out of refs. /mast
+void zero_udp(struct object *ignored);
+void exit_udp(struct object *ignored);
 
 #undef THIS
 #define THIS ((struct udp_storage *)Pike_fp->current_storage)
 #define THISOBJ (Pike_fp->current_object)
-#define FD (THIS->fd)
+#define FD (THIS->box.fd)
 
 /*! @module Stdio
  */
@@ -158,41 +155,64 @@ struct udp_storage {
 /*! @class UDP
  */
 
-extern void get_inet_addr(struct sockaddr_in *addr,char *name);
+/*! @decl int(0..1) close()
+ *!
+ *! Closes an open UDP port.
+ *!
+ *! @note
+ *!   This method was introduced in Pike 7.5.
+ */
+static void udp_close(INT32 args)
+{
+  exit_udp(NULL);
+  zero_udp(NULL);
+  pop_n_elems(args);
+  /* FIXME: Should look at the returnvalue from close(). */
+  push_int(1);
+}
 
-/*! @decl object bind(int port)
- *! @decl object bind(int port, string address)
+/*! @decl object bind(int|string port)
+ *! @decl object bind(int|string port, string address)
  *!
  *! Binds a port for recieving or transmitting UDP.
  */
 static void udp_bind(INT32 args)
 {
-  struct sockaddr_in addr;
+  PIKE_SOCKADDR addr;
+  int addr_len;
   int o;
   int fd,tmp;
 #if !defined(SOL_IP) && defined(HAVE_GETPROTOBYNAME)
   static int ip_proto_num = -1;
 #endif /* !SOL_IP && HAVE_GETPROTOBYNAME */
 
-  
-  if(args < 1) Pike_error("Too few arguments to udp->bind()\n");
+  if(args < 1)
+    SIMPLE_TOO_FEW_ARGS_ERROR("Stdio.UDP->bind", 1);
 
-  if(Pike_sp[-args].type != PIKE_T_INT)
-    Pike_error("Bad argument 1 to udp->bind()\n");
+  if(Pike_sp[-args].type != PIKE_T_INT &&
+     (Pike_sp[-args].type != PIKE_T_STRING ||
+      Pike_sp[-args].u.string->size_shift))
+    SIMPLE_BAD_ARG_ERROR("Stdio.UDP->bind", 1, "int|string (8bit)");
 
   if(FD != -1)
   {
-    set_read_callback( FD, 0, 0 );
     fd_close(FD);
-    FD = -1;
+    change_fd_for_box (&THIS->box, -1);
   }
 
-  fd = fd_socket(AF_INET, THIS->type, THIS->protocol);
+  addr_len = get_inet_addr(&addr, (args > 1 && Pike_sp[1-args].type==PIKE_T_STRING?
+				   Pike_sp[1-args].u.string->str : NULL),
+			   (Pike_sp[-args].type == PIKE_T_STRING?
+			    Pike_sp[-args].u.string->str : NULL),
+			   (Pike_sp[-args].type == PIKE_T_INT?
+			    Pike_sp[-args].u.integer : -1), 1);
+
+  fd = fd_socket(SOCKADDR_FAMILY(addr), THIS->type, THIS->protocol);
   if(fd < 0)
   {
     pop_n_elems(args);
     THIS->my_errno=errno;
-    Pike_error("UDP.bind: failed to create socket\n");
+    Pike_error("Stdio.UDP->bind: failed to create socket\n");
   }
 
   /* Make sure this fd gets closed on exec. */
@@ -203,7 +223,7 @@ static void udp_bind(INT32 args)
   {
     fd_close(fd);
     THIS->my_errno=errno;
-    Pike_error("UDP.bind: setsockopt SO_REUSEADDR failed\n");
+    Pike_error("Stdio.UDP->bind: setsockopt SO_REUSEADDR failed\n");
   }
 
 #ifndef SOL_IP
@@ -230,23 +250,12 @@ static void udp_bind(INT32 args)
 
   if (THIS->type==SOCK_RAW && THIS->protocol==255 /* raw */)
      if(fd_setsockopt(fd, SOL_IP, IP_HDRINCL, (char *)&o, sizeof(int)))
-	Pike_error("UDP.bind: setsockopt IP_HDRINCL failed\n");
+	Pike_error("Stdio.UDP->bind: setsockopt IP_HDRINCL failed\n");
 #endif /* IP_HDRINCL */
 
-  MEMSET((char *)&addr,0,sizeof(struct sockaddr_in));
-
-  if(args > 1 && Pike_sp[1-args].type==PIKE_T_STRING) {
-    get_inet_addr(&addr, Pike_sp[1-args].u.string->str);
-  } else {
-    addr.sin_addr.s_addr = htonl(INADDR_ANY);
-  }
-
-  addr.sin_port = htons( ((u_short)Pike_sp[-args].u.integer) );
-  addr.sin_family = AF_INET;
-
   THREADS_ALLOW_UID();
 
-  tmp=fd_bind(fd, (struct sockaddr *)&addr, sizeof(addr))<0;
+  tmp=fd_bind(fd, (struct sockaddr *)&addr, addr_len)<0;
 
   THREADS_DISALLOW_UID();
 
@@ -254,12 +263,12 @@ static void udp_bind(INT32 args)
   {
     fd_close(fd);
     THIS->my_errno=errno;
-    Pike_error("UDP.bind: failed to bind to port %d\n",
-	       (u_short)Pike_sp[-args].u.integer);
+    Pike_error("Stdio.UDP->bind: failed to bind to port %d\n",
+	       (unsigned INT16)Pike_sp[-args].u.integer);
     return;
   }
 
-  FD=fd;
+  change_fd_for_box (&THIS->box, fd);
   pop_n_elems(args);
   ref_push_object(THISOBJ);
 }
@@ -272,7 +281,7 @@ static void udp_bind(INT32 args)
  *! broadcast address.
  *!
  *! @returns
- *!  Returns @tt{1@} on success, @tt{0@} (zero) otherwise.
+ *!  Returns @expr{1@} on success, @expr{0@} (zero) otherwise.
  *!
  *! @note
  *!  This is normally only avalable to root users.
@@ -296,7 +305,7 @@ void udp_enable_broadcast(INT32 args)
  *! Check for data and wait max. @[timeout] seconds.
  *!
  *! @returns
- *!  Returns @tt{1@} if data are ready, @tt{0@} (zero) otherwise.
+ *!  Returns @expr{1@} if data are ready, @expr{0@} (zero) otherwise.
  */
 void udp_wait(INT32 args)
 {
@@ -377,6 +386,13 @@ void udp_wait(INT32 args)
   push_int(res);
 }
 
+/* NOTE: Some versions of AIX seem to have a
+ *         #define events reqevents
+ *       in one of the poll headerfiles. This will break
+ *       the fd_box event handling.
+ */
+#undef events
+
 #define UDP_BUFFSIZE 65536
 
 /*! @decl mapping(string:int|string) read()
@@ -389,8 +405,8 @@ void udp_wait(INT32 args)
  *! @returns
  *!  mapping(string:int|string) in the form 
  *!	([
- *!	   "data" : string recieved data
- *!	   "ip" : string   recieved from this ip
+ *!	   "data" : string received data
+ *!	   "ip" : string   received from this ip
  *!	   "port" : int    ...and this port
  *!	])
  *!
@@ -400,9 +416,9 @@ void udp_wait(INT32 args)
 void udp_read(INT32 args)
 {
   int flags = 0, res=0, fd, e;
-  struct sockaddr_in from;
+  PIKE_SOCKADDR from;
   char buffer[UDP_BUFFSIZE];
-  ACCEPT_SIZE_T fromlen = sizeof(struct sockaddr_in);
+  ACCEPT_SIZE_T fromlen = sizeof(from);
   
   if(args)
   {
@@ -423,7 +439,7 @@ void udp_read(INT32 args)
   pop_n_elems(args);
   fd = FD;
   if (FD < 0)
-    Pike_error("UDP: not open\n");
+    Pike_error("Stdio.UDP->read: not open\n");
   do {
     THREADS_ALLOW();
     res = fd_recvfrom(fd, buffer, UDP_BUFFSIZE, flags,
@@ -434,7 +450,7 @@ void udp_read(INT32 args)
     check_threads_etc();
   } while((res==-1) && (e==EINTR));
 
-  THIS->my_errno=e;
+  THIS->my_errno=errno=e;
 
   if(res<0)
   {
@@ -444,13 +460,15 @@ void udp_read(INT32 args)
        case WSAEBADF:
 #endif
        case EBADF:
-	  set_read_callback( FD, 0, 0 );
+	  if (THIS->box.backend)
+	    set_fd_callback_events (&THIS->box, 0);
 	  Pike_error("Socket closed\n");
 #ifdef ESTALE
        case ESTALE:
 #endif
        case EIO:
-	  set_read_callback( FD, 0, 0 );
+	  if (THIS->box.backend)
+	    set_fd_callback_events (&THIS->box, 0);
 	  Pike_error("I/O error\n");
        case ENOMEM:
 #ifdef ENOSR
@@ -472,19 +490,24 @@ void udp_read(INT32 args)
   /* Now comes the interresting part.
    * make a nice mapping from this stuff..
    */
-  push_text("ip");
-  push_text( inet_ntoa( from.sin_addr ) );
-
-  push_text("port");
-  push_int(ntohs(from.sin_port));
-
-  push_text("data");
+  push_constant_text("data");
   push_string( make_shared_binary_string(buffer, res) );
+
+  push_constant_text("ip");
+#ifdef HAVE_INET_NTOP
+  push_text( inet_ntop( SOCKADDR_FAMILY(from), SOCKADDR_IN_ADDR(from),
+			buffer, sizeof(buffer) ) );
+#else
+  push_text( inet_ntoa( *SOCKADDR_IN_ADDR(from) ) );
+#endif
+
+  push_constant_text("port");
+  push_int(ntohs(from.ipv4.sin_port));
   f_aggregate_mapping( 6 );
 }
 
-/*! @decl int send(string to, int port, string message)
- *! @decl int send(string to, int port, string message, int flags)
+/*! @decl int send(string to, int|string port, string message)
+ *! @decl int send(string to, int|string port, string message, int flags)
  *!
  *! Send data to a UDP socket. The recepient address will be @[to]
  *! and port will be @[port].
@@ -499,7 +522,8 @@ void udp_sendto(INT32 args)
 {
   int flags = 0, fd, e;
   ptrdiff_t res = 0;
-  struct sockaddr_in to;
+  PIKE_SOCKADDR to;
+  int to_len;
   char *str;
   ptrdiff_t len;
 
@@ -507,7 +531,7 @@ void udp_sendto(INT32 args)
     Pike_error("UDP: not open\n");
   
   check_all_args("send", args,
-		 BIT_STRING, BIT_INT, BIT_STRING, BIT_INT|BIT_VOID, 0);
+		 BIT_STRING, BIT_INT|BIT_STRING, BIT_STRING, BIT_INT|BIT_VOID, 0);
   
   if(args>3)
   {
@@ -523,22 +547,15 @@ void udp_sendto(INT32 args)
     }
     if(Pike_sp[3-args].u.integer & ~3) {
       Pike_error("Illegal 'flags' value passed to "
-		 "udp->send(string to, int port, string message, int flags)\n");
+		 "Stdio.UDP->send(string to, int|string port, string message, int flags)\n");
     }
   }
-  else if(args != 3)
-    Pike_error("Illegal number of arguments to udp->sendto(string to"
-	       ", int port, string message, void|int flags)\n");
 
-  if( Pike_sp[2-args].type!=PIKE_T_STRING ) 
-    Pike_error("Illegal type of argument to sendto, got non-string message.\n");
-
-  if( Pike_sp[-args].type==PIKE_T_STRING ) 
-    get_inet_addr(&to, Pike_sp[-args].u.string->str);
-  else
-    Pike_error("Illegal type of argument to sendto, got non-string to-address.\n");
-
-  to.sin_port = htons( ((u_short)Pike_sp[1-args].u.integer) );
+  to_len = get_inet_addr(&to, Pike_sp[-args].u.string->str,
+			 (Pike_sp[1-args].type == PIKE_T_STRING?
+			  Pike_sp[1-args].u.string->str : NULL),
+			 (Pike_sp[1-args].type == PIKE_T_INT?
+			  Pike_sp[1-args].u.integer : -1), 1);
 
   fd = FD;
   str = Pike_sp[2-args].u.string->str;
@@ -546,8 +563,7 @@ void udp_sendto(INT32 args)
 
   do {
     THREADS_ALLOW();
-    res = fd_sendto( fd, str, len, flags, (struct sockaddr *)&to,
-		     sizeof( struct sockaddr_in ));
+    res = fd_sendto( fd, str, len, flags, (struct sockaddr *)&to, to_len);
     e = errno;
     THREADS_DISALLOW();
 
@@ -563,7 +579,8 @@ void udp_sendto(INT32 args)
 #endif
 	  Pike_error("Too big message\n");
        case EBADF:
-	  set_read_callback( FD, 0, 0 );
+	  if (THIS->box.backend)
+	    set_fd_callback_events (&THIS->box, 0);
 	  Pike_error("Socket closed\n");
        case ENOMEM:
 #ifdef ENOSR
@@ -573,7 +590,8 @@ void udp_sendto(INT32 args)
        case EINVAL:
 #ifdef ENOTSOCK
        case ENOTSOCK:
-	  set_read_callback( FD, 0, 0 );
+	  if (THIS->box.backend)
+	    set_fd_callback_events (&THIS->box, 0);
 	  Pike_error("Not a socket!!!\n");
 #endif
        case EWOULDBLOCK:
@@ -585,14 +603,15 @@ void udp_sendto(INT32 args)
 }
 
 
-void zero_udp(struct object *ignored)
+void zero_udp(struct object *o)
 {
-  MEMSET(THIS, 0, sizeof(struct udp_storage));
-  THIS->read_callback.type=PIKE_T_INT;
+  THIS->box.backend = NULL;
+  THIS->box.ref_obj = o;
   FD = -1;
-
+  THIS->my_errno = 0;
   THIS->type=SOCK_DGRAM;
   THIS->protocol=0;
+  /* map_variable handles read_callback. */
 }
 
 void exit_udp(struct object *ignored)
@@ -601,46 +620,59 @@ void exit_udp(struct object *ignored)
 
   if(fd != -1)
   {
-    set_read_callback( fd, 0, 0 );
-
     THREADS_ALLOW();
     fd_close(fd);
     THREADS_DISALLOW();
   }
-  free_svalue(& THIS->read_callback );
-  THIS->read_callback.type = PIKE_T_INT; /* Avoid uncertainty. */
+
+  unhook_fd_callback_box (&THIS->box);
+
+  /* map_variable handles read_callback. */
 }
 
-#define THIS_DATA ((struct udp_storage *)data)
-
-static void udp_read_callback( int fd, void *data )
+static int got_udp_event (struct fd_callback_box *box, int event)
 {
-  if(UNSAFE_IS_ZERO(&THIS_DATA->read_callback))
-    set_read_callback(THIS_DATA->fd, 0, 0);
+  struct udp_storage *u = (struct udp_storage *) box;
+#ifdef PIKE_DEBUG
+  if (event != PIKE_FD_READ)
+    Pike_fatal ("Got unexpected event %d.\n", event);
+#endif
+
+  u->my_errno = errno;		/* Propagate backend setting. */
+
+  check_destructed (&u->read_callback);
+  if (UNSAFE_IS_ZERO (&u->read_callback))
+    set_fd_callback_events (&u->box, 0);
   else {
-    apply_svalue(& THIS_DATA->read_callback, 0);
+    apply_svalue (&u->read_callback, 0);
+    if (Pike_sp[-1].type == PIKE_T_INT && Pike_sp[-1].u.integer == -1) {
+      pop_stack();
+      return -1;
+    }
     pop_stack();
   }
-  return;
+  return 0;
 }
 
 static void udp_set_read_callback(INT32 args)
 {
-  if(FD < 0)
-    Pike_error("File is not open.\n");
-
-  if(args != 1)
-    Pike_error("Wrong number of arguments to file->set_read_callback().\n");
+  struct udp_storage *u = THIS;
+  if(args < 1)
+    SIMPLE_TOO_FEW_ARGS_ERROR("Stdio.UDP->set_read_callback", 1);
+  if(args > 1)
+    pop_n_elems(args-1);
 
+  assign_svalue(& u->read_callback, Pike_sp-1);
   if (UNSAFE_IS_ZERO (Pike_sp - 1)) {
-    free_svalue (&THIS->read_callback);
-    THIS->read_callback.type = PIKE_T_INT;
-    THIS->read_callback.u.integer = 0;
-    set_read_callback (FD, 0, 0);
+    if (u->box.backend)
+      set_fd_callback_events (&u->box, 0);
   }
   else {
-    assign_svalue(& THIS->read_callback, Pike_sp-1);
-    set_read_callback(FD, udp_read_callback, THIS);
+    if (!u->box.backend)
+      INIT_FD_CALLBACK_BOX (&u->box, default_backend, u->box.ref_obj,
+			    u->box.fd, PIKE_BIT_FD_READ, got_udp_event);
+    else
+      set_fd_callback_events (&u->box, PIKE_BIT_FD_READ);
   }
 
   pop_stack();
@@ -672,7 +704,7 @@ static void udp_set_blocking(INT32 args)
   ref_push_object(THISOBJ);
 }
 
-/*! @decl int(0..1) connect(string address, int port)
+/*! @decl int(0..1) connect(string address, int|string port)
  *!
  *!   Establish an UDP connection.
  *!
@@ -681,7 +713,7 @@ static void udp_set_blocking(INT32 args)
  *!   number for the remote machine. 
  *!
  *! @returns
- *!   Returns @tt{1@} on success, @tt{0@} (zero) otherwise.
+ *!   Returns @expr{1@} on success, @expr{0@} (zero) otherwise.
  *!
  *! @note
  *!   If the socket is in nonblocking mode, you have to wait
@@ -693,39 +725,45 @@ static void udp_set_blocking(INT32 args)
  */
 static void udp_connect(INT32 args)
 {
-  struct sockaddr_in addr;
+  PIKE_SOCKADDR addr;
+  int addr_len;
   struct pike_string *dest_addr = NULL;
-  struct pike_string *src_addr = NULL;
-  INT_TYPE dest_port = 0;
-  INT_TYPE src_port = 0;
+  struct svalue *dest_port = NULL;
 
   int tmp;
 
-  get_all_args("UDP.connect", args, "%S%i", &dest_addr, &dest_port);
+  get_all_args("UDP.connect", args, "%S%*", &dest_addr, &dest_port);
+
+  if(dest_port->type != PIKE_T_INT &&
+     (dest_port->type != PIKE_T_STRING || dest_port->u.string->size_shift))
+    SIMPLE_BAD_ARG_ERROR("UDP.connect", 2, "int|string (8bit)");
+
+  addr_len =  get_inet_addr(&addr, dest_addr->str,
+			    (dest_port->type == PIKE_T_STRING?
+			     dest_port->u.string->str : NULL),
+			    (dest_port->type == PIKE_T_INT?
+			     dest_port->u.integer : -1), 0);
 
   if(FD < 0)
   {
-     FD = fd_socket(AF_INET, SOCK_DGRAM, 0);
+     FD = fd_socket(SOCKADDR_FAMILY(addr), SOCK_DGRAM, 0);
      if(FD < 0)
      {
 	THIS->my_errno=errno;
-	Pike_error("UDP.connect: failed to create socket\n");
+	Pike_error("Stdio.UDP->connect: failed to create socket\n");
      }
      set_close_on_exec(FD, 1);
   }
 
-  get_inet_addr(&addr, dest_addr->str);
-  addr.sin_port = htons(((u_short)dest_port));
-
   tmp=FD;
   THREADS_ALLOW();
-  tmp=fd_connect(tmp, (struct sockaddr *)&addr, sizeof(addr));
+  tmp=fd_connect(tmp, (struct sockaddr *)&addr, addr_len);
   THREADS_DISALLOW();
 
   if(tmp < 0)
   {
     THIS->my_errno=errno;
-    Pike_error("UDP.connect: failed to connect\n");
+    Pike_error("Stdio.UDP->connect: failed to connect\n");
   }else{
     THIS->my_errno=0;
     pop_n_elems(args);
@@ -741,14 +779,14 @@ static void udp_connect(INT32 args)
  */
 static void udp_query_address(INT32 args)
 {
-  struct sockaddr_in addr;
+  PIKE_SOCKADDR addr;
   int i;
-  int fd = THIS->fd;
+  int fd = FD;
   char buffer[496],*q;
   ACCEPT_SIZE_T len;
 
   if(fd <0)
-    Pike_error("socket->query_address(): Port not bound yet.\n");
+    Pike_error("Stdio.UDP->query_address(): Port not bound yet.\n");
 
   THREADS_ALLOW();
 
@@ -758,18 +796,72 @@ static void udp_query_address(INT32 args)
   THREADS_DISALLOW();
 
   pop_n_elems(args);
-  if(i < 0 || len < (int)sizeof(addr))
+  if(i < 0 || len < (int)sizeof(addr.ipv4))
   {
     push_int(0);
     return;
   }
 
-  q=inet_ntoa(addr.sin_addr);
+#ifdef HAVE_INET_NTOP
+  inet_ntop(SOCKADDR_FAMILY(addr), SOCKADDR_IN_ADDR(addr),
+	    buffer, sizeof(buffer)-20);
+#else
+  q=inet_ntoa(*SOCKADDR_IN_ADDR(addr));
   strncpy(buffer,q,sizeof(buffer)-20);
   buffer[sizeof(buffer)-20]=0;
-  sprintf(buffer+strlen(buffer)," %d",(int)(ntohs(addr.sin_port)));
+#endif
+  sprintf(buffer+strlen(buffer)," %d",(int)(ntohs(addr.ipv4.sin_port)));
 
-  push_string(make_shared_string(buffer));
+  push_text(buffer);
+}
+
+/*! @decl void set_backend (Pike.Backend backend)
+ *!
+ *! Set the backend used for the read callback.
+ *!
+ *! @note
+ *! The backend keeps a reference to this object as long as there can
+ *! be calls to the read callback, but this object does not keep a
+ *! reference to the backend.
+ *!
+ *! @seealso
+ *!   @[query_backend]
+ */
+static void udp_set_backend (INT32 args)
+{
+  struct udp_storage *u = THIS;
+  struct Backend_struct *backend;
+
+  if (!args)
+    SIMPLE_TOO_FEW_ARGS_ERROR ("Stdio.UDP->set_backend", 1);
+  if (Pike_sp[-args].type != PIKE_T_OBJECT)
+    SIMPLE_BAD_ARG_ERROR ("Stdio.UDP->set_backend", 1, "object(Pike.Backend)");
+  backend = (struct Backend_struct *)
+    get_storage (Pike_sp[-args].u.object, Backend_program);
+  if (!backend)
+    SIMPLE_BAD_ARG_ERROR ("Stdio.UDP->set_backend", 1, "object(Pike.Backend)");
+
+  if (u->box.backend)
+    change_backend_for_box (&u->box, backend);
+  else
+    INIT_FD_CALLBACK_BOX (&u->box, backend, u->box.ref_obj,
+			  u->box.fd, 0, got_udp_event);
+
+  pop_n_elems (args - 1);
+}
+
+/*! @decl Pike.Backend query_backend()
+ *!
+ *! Return the backend used for the read callback.
+ *!
+ *! @seealso
+ *!   @[set_backend]
+ */
+static void udp_query_backend (INT32 args)
+{
+  pop_n_elems (args);
+  ref_push_object (get_backend_obj (THIS->box.backend ? THIS->box.backend :
+				    default_backend));
 }
 
 /*! @decl int errno()
@@ -793,9 +885,9 @@ static void udp_set_type(INT32 args)
    INT_TYPE type=0;
    INT_TYPE proto=0;
    if (args<2)
-      get_all_args("UDP.set_type",args,"%i",&type);
+      get_all_args("Stdio.UDP->set_type",args,"%i",&type);
    else
-      get_all_args("UDP.set_type",args,"%i%i",&type,&proto);
+      get_all_args("Stdio.UDP->set_type",args,"%i%i",&type,&proto);
 
    THIS->type=(int)type;
    THIS->protocol=(int)proto;
@@ -834,17 +926,22 @@ static void udp_get_type(INT32 args)
 
 void init_udp(void)
 {
-  start_new_program();
+  START_NEW_PROGRAM_ID (STDIO_UDP);
 
   ADD_STORAGE(struct udp_storage);
 
+  PIKE_MAP_VARIABLE("_read_callback", OFFSETOF(udp_storage, read_callback),
+		    tFunc(tNone, tInt_10), PIKE_T_MIXED, ID_STATIC|ID_PRIVATE);
+
   ADD_FUNCTION("set_type",udp_set_type,
 	       tFunc(tInt tOr(tVoid,tInt),tObj),0);
   ADD_FUNCTION("get_type",udp_get_type,
 	       tFunc(tNone,tArr(tInt)),0);
 
+  ADD_FUNCTION("close", udp_close, tFunc(tNone, tInt01), 0);
+
   ADD_FUNCTION("bind",udp_bind,
-	       tFunc(tInt tOr(tVoid,tStr),tObj),0);
+	       tFunc(tOr(tInt,tStr) tOr(tVoid,tStr),tObj),0);
 
   ADD_FUNCTION("enable_broadcast", udp_enable_broadcast,
 	       tFunc(tNone,tVoid), 0);
@@ -860,10 +957,10 @@ void init_udp(void)
 #endif /* MSG_PEEK */
 
   ADD_FUNCTION("send",udp_sendto,
-	       tFunc(tStr tInt tStr tOr(tVoid,tInt),tInt),0);
+	       tFunc(tStr tOr(tInt,tStr) tStr tOr(tVoid,tInt),tInt),0);
 
   ADD_FUNCTION("connect",udp_connect,
-	       tFunc(tString tInt,tInt),0);
+	       tFunc(tString tOr(tInt,tStr),tInt),0);
   
   ADD_FUNCTION("_set_nonblocking", udp_set_nonblocking,
 	       tFunc(tOr(tFunc(tVoid,tVoid),tVoid),tObj), 0 );
@@ -873,6 +970,9 @@ void init_udp(void)
   ADD_FUNCTION("set_blocking",udp_set_blocking,tFunc(tVoid,tObj), 0 );
   ADD_FUNCTION("query_address",udp_query_address,tFunc(tNone,tStr),0);
 
+  ADD_FUNCTION ("set_backend", udp_set_backend, tFunc(tObj,tVoid), 0);
+  ADD_FUNCTION ("query_backend", udp_query_backend, tFunc(tVoid,tObj), 0);
+
   ADD_FUNCTION("errno",udp_errno,tFunc(tNone,tInt),0);
 
   set_init_callback(zero_udp);
@@ -880,7 +980,7 @@ void init_udp(void)
 
   end_class("UDP",0);
 
-  start_new_program();
+  START_NEW_PROGRAM_ID (STDIO_SOCK);
 #ifdef SOCK_STREAM
   add_integer_constant("STREAM",SOCK_STREAM,0);
 #endif
@@ -904,7 +1004,7 @@ void init_udp(void)
    simple_add_constant("SOCK",Pike_sp-1,0);
    pop_stack();
 
-   start_new_program();
+   START_NEW_PROGRAM_ID (STDIO_IPPROTO);
    add_integer_constant("IP",0,0);       /* Dummy protocol for TCP.  */
    add_integer_constant("HOPOPTS",0,0);  /* IPv6 Hop-by-Hop options.  */
    add_integer_constant("ICMP",1,0);     /* Internet Control Message Protocol.  */
