@@ -1,8 +1,14 @@
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: sybase.c,v 1.13 2003/08/18 11:29:59 mast Exp $
+*/
+
 /*
  * Sybase driver for the Pike programming language.
  *
  * By Francesco Chemolli <kinkie@roxen.com> 10/12/1999
- * (C) Roxen IS
  *
  */
 
@@ -24,7 +30,7 @@
 #include "sybase_config.h"
 #include "global.h"
 
-RCSID("$Id: sybase.c,v 1.6 2000/12/05 21:08:38 per Exp $");
+RCSID("$Id: sybase.c,v 1.13 2003/08/18 11:29:59 mast Exp $");
 
 #ifdef HAVE_SYBASE
 
@@ -44,8 +50,7 @@ RCSID("$Id: sybase.c,v 1.6 2000/12/05 21:08:38 per Exp $");
 #include "sybase.h"
 
 
-/* This must be included last! */
-#include "module_magic.h"
+#define sp Pike_sp
 
 /* define this to enable debugging */
 /* #define SYBDEBUG */
@@ -105,7 +110,7 @@ static MUTEX_T mainlock;
 #define FAILED(status) (status!=CS_SUCCEED)
 #define OK(status) (status==CS_SUCCEED)
 
-#define THIS ((pike_sybase_connection *) (fp->current_storage))
+#define THIS ((pike_sybase_connection *) (Pike_fp->current_storage))
 
 
 
@@ -314,7 +319,7 @@ static void sybase_destroy (struct object * o) {
     }
     this->busy--;
     sybdebug((stderr,"Busy status: %d\n",this->busy));
-    /* if we fail, it's useless anyways. Maybe we should fatal() */
+    /* if we fail, it's useless anyways. Maybe we should Pike_fatal() */
   }
   
   if (this->cmd) {
@@ -325,7 +330,7 @@ static void sybase_destroy (struct object * o) {
       sybdebug((stderr,"\tHm... failed\n"));
     }
     this->cmd=NULL; 
-    /* if we fail, it's useless anyways. Maybe we should fatal() */
+    /* if we fail, it's useless anyways. Maybe we should Pike_fatal() */
   }
 
   if (this->results) {
@@ -356,7 +361,7 @@ static void sybase_destroy (struct object * o) {
       sybdebug((stderr,"Gasp! Failed!\n"));
     }
     this->connection=NULL;
-    /* if we fail, it's useless anyways. Maybe we should fatal() */
+    /* if we fail, it's useless anyways. Maybe we should Pike_fatal() */
   }
 
   SYB_LOCK(mainlock); /* this is really needed only here */
@@ -374,7 +379,7 @@ static void sybase_destroy (struct object * o) {
       sybdebug((stderr,"\tGosh Robin! You're right!\n"));
     }
     this->context=NULL;
-    /* if we fail, it's useless anyways. Maybe we should fatal() */
+    /* if we fail, it's useless anyways. Maybe we should Pike_fatal() */
   }
 
   SYB_UNLOCK(mainlock);
@@ -536,7 +541,7 @@ static void f_create (INT32 args) {
   
   /* if connected, disconnect */
   if (this->context)
-    sybase_destroy(fp->current_object);
+    sybase_destroy(Pike_fp->current_object);
 
   THREADS_ALLOW();
   SYB_LOCK(mainlock);
@@ -722,7 +727,7 @@ static void f_big_query(INT32 args) {
 /*       this->busy--; */
       flush_results_queue(this);
       break;
-    default: /* Ok */
+    default:; /* Ok */
     }
 
     if (err||done) break; /* get out of the while cycle */
@@ -897,7 +902,7 @@ static void f_big_query(INT32 args) {
     push_int(0);
     break;
   case PS_RESULT_THIS:
-    ref_push_object(fp->current_object);
+    ref_push_object(Pike_fp->current_object);
     break;
   case PS_RESULT_STATUS:
     this->busy--;
@@ -906,7 +911,7 @@ static void f_big_query(INT32 args) {
     function_result=NULL;
     break;
   default:
-    fatal("Internal error! Wrong result in big_query\n");
+    Pike_fatal("Internal error! Wrong result in big_query\n");
     break;
   }
   /* extra safety check. Paranoia. */
@@ -1178,12 +1183,12 @@ static void f_fetch_fields(INT32 args) {
 /********/
 
 static struct program* sybase_program;
-void pike_module_exit (void) {
+PIKE_MODULE_EXIT {
   SYB_MT_EXIT(mainlock);
   if(sybase_program) free_program(sybase_program);
 }
 
-void pike_module_init (void) {
+PIKE_MODULE_INIT {
 
   sybdebug((stderr,"sybase driver release " SYBASE_DRIVER_VERSION "\n"));
 
@@ -1203,10 +1208,10 @@ void pike_module_init (void) {
                                          tOr(tInt,tVoid), tVoid),
                0);
   ADD_FUNCTION("error",f_error,tFunc(tVoid,tOr(tVoid,tStr)),
-               OPT_RETURN);
+	       0);
 
   ADD_FUNCTION("big_query",f_big_query,tFunc(tString,tOr(tInt,tObj)),
-               OPT_RETURN);
+	       0);
 
   ADD_FUNCTION("fetch_row", f_fetch_row,
                tFunc(tVoid,tOr(tVoid,tArr(tMix))), 0);
@@ -1252,8 +1257,8 @@ void pike_module_init (void) {
 
 #else /* HAVE_SYBASE */
 /* This must be included last! */
-#include "module_magic.h"
+#include "module.h"
 
-void pike_module_init (void) {}
-void pike_module_exit (void) {}
+PIKE_MODULE_INIT {}
+PIKE_MODULE_EXIT {}
 #endif /* HAVE_SYBASE */
