@@ -2,14 +2,12 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: multiset.h,v 1.29 2002/11/24 22:47:06 mast Exp $
+|| $Id: multiset.h,v 1.34 2004/04/18 02:16:06 mast Exp $
 */
 
 #ifndef MULTISET_H
 #define MULTISET_H
 
-#ifdef PIKE_NEW_MULTISETS
-
 /* Multisets using rbtree.
  *
  * Created by Martin Stjernholm 2001-05-07
@@ -153,7 +151,7 @@ PMOD_EXPORT void multiset_clear_node_refs (struct multiset *l);
 
 #ifdef PIKE_DEBUG
 /* To get good type checking. */
-static inline union msnode *msnode_check (union msnode *x)
+static INLINE union msnode *msnode_check (union msnode *x)
   {return x;}
 PMOD_EXPORT extern const char msg_no_multiset_flag_marker[];
 #else
@@ -190,7 +188,7 @@ union msnode *low_multiset_find_eq (struct multiset *l, struct svalue *key);
 #define low_use_multiset_index(NODE, VAR)				\
   ((VAR) = msnode_check (NODE)->i.ind,					\
    DO_IF_DEBUG ((VAR).type & MULTISET_FLAG_MARKER ? 0 :			\
-		Pike_fatal (msg_no_multiset_flag_marker) COMMA)		\
+		(Pike_fatal (msg_no_multiset_flag_marker), 0) COMMA)	\
    (VAR).type &= ~MULTISET_FLAG_MASK,					\
    &(VAR))
 
@@ -226,6 +224,10 @@ PMOD_PROTO void really_free_multiset (struct multiset *l);
 #define free_multiset(L) do {						\
     struct multiset *_ms_ = (L);					\
     debug_malloc_touch (_ms_);						\
+    DO_IF_PIKE_CLEANUP (						\
+      if (gc_external_refs_zapped)					\
+	gc_check_zapped (_ms_, PIKE_T_MULTISET, __FILE__, __LINE__);	\
+    );									\
     if (!sub_ref (_ms_)) really_free_multiset (_ms_);			\
   } while (0)
 
@@ -420,7 +422,7 @@ void gc_mark_all_multisets (void);
 void gc_zap_ext_weak_refs_in_multisets (void);
 void real_gc_cycle_check_multiset (struct multiset *l, int weak);
 void gc_cycle_check_all_multisets (void);
-void gc_free_all_unreferenced_multisets (void);
+size_t gc_free_all_unreferenced_multisets (void);
 #define gc_cycle_check_multiset(X, WEAK) \
   gc_cycle_enqueue ((gc_cycle_check_cb *) real_gc_cycle_check_multiset, (X), (WEAK))
 
@@ -435,68 +437,4 @@ void init_multiset (void);
 void exit_multiset (void);
 void test_multiset (void);
 
-#else  /* PIKE_NEW_MULTISETS */
-
-#include "las.h"
-
-struct multiset
-{
-  PIKE_MEMORY_OBJECT_MEMBERS;  /* Must be first */
-
-  struct multiset *next,*prev;
-  struct array *ind;
-};
-
-extern struct multiset *first_multiset;
-extern struct multiset *gc_internal_multiset;
-
-#define free_multiset(L) do{ struct multiset *l_=(L); debug_malloc_touch(l_); if(!sub_ref(l_)) really_free_multiset(l_); }while(0)
-
-#define l_sizeof(L) ((L)->ind->size)
-#define multiset_sizeof(L) l_sizeof (L)
-#define multiset_is_empty(L) (!multiset_sizeof (L))
-#define multiset_ind_types(L) ((L)->ind->type_field)
-#define multiset_get_flags(L) ((L)->ind->flags)
-#define multiset_fix_type_field(L) array_fix_type_field ((L)->ind)
-
-/* Prototypes begin here */
-PMOD_EXPORT int multiset_member(struct multiset *l, struct svalue *ind);
-PMOD_EXPORT struct multiset *allocate_multiset(struct array *ind);
-PMOD_EXPORT void really_free_multiset(struct multiset *l);
-PMOD_EXPORT void do_free_multiset(struct multiset *l);
-PMOD_EXPORT void order_multiset(struct multiset *l);
-PMOD_EXPORT struct multiset *mkmultiset(struct array *ind);
-PMOD_EXPORT void multiset_insert(struct multiset *l,
-				 struct svalue *v);
-struct array *multiset_indices(struct multiset *l);
-PMOD_EXPORT void multiset_delete(struct multiset *l,struct svalue *v);
-PMOD_EXPORT void check_multiset_for_destruct(struct multiset *l);
-PMOD_EXPORT struct multiset *copy_multiset(struct multiset *tmp);
-PMOD_EXPORT struct multiset *merge_multisets(struct multiset *a,
-			  struct multiset *b,
-			  INT32 operator);
-PMOD_EXPORT struct multiset *add_multisets(struct svalue *argp,INT32 args);
-int multiset_equal_p(struct multiset *a, struct multiset *b, struct processing *p);
-void describe_multiset(struct multiset *l,struct processing *p,int indent);
-node * make_node_from_multiset(struct multiset *l);
-PMOD_EXPORT void f_aggregate_multiset(INT32 args);
-struct multiset *copy_multiset_recursively(struct multiset *l,
-				   struct processing *p);
-void gc_mark_multiset_as_referenced(struct multiset *l);
-void real_gc_cycle_check_multiset(struct multiset *l, int weak);
-unsigned gc_touch_all_multisets(void);
-void gc_check_all_multisets(void);
-void gc_mark_all_multisets(void);
-void gc_cycle_check_all_multisets(void);
-void gc_free_all_unreferenced_multisets(void);
-void count_memory_in_multisets(INT32 *num_, INT32 *size_);
-int multiset_is_constant(struct multiset *m,
-			 struct processing *p);
-/* Prototypes end here */
-
-#define gc_cycle_check_multiset(X, WEAK) \
-  gc_cycle_enqueue((gc_cycle_check_cb *) real_gc_cycle_check_multiset, (X), (WEAK))
-
-#endif /* PIKE_NEW_MULTISETS */
-
 #endif /* MULTISET_H */
