@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: preprocessor.h,v 1.57 2004/06/30 17:57:16 grubba Exp $
+|| $Id: preprocessor.h,v 1.64 2004/04/22 15:43:51 grubba Exp $
 */
 
 /*
@@ -431,13 +431,14 @@ static ptrdiff_t calcC(struct cpp *this, WCHAR *data, ptrdiff_t len,
   }
   
   default:
-    fprintf(stderr, "Bad char %c (%d)\n", data[pos], data[pos]);
+    /* fprintf(stderr, "Bad char %c (%d)\n", data[pos], data[pos]); */
 #ifdef PIKE_DEBUG
     if(WC_ISIDCHAR(data[pos]))
       cpp_error(this, "Syntax error in #if (should not happen).");
 #endif
 
-    cpp_error(this, "Syntax error in #if.");
+    cpp_error_sprintf(this, "Syntax error in #if bad character %c (%d).",
+		      data[pos], data[pos]);
   }
   
 
@@ -800,17 +801,7 @@ static ptrdiff_t calc(struct cpp *this, WCHAR *data, ptrdiff_t len,
 
   if (SETJMP(recovery))
   {
-    struct svalue thrown = throw_value;
-    throw_value.type = T_INT;
-
-    cpp_error(this, "Error evaluating expression.");
-
-    push_svalue(&thrown);
-    low_safe_apply_handler("compile_exception", error_handler, compat_handler, 1);
-    if (SAFE_IS_ZERO(sp-1)) cpp_describe_exception(this, &thrown);
-    pop_stack();
-    free_svalue(&thrown);
-
+    cpp_handle_exception (this, "Error evaluating expression.");
     pos=tmp;
     FIND_EOL();
     push_int(0);
@@ -944,7 +935,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	  if(d->args>=0)
 	  {
 	    SKIPWHITE();
-	    
+
 	    if(!GOBBLE('('))
 	    {
 	      if (s) {
@@ -964,8 +955,8 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 		SKIPWHITE();
 		if(data[pos]==')')
 		{
- 		  if((d->varargs && arg + 1 == d->args) ||
-		     (!arg && (d->args == 1))) {
+		  if(d->varargs && arg + 1 == d->args)
+		  {
 		    arguments[arg].arg = MKPCHARP(data + pos, SHIFT);
 		    arguments[arg].len=0;
 		    continue;
@@ -1004,6 +995,10 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 		default: continue;
 		  
 		case '"':
+		  /* Note: Strings may contain \-escaped newlines.
+		   *       They must be removed on insertion to
+		   *       avoid being counted twice.
+		   */
 		  if(data[pos-2]!='#') {
 		    FIND_END_OF_STRING();
 		  }else{
@@ -1145,6 +1140,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 		}else{
 		  struct string_builder save;
 		  INT32 line=this->current_line;
+		  /* FIXME: Shouldn't we save current_file too? */
 		  save=this->buf;
 		  this->buf=tmp;
 		  d->inside = 2;
@@ -1433,37 +1429,25 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	  if(OUTP())
 	  {
 	    struct pike_string *new_file;
-	    
-	    if(this->compat_handler)
-	    {
-	      safe_apply(this->compat_handler,"handle_include",3);
-	    }else{
-	      SAFE_APPLY_MASTER("handle_include",3);
-	    }
-	  
-	    if(Pike_sp[-1].type != PIKE_T_STRING)
-	    {
-	      cpp_error(this, "Couldn't include file.");
+
+	    if (!safe_apply_handler ("handle_include",
+				     this->handler, this->compat_handler,
+				     3, BIT_STRING) ||
+		!(new_file=Pike_sp[-1].u.string) ) {
+	      cpp_handle_exception (this, "Couldn't include file.");
 	      pop_n_elems(Pike_sp-save_sp);
 	      break;
 	    }
-	    
-	    new_file=Pike_sp[-1].u.string;
 
 	    /* Why not just use ref_push_string(new_file)? */
 	    assign_svalue_no_free(Pike_sp,Pike_sp-1);
 	    Pike_sp++;
-	    
-	    if(this->compat_handler)
-	    {
-	      safe_apply(this->compat_handler,"read_include",1);
-	    }else{
-	      SAFE_APPLY_MASTER("read_include",1);
-	    }
-	    
-	    if(Pike_sp[-1].type != PIKE_T_STRING)
-	    {
-	      cpp_error(this, "Couldn't read include file.");
+	    dmalloc_touch_svalue(Pike_sp-1);
+
+	    if (!safe_apply_handler ("read_include",
+				     this->handler, this->compat_handler,
+				     1, BIT_STRING)) {
+	      cpp_handle_exception (this, "Couldn't read include file.");
 	      pop_n_elems(Pike_sp-save_sp);
 	      break;
 	    }
@@ -1491,15 +1475,17 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	      }else{
 		/* #include */
 		if (auto_convert) {
-		  struct pike_string *new_str = recode_string(Pike_sp[-1].u.string);
+		  struct pike_string *new_str =
+		    recode_string(this, Pike_sp[-1].u.string);
 		  free_string(Pike_sp[-1].u.string);
 		  Pike_sp[-1].u.string = new_str;
 		} else if (charset) {
 		  ref_push_string(charset);
-		  SAFE_APPLY_MASTER("decode_charset", 2);
-		  if (Pike_sp[-1].type != PIKE_T_STRING) {
-		    cpp_error(this,
-			      "Charset decoding failed for included file.");
+		  if (!safe_apply_handler ("decode_charset",
+					   this->handler, this->compat_handler,
+					   2, BIT_STRING)) {
+		    cpp_handle_exception (this,
+					  "Charset decoding failed for included file.");
 		    pop_n_elems(Pike_sp - save_sp);
 		    break;
 		  }
@@ -2102,11 +2088,9 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	  MEMCPY(s->str, data + p, pos - p);
 	  push_string(end_shared_string(s));
 
-	  SAFE_APPLY_MASTER("decode_charset", 2);
-
-	  if (Pike_sp[-1].type != PIKE_T_STRING) {
-	    pop_stack();
-	    cpp_error(this, "Unknown charset.");
+	  if (!safe_apply_handler ("decode_charset", this->handler, this->compat_handler,
+				   2, BIT_STRING)) {
+	    cpp_handle_exception (this, NULL);
 	  } else {
 	    low_cpp(this, Pike_sp[-1].u.string->str, Pike_sp[-1].u.string->len,
 		    Pike_sp[-1].u.string->size_shift, flags,
@@ -2170,6 +2154,34 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	  break;
 	}
       }
+    case 'w': /* warning */
+      {
+	static WCHAR warning_[] = { 'w', 'a', 'r', 'n', 'i', 'n', 'g' };
+
+	if(WGOBBLE2(warning_))
+	{
+	  ptrdiff_t foo;
+
+          SKIPSPACE();
+          foo=pos;
+          FIND_EOL();
+	  if(OUTP())
+	  {
+#if (SHIFT == 0)
+	    push_string(make_shared_binary_string((char *)data+foo, pos-foo));
+#else /* SHIFT != 0 */
+#if (SHIFT == 1)
+	    push_string(make_shared_binary_string1(data+foo, pos-foo));
+#else /* SHIFT != 1 */
+	    push_string(make_shared_binary_string2(data+foo, pos-foo));
+#endif /* SHIFT == 1 */
+#endif /* SHIFT == 0 */
+	    cpp_warning(this, Pike_sp[-1].u.string->str);
+	  }
+	  break;
+	}
+	goto unknown_preprocessor_directive;
+      }
     default:
     unknown_preprocessor_directive:
       {
