@@ -1,12 +1,10 @@
-/*\
-||| This file a part of Pike, and is copyright by Fredrik Hubinette
-||| Pike is distributed as GPL (General Public License)
-||| See the files COPYING and DISCLAIMER for more information.
-\*/
-
 /*
- * $Id: stralloc.h,v 1.60 2001/07/04 00:03:03 hubbe Exp $
- */
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: stralloc.h,v 1.81 2004/04/15 17:34:40 mast Exp $
+*/
+
 #ifndef STRALLOC_H
 #define STRALLOC_H
 #include "global.h"
@@ -18,23 +16,24 @@
 #ifndef STRUCT_PIKE_STRING_DECLARED
 #define STRUCT_PIKE_STRING_DECLARED
 #endif
+
 #define PIKE_STRING_CONTENTS						\
   INT32 refs;								\
   INT32 size_shift; /* 30 bit waste, but good for alignment... */	\
-  ptrdiff_t len;							\
+  ptrdiff_t len; /* Not counting terminating NUL. */			\
   size_t hval;								\
   struct pike_string *next 
 
 struct pike_string
 {
   PIKE_STRING_CONTENTS;
-  char str[1];
+  char str[1];			/* NUL terminated. */
 };
 
 struct string_builder
 {
   struct pike_string *s;
-  size_t malloced;
+  ptrdiff_t malloced;
   size_t known_shift;
 };
 
@@ -42,10 +41,14 @@ struct string_builder
 PMOD_EXPORT struct pike_string *debug_findstring(const struct pike_string *foo);
 #endif
 
-#define free_string(s) do{ struct pike_string *_=(s); debug_malloc_touch(_); if(--_->refs<=0) really_free_string(_); }while(0)
+#define free_string(s) do{ \
+    struct pike_string *_=(s); \
+    debug_malloc_touch(_); \
+    if(sub_ref(_)<=0) \
+      really_free_string(_); \
+  } while(0)
 
-#define my_hash_string(X) ((size_t)(X))
-#define my_order_strcmp(X,Y) ((char *)(X)-(char *)(Y))
+#define my_hash_string(X) PTR_TO_INT(X)
 #define is_same_string(X,Y) ((X)==(Y))
 
 #ifdef PIKE_DEBUG
@@ -77,8 +80,8 @@ PMOD_EXPORT struct pike_string *debug_findstring(const struct pike_string *foo);
 #define LOW_SUBTRACT_PCHARP(X,Y) (LOW_COMPARE_PCHARP((X),-,(Y))>>(X).shift)
 
 #ifdef PIKE_DEBUG
-#define SUBTRACT_PCHARP(X,Y)    ((X).shift!=(Y).shift?(fatal("Subtracting different size charp!\n")),0:LOW_SUBTRACT_PCHARP((X),(Y)))
-#define COMPARE_PCHARP(X,CMP,Y) ((X).shift!=(Y).shift?(fatal("Subtracting different size charp!\n")),0:LOW_COMPARE_PCHARP((X),CMP,(Y)))
+#define SUBTRACT_PCHARP(X,Y)    ((X).shift!=(Y).shift?(Pike_fatal("Subtracting different size charp!\n")),0:LOW_SUBTRACT_PCHARP((X),(Y)))
+#define COMPARE_PCHARP(X,CMP,Y) ((X).shift!=(Y).shift?(Pike_fatal("Comparing different size charp!\n")),0:LOW_COMPARE_PCHARP((X),CMP,(Y)))
 #else
 #define SUBTRACT_PCHARP(X,Y) LOW_SUBTRACT_PCHARP((X),(Y))
 #define COMPARE_PCHARP(X,CMP,Y) LOW_COMPARE_PCHARP((X),CMP,(Y))
@@ -89,7 +92,7 @@ PMOD_EXPORT struct pike_string *debug_findstring(const struct pike_string *foo);
 static INLINE PCHARP MKPCHARP(void *ptr, int shift)
 {
   PCHARP tmp;
-  tmp.ptr=ptr;
+  tmp.ptr=(p_wchar0 *)ptr;
   tmp.shift=shift;
   return tmp;
 }
@@ -102,8 +105,8 @@ static INLINE PCHARP MKPCHARP(void *ptr, int shift)
 
 
 #ifdef DEBUG_MALLOC
-#define reference_shared_string(s) do { struct pike_string *S_=(s); debug_malloc_touch(S_); S_->refs++; }while(0)
-#define copy_shared_string(to,s) do { struct pike_string *S_=(to)=(s); debug_malloc_touch(S_); S_->refs++; }while(0)
+#define reference_shared_string(s) do { struct pike_string *S_=(s); add_ref(S_); }while(0)
+#define copy_shared_string(to,s) do { struct pike_string *S_=(to)=(s); add_ref(S_); }while(0)
 
 struct shared_string_location
 {
@@ -113,30 +116,38 @@ struct shared_string_location
 
 extern struct shared_string_location *all_shared_string_locations;
 
-#define MAKE_CONSTANT_SHARED_STRING(var, text) do {	\
+#define MAKE_CONST_STRING(var, text) do {		\
   static struct shared_string_location str_;		\
   if(!str_.s) { 					\
-    str_.s=make_shared_binary_string((text),CONSTANT_STRLEN(text));			\
+    str_.s=make_shared_binary_string((text),CONSTANT_STRLEN(text)); \
     str_.next=all_shared_string_locations;		\
     all_shared_string_locations=&str_;			\
   }							\
- copy_shared_string((var),str_.s);			\
+  var = str_.s;						\
 }while(0)
 
-
 #else
 
-#define reference_shared_string(s) (s)->refs++
-#define copy_shared_string(to,s) ((to)=(s))->refs++
+#define reference_shared_string(s) add_ref(s)
+#define copy_shared_string(to,s) add_ref((to)=(s))
 
-#define MAKE_CONSTANT_SHARED_STRING(var, text)                                  \
+#define MAKE_CONST_STRING(var, text)						\
  do { static struct pike_string *str_;                                          \
     if(!str_) str_=make_shared_binary_string((text),CONSTANT_STRLEN(text));     \
-    copy_shared_string((var), str_);                                            \
+    var = str_;									\
  }while(0)
 
 #endif
 
+#define REF_MAKE_CONST_STRING(var, text) do {				\
+    MAKE_CONST_STRING(var, text);					\
+    reference_shared_string(var);					\
+  } while (0)
+
+/* Compatibility. */
+#define MAKE_CONSTANT_SHARED_STRING(var, text)				\
+  REF_MAKE_CONST_STRING(var, text)
+
 #define convert_0_to_0(X,Y,Z) MEMCPY((char *)(X),(char *)(Y),(Z))
 #define convert_1_to_1(X,Y,Z) MEMCPY((char *)(X),(char *)(Y),(Z)<<1)
 #define convert_2_to_2(X,Y,Z) MEMCPY((char *)(X),(char *)(Y),(Z)<<2)
@@ -146,16 +157,17 @@ extern struct shared_string_location *all_shared_string_locations;
 #define compare_2_to_2(X,Y,Z) MEMCMP((char *)(X),(char *)(Y),(Z)<<2)
 
 #define CONVERT(FROM,TO) \
-INLINE void PIKE_CONCAT4(convert_,FROM,_to_,TO)(PIKE_CONCAT(p_wchar,TO) *to, const PIKE_CONCAT(p_wchar,FROM) *from, ptrdiff_t len); \
-INLINE INT32 PIKE_CONCAT4(compare_,FROM,_to_,TO)(const PIKE_CONCAT(p_wchar,TO) *to, const PIKE_CONCAT(p_wchar,FROM) *from, ptrdiff_t len);
+void PIKE_CONCAT4(convert_,FROM,_to_,TO)(PIKE_CONCAT(p_wchar,TO) *to, const PIKE_CONCAT(p_wchar,FROM) *from, ptrdiff_t len); \
+INT32 PIKE_CONCAT4(compare_,FROM,_to_,TO)(const PIKE_CONCAT(p_wchar,TO) *to, const PIKE_CONCAT(p_wchar,FROM) *from, ptrdiff_t len);
 
+PMOD_EXPORT extern struct pike_string *empty_pike_string;
 
 /* Prototypes begin here */
-PMOD_EXPORT INLINE unsigned INT32 index_shared_string(struct pike_string *s,
-						      ptrdiff_t pos);
-PMOD_EXPORT INLINE void low_set_index(struct pike_string *s, ptrdiff_t pos,
-				      int value);
-PMOD_EXPORT INLINE struct pike_string *debug_check_size_shift(struct pike_string *a,int shift);
+PMOD_EXPORT unsigned INT32 index_shared_string(struct pike_string *s,
+					       ptrdiff_t pos);
+PMOD_EXPORT void low_set_index(struct pike_string *s, ptrdiff_t pos,
+			       int value);
+PMOD_EXPORT struct pike_string *debug_check_size_shift(struct pike_string *a,int shift);
 CONVERT(0,1)
 CONVERT(0,2)
 CONVERT(1,0)
@@ -167,8 +179,7 @@ PMOD_EXPORT int generic_compare_strings(const void *a, ptrdiff_t alen, int asize
 PMOD_EXPORT void generic_memcpy(PCHARP to,
 				PCHARP from,
 				ptrdiff_t len);
-PMOD_EXPORT INLINE void pike_string_cpy(PCHARP to,
-			    struct pike_string *from);
+PMOD_EXPORT void pike_string_cpy(PCHARP to, struct pike_string *from);
 PMOD_EXPORT struct pike_string *binary_findstring(const char *foo, ptrdiff_t l);
 PMOD_EXPORT struct pike_string *findstring(const char *foo);
 struct short_pike_string0;
@@ -225,7 +236,7 @@ PMOD_EXPORT struct pike_string *realloc_shared_string(struct pike_string *a,
 						      ptrdiff_t size);
 PMOD_EXPORT struct pike_string *new_realloc_shared_string(struct pike_string *a, INT32 size, int shift);
 PMOD_EXPORT struct pike_string *modify_shared_string(struct pike_string *a,
-					 INT32 index,
+					 INT32 position,
 					 INT32 c);
 PMOD_EXPORT struct pike_string *add_shared_strings(struct pike_string *a,
 					 struct pike_string *b);
@@ -245,11 +256,16 @@ void cleanup_shared_string_table(void);
 void count_memory_in_strings(INT32 *num, INT32 *size);
 unsigned gc_touch_all_strings(void);
 void gc_mark_all_strings(void);
+struct pike_string *next_pike_string (struct pike_string *s);
 PMOD_EXPORT void init_string_builder(struct string_builder *s, int mag);
-PMOD_EXPORT void init_string_builder_alloc(struct string_builder *s,
-					   ptrdiff_t length, int mag);
-PMOD_EXPORT void *string_builder_allocate(struct string_builder *s,
-					  ptrdiff_t chars, int mag);
+PMOD_EXPORT void init_string_builder_alloc(struct string_builder *s, ptrdiff_t length, int mag);
+PMOD_EXPORT void init_string_builder_copy(struct string_builder *to,
+					  struct string_builder *from);
+PMOD_EXPORT int init_string_builder_with_string (struct string_builder *s,
+						 struct pike_string *str);
+PMOD_EXPORT void string_build_mkspace(struct string_builder *s,
+				      ptrdiff_t chars, int mag);
+PMOD_EXPORT void *string_builder_allocate(struct string_builder *s, ptrdiff_t chars, int mag);
 PMOD_EXPORT void string_builder_putchar(struct string_builder *s, int ch);
 PMOD_EXPORT void string_builder_binary_strcat(struct string_builder *s, char *str, ptrdiff_t len);
 PMOD_EXPORT void string_builder_append(struct string_builder *s,
@@ -274,7 +290,7 @@ PMOD_EXPORT int string_to_svalue_inumber(struct svalue *r,
 			     int maxlength);
 PMOD_EXPORT int wide_string_to_svalue_inumber(struct svalue *r,
 					      void * str,
-					      void **ptr,
+					      void *ptr,
 					      int base,
 					      ptrdiff_t maxlength,
 					      int shift);
