@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: svalue.h,v 1.116 2003/09/09 14:58:54 mast Exp $
+|| $Id: svalue.h,v 1.128 2004/04/03 18:24:39 mast Exp $
 */
 
 #ifndef SVALUE_H
@@ -45,6 +45,12 @@ struct pike_string;
 struct callable;
 #endif
 
+#ifndef STRUCT_NODE_S_DECLARED
+#define STRUCT_NODE_S_DECLARED
+struct node_s;
+typedef struct node_s node;
+#endif
+
 struct processing
 {
   struct processing *next;
@@ -67,8 +73,8 @@ union anything
   INT32 *refs;
   struct ref_dummy *dummy;
   FLOAT_TYPE float_number;
-  struct svalue *lval;   /* only used on stack */
-  union anything *short_lval;   /* only used on stack */
+  int identifier;		/* Used with T_OBJ_INDEX. */
+  struct svalue *lval;		/* Used with T_SVALUE_PTR. */
   void *ptr;
 };
 
@@ -98,8 +104,12 @@ struct svalue
 #define PIKE_T_FLOAT 9
 
 #define PIKE_T_ZERO  14	/* Can return 0, but nothing else */
-#define T_UNFINISHED 15	/* Reserved for the garbage-collector */
-#define T_VOID       16	/* Can't return any value */
+#define T_UNFINISHED 15
+
+#define T_VOID       16
+/* Can't return any value. Also used on stack to fill out the second
+ * svalue on an lvalue when it isn't used. */
+
 #define T_MANY       17
 
 #define PIKE_T_RING 240
@@ -109,8 +119,15 @@ struct svalue
 #define T_ASSIGN 245
 #define T_DELETED 246
 #define PIKE_T_UNKNOWN 247
-#define T_SHORT_LVALUE 248
-#define T_LVALUE 249
+
+#define T_OBJ_INDEX 248
+/* svalue.u.identifer is an identifier index in an object. Primarily
+ * used in lvalues on stack. */
+
+#define T_SVALUE_PTR 249
+/* svalue.u.lval points to an svalue. Primarily used in lvalues on
+ * stack. */
+
 #define T_ARRAY_LVALUE 250
 #define PIKE_T_MIXED 251
 #define T_NOT 253
@@ -122,6 +139,7 @@ struct svalue
 #define T_MAPPING_DATA 10001
 #define T_PIKE_FRAME 10002
 #define T_MULTISET_DATA 10003
+#define T_STRUCT_CALLABLE 10004
 
 #define tArr(VAL) "\000" VAL
 #define tArray tArr(tMix)
@@ -163,6 +181,7 @@ struct svalue
 #define tIntPos "\010\000\000\000\000\177\377\377\377"
 #define tInt1Plus "\010\000\000\000\001\177\377\377\377"
 #define tInt2Plus "\010\000\000\000\002\177\377\377\377"
+#define tInt_10 "\010\377\377\377\377\000\000\000\000"
 #define tByte "\010\000\000\000\000\000\000\000\377"
 #define tFlt "\011"
 #define tFloat "\011"
@@ -216,8 +235,8 @@ struct svalue
 
 #define BIT_ZERO (1<<PIKE_T_ZERO)
 
-/* Used to signify that this array might not be finished yet */
-/* garbage collect uses this */
+/* Used to signify that the type field hasn't been set according to
+ * reality. */
 #define BIT_UNFINISHED (1 << T_UNFINISHED)
 
 /* This is only used in typechecking to signify that this 
@@ -235,6 +254,7 @@ struct svalue
 #define BIT_BASIC (BIT_INT|BIT_FLOAT|BIT_STRING|BIT_TYPE)
 #define BIT_COMPLEX (BIT_ARRAY|BIT_MULTISET|BIT_OBJECT|BIT_PROGRAM|BIT_MAPPING|BIT_FUNCTION)
 #define BIT_CALLABLE (BIT_FUNCTION|BIT_PROGRAM|BIT_ARRAY|BIT_OBJECT)
+#define BIT_REF_TYPES (BIT_STRING|BIT_TYPE|BIT_COMPLEX)
 
 /* Max type which contains svalues */
 #define MAX_COMPLEX PIKE_T_PROGRAM
@@ -243,10 +263,9 @@ struct svalue
 /* Max type handled by svalue primitives */
 #define MAX_TYPE PIKE_T_FLOAT
 
-#define NUMBER_NUMBER     0
-#define NUMBER_UNDEFINED  1
+#define NUMBER_NUMBER 0
+#define NUMBER_UNDEFINED 1
 #define NUMBER_DESTRUCTED 2
-#define NUMBER_LOCKED     USHRT_MAX	/* NOTE: Kludge, only in Pike 7.4. */
 
 #define FUNCTION_BUILTIN USHRT_MAX
 
@@ -318,24 +337,46 @@ do{ \
 #ifdef PIKE_DEBUG
 PMOD_EXPORT extern void describe(void *); /* defined in gc.c */
 PMOD_EXPORT extern const char msg_type_error[];
-#define check_type(T) if(T > MAX_TYPE && T!=T_LVALUE && T!=T_SHORT_LVALUE && T!=T_VOID && T!=T_DELETED && T!=T_ARRAY_LVALUE) Pike_fatal(msg_type_error,T)
+#define check_type(T) if(T > MAX_TYPE && T!=T_SVALUE_PTR && T!=T_OBJ_INDEX && T!=T_VOID && T!=T_DELETED && T!=T_ARRAY_LVALUE) Pike_fatal(msg_type_error,T)
 
 #define check_svalue(S) debug_check_svalue(dmalloc_check_svalue(S,DMALLOC_LOCATION()))
 
+void low_thorough_check_short_svalue (const union anything *u, TYPE_T type);
+#define thorough_check_short_svalue(U, T) do {				\
+    union anything *anyth_ = (U);					\
+    TYPE_T typ_ = (T);							\
+    check_short_svalue (anyth_, typ_);					\
+    if (d_flag <= 50) /* Done directly by check_svalue otherwise. */	\
+      if (typ_ <= MAX_REF_TYPE)						\
+	low_thorough_check_short_svalue (anyth_, typ_);			\
+  } while (0)
+#define thorough_check_svalue(S) do {					\
+    struct svalue *sval_ = (S);						\
+    check_svalue (sval_);						\
+    if (d_flag <= 50) /* Done directly by check_svalue otherwise. */	\
+      if (sval_->type <= MAX_REF_TYPE)					\
+	low_thorough_check_short_svalue (&sval_->u, sval_->type);	\
+  } while (0)
+
 PMOD_EXPORT extern const char msg_sval_obj_wo_refs[];
 #define check_refs(S) do {\
  if((S)->type <= MAX_REF_TYPE && (!(S)->u.refs || (S)->u.refs[0] < 0)) { \
- describe((S)->u.refs); \
- Pike_fatal(msg_sval_obj_wo_refs); \
-} }while(0)
+   fprintf (stderr, msg_sval_obj_wo_refs);				\
+   describe((S)->u.refs);						\
+   Pike_fatal(msg_sval_obj_wo_refs);					\
+ } }while(0)
 
 PMOD_EXPORT extern const char msg_ssval_obj_wo_refs[];
 #define check_refs2(S,T) do { \
 if((T) <= MAX_REF_TYPE && (S)->refs && (S)->refs[0] <= 0) {\
- describe((S)->refs); \
- Pike_fatal(msg_ssval_obj_wo_refs); \
+  fprintf (stderr, msg_ssval_obj_wo_refs);		   \
+  describe((S)->refs);					   \
+  Pike_fatal(msg_ssval_obj_wo_refs);			   \
 } }while(0)
 
+#define check_type_hint(SVALS, NUM, TYPE_HINT)				\
+  debug_check_type_hint ((SVALS), (NUM), (TYPE_HINT))
+
 #ifdef DEBUG_MALLOC
 static inline struct svalue *dmalloc_check_svalue(struct svalue *s, char *l)
 {
@@ -361,11 +402,11 @@ static inline union anything *dmalloc_check_union(union anything *u,int type, ch
 #undef sub_ref
 
 #ifdef PIKE_RUN_UNLOCKED
-#define add_ref(X) pike_atomic_inc32((INT32 *)debug_malloc_pass( &((X)->refs)))
-#define sub_ref(X) pike_atomic_dec_and_test32((INT32 *)debug_malloc_pass( &((X)->refs)))
+#define add_ref(X) pike_atomic_inc32((INT32 *)debug_malloc_update_location( &((X)->refs), DMALLOC_NAMED_LOCATION(" add_ref")))
+#define sub_ref(X) pike_atomic_dec_and_test32((INT32 *)debug_malloc_update_location( &((X)->refs), DMALLOC_NAMED_LOCATION(" sub_ref")))
 #else
-#define add_ref(X) (((INT32 *)debug_malloc_pass( &((X)->refs)))[0]++)
-#define sub_ref(X) (--((INT32 *)debug_malloc_pass( &((X)->refs)))[0] > 0)
+#define add_ref(X) (((INT32 *)debug_malloc_update_location( &((X)->refs), DMALLOC_NAMED_LOCATION(" add_ref")))[0]++)
+#define sub_ref(X) (--((INT32 *)debug_malloc_update_location( &((X)->refs), DMALLOC_NAMED_LOCATION(" sub_ref")))[0] > 0)
 #endif
 
 
@@ -375,16 +416,17 @@ static inline union anything *dmalloc_check_union(union anything *u,int type, ch
 
 #endif
 
-#else
+#else  /* !PIKE_DEBUG */
 
 #define check_svalue(S)
 #define check_type(T)
 #define check_refs(S)
 #define check_refs2(S,T)
+#define check_type_hint(SVALS, NUM, TYPE_HINT)
 #define dmalloc_check_svalue(S,L) (S)
 #define dmalloc_check_union(U,T,L) (U)
 
-#endif
+#endif	/* !PIKE_DEBUG */
 
 
 /* This define
@@ -402,6 +444,8 @@ static inline union anything *dmalloc_check_union(union anything *u,int type, ch
   struct svalue *_b=(Y);				\
   struct svalue _tmp;					\
   assert_svalue_locked(_a); assert_svalue_locked(_b);	\
+  dmalloc_touch_svalue(_a);				\
+  dmalloc_touch_svalue(_b);				\
   _tmp=*_a; *_a=*_b; *_b=_tmp;				\
 }while(0)
 
@@ -410,7 +454,6 @@ static inline union anything *dmalloc_check_union(union anything *u,int type, ch
   assert_svalue_locked(_s);					\
   check_type(_s->type); check_refs(_s);				\
   if(_s->type<=MAX_REF_TYPE) {					\
-    debug_malloc_touch(_s->u.refs);				\
     if(sub_ref(_s->u.dummy) <=0) { really_free_svalue(_s); }	\
   }								\
   DO_IF_DMALLOC(_s->type=PIKE_T_UNKNOWN;_s->u.refs=(void *)-1); \
@@ -471,10 +514,10 @@ extern struct svalue dest_ob_zero;
     dmalloc_touch_svalue(s_);			\
     free_svalue(s_++);				\
   }						\
-}while(0);
+}while(0)
 
 #ifdef DEBUG_MALLOC
-#define free_svalues(X,Y,Z) debug_free_svalues((X),(Y),(Z), DMALLOC_LOCATION());
+#define free_svalues(X,Y,Z) debug_free_svalues((X),(Y),(Z), DMALLOC_NAMED_LOCATION(" free_svalues"));
 #else
 #define free_svalues(X,Y,Z) debug_free_svalues((X),(Y),(Z));
 #endif
@@ -495,6 +538,7 @@ extern struct svalue dest_ob_zero;
 
 #define really_free_short_svalue(U, TYPE) do {				\
     union anything *any_ = (U);						\
+    debug_malloc_touch (any_->ptr);					\
     really_free_short_svalue_ptr (&any_->ptr, (TYPE));			\
   } while (0)
 
@@ -504,14 +548,14 @@ PMOD_EXPORT void really_free_svalue(struct svalue *s);
 PMOD_EXPORT void do_free_svalue(struct svalue *s);
 PMOD_EXPORT void debug_free_svalues(struct svalue *s, size_t num, INT32 type_hint DMALLOC_LINE_ARGS);
 PMOD_EXPORT void debug_free_mixed_svalues(struct svalue *s, size_t num, INT32 type_hint DMALLOC_LINE_ARGS);
-PMOD_EXPORT void assign_svalues_no_free(struct svalue *to,
-			    const struct svalue *from,
-			    size_t num,
-			    TYPE_FIELD type_hint);
-PMOD_EXPORT void assign_svalues(struct svalue *to,
-		    const struct svalue *from,
-		    size_t num,
-		    TYPE_FIELD types);
+PMOD_EXPORT TYPE_FIELD assign_svalues_no_free(struct svalue *to,
+					      const struct svalue *from,
+					      size_t num,
+					      TYPE_FIELD type_hint);
+PMOD_EXPORT TYPE_FIELD assign_svalues(struct svalue *to,
+				      const struct svalue *from,
+				      size_t num,
+				      TYPE_FIELD type_hint);
 PMOD_EXPORT void assign_to_short_svalue(union anything *u,
 			    TYPE_T type,
 			    const struct svalue *s);
@@ -543,14 +587,18 @@ PMOD_EXPORT int is_equal(const struct svalue *a, const struct svalue *b);
 PMOD_EXPORT int is_lt(const struct svalue *a, const struct svalue *b);
 PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct processing *p);
 PMOD_EXPORT void print_svalue (FILE *out, const struct svalue *s);
+PMOD_EXPORT void print_short_svalue (FILE *out, const union anything *a, TYPE_T type);
 PMOD_EXPORT void print_svalue_compact (FILE *out, const struct svalue *s);
+PMOD_EXPORT void print_short_svalue_compact (FILE *out, const union anything *a, TYPE_T type);
 PMOD_EXPORT void copy_svalues_recursively_no_free(struct svalue *to,
 				      const struct svalue *from,
 				      size_t num,
 				      struct processing *p);
 void check_short_svalue(const union anything *u, TYPE_T type);
 void debug_check_svalue(const struct svalue *s);
-PMOD_EXPORT void real_gc_xmark_svalues(const struct svalue *s, ptrdiff_t num);
+void debug_check_type_hint (const struct svalue *svals, size_t num, TYPE_FIELD type_hint);
+PMOD_EXPORT void real_gc_mark_external_svalues(const struct svalue *s, ptrdiff_t num,
+					       const char *place);
 PMOD_EXPORT void real_gc_check_svalues(const struct svalue *s, size_t num);
 void gc_check_weak_svalues(const struct svalue *s, size_t num);
 PMOD_EXPORT void real_gc_check_short_svalue(const union anything *u, TYPE_T type);
@@ -577,7 +625,8 @@ int svalues_are_constant(struct svalue *s,
 #define gc_cycle_check_without_recurse gc_mark_without_recurse
 #define gc_cycle_check_weak_without_recurse gc_mark_without_recurse
 
-#define gc_xmark_svalues(S,N) real_gc_xmark_svalues(dmalloc_check_svalue(S,DMALLOC_LOCATION()),N)
+#define gc_mark_external_svalues(S, NUM, PLACE)				\
+  real_gc_mark_external_svalues (dmalloc_check_svalue (S, DMALLOC_LOCATION()), NUM, PLACE)
 #define gc_check_svalues(S,N) real_gc_check_svalues(dmalloc_check_svalue(S,DMALLOC_LOCATION()),N)
 #define gc_check_short_svalue(U,T) real_gc_check_short_svalue(dmalloc_check_union((U),(T),DMALLOC_LOCATION()),T)
 #define gc_mark_svalues(S,N) real_gc_mark_svalues(dmalloc_check_svalue((S),DMALLOC_LOCATION()),N)
@@ -644,7 +693,7 @@ static inline void free_svalue(struct svalue *s)
 #endif
 
 #ifndef free_short_svalue
-static inline void free_short_svalue(union anything *s, INT16 t)
+static inline void free_short_svalue(union anything *s, int t)
 {
   if(t <= MAX_REF_TYPE)
   {
@@ -747,7 +796,8 @@ static inline void assign_svalue(struct svalue *to, const struct svalue *from)
 
 #define INIT_PIKE_MEMOBJ(X) do {			\
   struct ref_dummy *v_=(struct ref_dummy *)(X);		\
-  v_->refs=1;						\
+  v_->refs=0;						\
+  add_ref(v_); /* For DMALLOC... */			\
   DO_IF_SECURITY( INITIALIZE_PROT(v_) );		\
   IF_LOCAL_MUTEX(mt_init_recursive(&(v_->mutex)));	\
 }while(0)
